Package: ggforestplotR
Title: Publication-Ready Forest Plots with 'ggplot2'
Version: 0.3.1
Authors@R: 
    person(given = "Carson",
           family = "Richardson",
           role = c("aut", "cre", "cph"),
           email = "carson.richardson@outlook.com")
Description: Transform model coefficients into flexible forest
    plots using 'ggplot2'. Provides helpers to standardize
    coefficient data from a range of modelling workflows and render
    publication-ready forest plots with a consistent interface.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: ggplot2, patchwork, rlang
Suggests: broom, broom.mixed, dplyr, knitr, lme4, rmarkdown, survival,
        testthat (>= 3.0.0), tibble
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/Needs/website: pkgdown
URL: https://thatoneguy006.github.io/ggforestplotR/,
        https://github.com/thatoneguy006/ggforestplotR
BugReports: https://github.com/thatoneguy006/ggforestplotR/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-04 20:06:42 UTC; Carso
Author: Carson Richardson [aut, cre, cph]
Maintainer: Carson Richardson <carson.richardson@outlook.com>
Repository: CRAN
Date/Publication: 2026-08-04 22:10:14 UTC
Built: R 4.5.2; ; 2026-08-04 23:13:24 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(ggplot_add,ScaleContinuousPosition)
S3method(ggplot_add,ScaleDiscretePosition)
S3method(ggplot_add,ggforestplot_split_table_adder)
S3method(ggplot_add,ggforestplot_table_adder)
export(add_forest_table)
export(add_split_table)
export(as_forest_data)
export(bind_forest_models)
export(ggforestplot)
export(tidy_forest_model)
importFrom(ggplot2,ggplot_add)
importFrom(rlang,.data)

# ggforestplotR <img src="man/figures/ggforestplotR_logo.png" align="right" height="160" alt="" />
___
<!-- badges: start -->
  [![R-CMD-check](https://github.com/thatoneguy006/ggforestplotR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/thatoneguy006/ggforestplotR/actions/workflows/R-CMD-check.yaml)
  [![Codecov](https://codecov.io/gh/thatoneguy006/ggforestplotR/branch/dev/graph/badge.svg)](https://app.codecov.io/gh/thatoneguy006/ggforestplotR)
  [![CRAN version](https://www.r-pkg.org/badges/version/ggforestplotR)](https://CRAN.R-project.org/package=ggforestplotR)
<!-- badges: end -->

## Overview
`ggforestplotR` provides a `ggplot2`-first workflow for building forest plots
from tidy coefficient tables or fitted model objects.

## Installation

### CRAN
```r
install.packages("ggforestplotR")
```

### Development

```r
#install.packages("remotes")
remotes::install_github("thatoneguy006/ggforestplotR")
```

## Supported workflows

`ggforestplotR` currently supports two core workflows:

- Plot directly from a table of coefficient data.
- Plot using data from a fitted model object.

## Basic example

```r
library(ggforestplotR)
library(ggplot2)

sectioned_coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III", "Nodes"),
  estimate = c(0.10, -0.08, 0.20, 0.34, 0.52, 0.28),
  conf.low = c(0.02, -0.16, 0.05, 0.12, 0.20, 0.06),
  conf.high = c(0.18, 0.00, 0.35, 0.56, 0.84, 0.50),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor", "Tumor")
)

ggforestplot(
  sectioned_coefs,
  facet = "section",
  striped_rows = TRUE,
  stripe_fill = "grey94",
  facet_strip_position = "right"
)
```

![](man/figures/README-forestplot-example.png)

## Add a summary table
```r
ggforestplot(
  sectioned_coefs,
  striped_rows = TRUE,
  stripe_fill = "grey94"
) +
  add_forest_table()
```

![](man/figures/README-forestplot-table-example.png)

## Add a split summary table
```r
ggforestplot(
  sectioned_coefs,
  striped_rows = TRUE,
  stripe_fill = "grey94"
) +
  add_split_table()
```

![](man/figures/README-forestplot-split-table-example.png)

## Learn more

- Get started: <https://thatoneguy006.github.io/ggforestplotR/articles/ggforestplotR-get-started.html>
- Plot & Table customization: <https://thatoneguy006.github.io/ggforestplotR/articles/ggforestplotR-plot-customization.html>
- Data helpers: <https://thatoneguy006.github.io/ggforestplotR/articles/ggforestplotR-data-helpers.html>

## Main functions

- `ggforestplot()` builds the plotting panel from a data frame or supported model object.
- `add_forest_table()` attaches a summary table to the left or right side of the plot.
- `add_split_table()` creates a more traditional forestplot layout with table columns on both sides of the plot.
- `bind_forest_models()` binds output from several models for grouped plotting.
- `as_forest_data()` standardizes custom coefficient data.
- `tidy_forest_model()` converts fitted models into plotting-ready coefficient data.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8.5,
  fig.height = 5.5
)

## ----setup--------------------------------------------------------------------
library(ggforestplotR)
library(ggplot2)

## ----facet-right--------------------------------------------------------------
coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III"),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18),
  conf.high = c(0.21, 0.02, 0.32, 0.50, 0.74),
  sample_size = c(120, 115, 98, 87, 83),
  p_value = c(0.04, 0.15, 0.29, 0.001, 0.075),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor")
)


ggforestplot(
  coefs,
  facet = "section",
  facet_strip_position = "right",
  striped_rows = TRUE
)

## ----separators---------------------------------------------------------------
block_coefs <- data.frame(
  term = c("race_black", "race_white", "race_other", "age", "bmi"),
  label = c("Black", "White", "Other", "Age", "BMI"),
  estimate = c(0.24, 0.08, -0.04, 0.12, -0.09),
  conf.low = c(0.10, -0.04, -0.18, 0.03, -0.17),
  conf.high = c(0.38, 0.20, 0.10, 0.21, -0.01),
  variable_block = c("Race", "Race", "Race", "Age", "BMI")
)

ggforestplot(
  block_coefs,
  label = "label",
  separate_groups = "variable_block",
  separate_lines = TRUE,
  striped_rows = TRUE
) +
  scale_y_discrete(limits = rev(c("BMI", "Age", "Race: White", 
                                  "Race: Black", "Race: Other")))

## ----left-side-table----------------------------------------------------------
ggforestplot(
  coefs,
  facet = "section",
  facet_strip_position = "right",
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE,
  term_labels = c("Smoking" = "Smoking status")
) +
  add_forest_table(
    columns = c("term", "sample_size", "estimate", "p_value"),
    column_labels = c("term" = "Variable", "sample_size" = "N",
                      "estimate" = "Beta (95% CI)", "p_value" = "P-value")
  )

## -----------------------------------------------------------------------------
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  add_forest_table(
    position = "left",
    grid_lines = T,
    grid_line_linetype = 2,
    grid_line_colour = "red"
  )

## ----split-table--------------------------------------------------------------
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  scale_x_continuous(limits = c(-.8,.8)) +
  add_split_table(
    left_columns = c("term","n"),
    right_columns = c("estimate","p"),
    column_labels = c("estimate" = "Beta [95% CI]"),
    estimate_fmt = "{estimate} [{conf.low}, {conf.high}]",
    estimate_digits = 2,
    interval_digits = 3
  ) 

## ----logistic-regression-data-------------------------------------------------
data(CO2)

l1 <- glm(Treatment ~ conc + uptake + Type, family = binomial(link = "logit"), 
    data = CO2)

## ----logistic-regression, warning=FALSE---------------------------------------

ggforestplot(l1, exponentiate = TRUE, striped_rows = T, term_labels = c("TypeMississippi" = "Mississippi")) +
  add_forest_table(position = "left", 
                   show_p = F)

## ----survival-analysis-data---------------------------------------------------
lung <- survival::lung

lung <- lung |>  
  dplyr::mutate(
    status = dplyr::recode(status, `1` = 0, `2` = 1)
  )

s1 <- survival::coxph(Surv(time, status) ~ sex + age + ph.karno + pat.karno, data = lung)

## ----survival-analysis-plot---------------------------------------------------
ggforestplot(s1, exponentiate = T, striped_rows = T) +
  add_forest_table()

## ----comparison---------------------------------------------------------------
comparison_coefs <- data.frame(
  term = rep(c("Age", "BMI", "Smoking", "Stage II", "Stage III"), 2),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46, 0.08, -0.05, 0.24, 0.40, 0.58),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18, 0.00, -0.13, 0.10, 0.20, 0.30),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74, 0.16, 0.03, 0.38, 0.60, 0.86),
  model = rep(c("Model A", "Model B"), each = 5),
  section = rep(c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor"), 2)
)

ggforestplot(
  comparison_coefs,
  group = "model",
  facet = "section",
  striped_rows = TRUE,
  dodge_width = 0.5,
  facet_strip_position = "right"
) +
  theme(legend.position = "top") +
  scale_color_manual(values = c("#1F968BFF", "#453781FF")) +
  add_forest_table()

.compose_forest_table <- function(plot,
                                  position = c("left", "right"),
                                  show_terms = TRUE,
                                  show_n = NULL,
                                  show_events = NULL,
                                  show_estimate = TRUE,
                                  show_p = FALSE,
                                  columns = NULL,
                                  term_header = "Term",
                                  n_header = "N",
                                  events_header = "Events",
                                  estimate_label = "Estimate",
                                  p_header = "P-value",
                                  digits = NULL,
                                  text_size = NULL,
                                  header_text_size = NULL,
                                  header_fontface = "bold",
                                  header_family = NULL,
                                  striped_rows = NULL,
                                  stripe_fill = NULL,
                                  stripe_colour = NULL,
                                  grid_lines = FALSE,
                                  grid_line_colour = "black",
                                  grid_line_size = 0.3,
                                  grid_line_linetype = 1) {
  position <- match.arg(position)

  if (!inherits(plot, "ggplot")) {
    stop("`plot` must be a ggplot object created by `ggforestplot()`.", call. = FALSE)
  }

  state <- plot$ggforestplotR_state

  if (is.null(state)) {
    stop("`plot` must be created by `ggforestplot()` before calling `add_forest_table()`.", call. = FALSE)
  }

  if (is.null(show_n)) {
    show_n <- any(!is.na(state$forest_data$n) & nzchar(state$forest_data$n))
  }

  if (is.null(show_events)) {
    show_events <- any(!is.na(state$forest_data$events) & nzchar(state$forest_data$events))
  }

  if (is.null(digits)) {
    digits <- 2
  }

  if (is.null(text_size)) {
    text_size <- 3.2
  }

  if (is.null(header_text_size)) {
    header_text_size <- 11
  }

  if (is.null(striped_rows)) {
    striped_rows <- isTRUE(state$defaults$striped_rows)
  }

  if (is.null(stripe_fill)) {
    stripe_fill <- state$defaults$stripe_fill
  }

  if (is.null(stripe_colour)) {
    stripe_colour <- state$defaults$stripe_colour
  }

  if (isTRUE(show_n) && all(is.na(state$forest_data$n) | !nzchar(state$forest_data$n))) {
    stop("`show_n = TRUE` requires an `n` column in the underlying forest data.", call. = FALSE)
  }

  if (isTRUE(show_events) && all(is.na(state$forest_data$events) | !nzchar(state$forest_data$events))) {
    stop("`show_events = TRUE` requires an `events` column in the underlying forest data.", call. = FALSE)
  }

  if (isTRUE(show_p) && all(is.na(state$forest_data$p.value))) {
    stop("`show_p = TRUE` requires a `p.value` column in the underlying forest data.", call. = FALSE)
  }

  plot_out <- plot

  if (isTRUE(show_terms)) {
    plot_out <- plot_out + ggplot2::theme(
      axis.text.y = ggplot2::element_blank(),
      axis.ticks.y = ggplot2::element_blank()
    )
  }

  table_spec <- build_forest_table_data(
    state$forest_data,
    show_terms = show_terms,
    show_n = show_n,
    show_events = show_events,
    show_estimate = show_estimate,
    show_p = show_p,
    term_header = term_header,
    n_header = n_header,
    events_header = events_header,
    estimate_label = estimate_label,
    p_header = p_header,
    digits = digits,
    columns = columns
  )
  table_spec <- layout_center_table_spec(
    table_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = if (is.null(header_family)) "" else header_family
  )
  table_width <- max(2.4, table_spec$content_width + 0.15)

  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = position,
    striped_rows = striped_rows,
    stripe_fill = stripe_fill,
    stripe_colour = stripe_colour,
    text_size = text_size,
    grid_lines = grid_lines,
    grid_line_colour = grid_line_colour,
    grid_line_size = grid_line_size,
    grid_line_linetype = grid_line_linetype,
    x_expand = ggplot2::expansion(mult = 0.08),
    text_hjust = 0.5,
    header_hjust = 0.5,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family
  )

  combine_forest_plot_and_table(
    plot = plot_out,
    table_plot = table_plot,
    table_position = position,
    table_width = table_width
  )
}

#' Add a summary table to a forest plot
#'
#' Compose a summary table onto a forest plot.
#'
#' @param plot A plot created by [ggforestplot()]. Leave as `NULL` to use
#'   `+ add_forest_table(...)` syntax.
#' @param position Whether to place the table on the left or right of the
#'   forest plot.
#' @param show_terms Whether to show the term column in the table.
#' @param show_n Whether to show the `N` column. Defaults to `TRUE` when the
#'   underlying plot data includes an `n` column.
#' @param show_events Whether to show the `Events` column. Defaults to `TRUE`
#'   when the underlying plot data includes an `events` column.
#' @param show_estimate Whether to show the formatted estimate and confidence
#'   interval column.
#' @param show_p Whether to display the p-value column.
#' @param columns Optional explicit columns to display in the side table, in
#'   the order they should appear. Accepts names such as `"n"`, `"events"`,
#'   and `"term"`, or positions `1:5` corresponding to `term`, `n`, `events`,
#'   `estimate`, and `p`.
#'   When supplied, this overrides the default `show_*` column selection.
#' @param term_header Header text for the term column.
#' @param n_header Header text for the `N` column.
#' @param events_header Header text for the `Events` column.
#' @param estimate_label Header label for the estimate column.
#' @param p_header Header text for the p-value column.
#' @param digits Number of digits used when formatting estimates and p-values.
#'   Defaults to `2`.
#' @param text_size Text size for table contents. Defaults to `3.2`.
#' @param header_text_size Header text size for table column labels. Defaults
#'   to `11`.
#' @param header_fontface Font face used for table column labels. Defaults to
#'   `"bold"`.
#' @param header_family Optional font family used for table column labels.
#' @param striped_rows Whether to draw alternating row stripes behind the
#'   table. Defaults to the stripe setting used in [ggforestplot()].
#' @param stripe_fill Fill colour used for striped rows. Defaults to the
#'   stripe fill used in [ggforestplot()].
#' @param stripe_colour Outline colour for striped rows. Defaults to the
#'   stripe outline used in [ggforestplot()].
#' @param grid_lines Whether to draw black horizontal grid lines in the table.
#' @param grid_line_colour Colour used for the table grid lines.
#' @param grid_line_size Line width used for the table grid lines.
#' @param grid_line_linetype Line type used for the table grid lines.
#'
#' @return A patchwork-composed plot containing the forest plot and side
#'   table, or a ggplot add-on object when `plot = NULL`.
#' @export
#'
#' @examples
#' coefs <- data.frame(
#'   term = c("Age", "BMI", "Treatment"),
#'   estimate = c(0.3, -0.2, 0.4),
#'   conf.low = c(0.1, -0.4, 0.2),
#'   conf.high = c(0.5, 0.0, 0.6),
#'   sample_size = c(120, 115, 98),
#'   p_value = c(0.012, 0.031, 0.004)
#' )
#'
#' p <- ggforestplot(coefs, n = "sample_size", p.value = "p_value")
#' add_forest_table(
#'   p,
#'   position = "left",
#'   show_n = TRUE,
#'   show_p = TRUE,
#'   estimate_label = "Beta"
#' )
#'
#' ggforestplot(coefs, n = "sample_size", p.value = "p_value") +
#'   add_forest_table(
#'     position = "right",
#'     show_n = TRUE,
#'     show_p = TRUE,
#'     estimate_label = "Beta"
#'   )
add_forest_table <- function(plot = NULL,
                             position = c("left", "right"),
                             show_terms = TRUE,
                             show_n = NULL,
                             show_events = NULL,
                             show_estimate = TRUE,
                             show_p = FALSE,
                             columns = NULL,
                             term_header = "Term",
                             n_header = "N",
                             events_header = "Events",
                             estimate_label = "Estimate",
                             p_header = "P-value",
                             digits = NULL,
                             text_size = NULL,
                             header_text_size = NULL,
                             header_fontface = "bold",
                             header_family = NULL,
                             striped_rows = NULL,
                             stripe_fill = NULL,
                             stripe_colour = NULL,
                             grid_lines = FALSE,
                             grid_line_colour = "black",
                             grid_line_size = 0.3,
                             grid_line_linetype = 1) {
  position <- match.arg(position)

  if (is.null(plot)) {
    return(structure(
      list(
        position = position,
        show_terms = show_terms,
        show_n = show_n,
        show_events = show_events,
        show_estimate = show_estimate,
        show_p = show_p,
        columns = columns,
        term_header = term_header,
        n_header = n_header,
        events_header = events_header,
        estimate_label = estimate_label,
        p_header = p_header,
        digits = digits,
        text_size = text_size,
        header_text_size = header_text_size,
        header_fontface = header_fontface,
        header_family = header_family,
        striped_rows = striped_rows,
        stripe_fill = stripe_fill,
        stripe_colour = stripe_colour,
        grid_lines = grid_lines,
        grid_line_colour = grid_line_colour,
        grid_line_size = grid_line_size,
        grid_line_linetype = grid_line_linetype
      ),
      class = "ggforestplot_table_adder"
    ))
  }

  .compose_forest_table(
    plot = plot,
    position = position,
    show_terms = show_terms,
    show_n = show_n,
    show_events = show_events,
    show_estimate = show_estimate,
    show_p = show_p,
    columns = columns,
    term_header = term_header,
    n_header = n_header,
    events_header = events_header,
    estimate_label = estimate_label,
    p_header = p_header,
    digits = digits,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family,
    striped_rows = striped_rows,
    stripe_fill = stripe_fill,
    stripe_colour = stripe_colour,
    grid_lines = grid_lines,
    grid_line_colour = grid_line_colour,
    grid_line_size = grid_line_size,
    grid_line_linetype = grid_line_linetype
  )
}

#' @export
#' @keywords internal
ggplot_add.ggforestplot_table_adder <- function(object, plot, ...) {
  do.call(
    .compose_forest_table,
    c(list(plot = plot), object)
  )
}

make_contract_data <- function() {
  data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.30, -0.20, 0.40),
    conf.low = c(0.10, -0.40, 0.20),
    conf.high = c(0.50, 0.00, 0.60),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )
}

test_that("add_forest_table returns a two-panel patchwork", {
  p <- ggforestplot(make_contract_data(), n = "sample_size", events = "event_count", p.value = "p_value")

  out <- add_forest_table(
    p,
    position = "left",
    show_n = TRUE,
    show_events = TRUE,
    show_p = TRUE,
    estimate_label = "Beta"
  )

  expect_s3_class(out, "patchwork")
  expect_s3_class(out, "ggplot")
  expect_length(out$patches$layout$widths, 2L)
  expect_length(out$patches$plots, 1L)
  expect_s3_class(out$patches$plots[[1]], "ggplot")
})

test_that("add_forest_table supports ggplot add syntax as a terminal step", {
  out <- ggforestplot(make_contract_data(), n = "sample_size", events = "event_count", p.value = "p_value") +
    ggplot2::labs(title = "Contract") +
    add_forest_table(position = "right", show_n = TRUE, show_events = TRUE, show_p = TRUE)

  expect_s3_class(out, "patchwork")
  expect_s3_class(out, "ggplot")
  expect_length(out$patches$layout$widths, 2L)
  expect_length(out$patches$plots, 1L)
})

test_that("add_split_table returns left plot right panels in order", {
  p <- ggforestplot(make_contract_data(), n = "sample_size", events = "event_count", p.value = "p_value")

  out <- add_split_table(
    p,
    left_columns = c("term", "n", "events"),
    right_columns = c("estimate", "p"),
    estimate_label = "HR"
  )

  expect_s3_class(out, "patchwork")
  expect_s3_class(out, "ggplot")
  expect_length(out$patches$layout$widths, 3L)
  expect_length(out$patches$plots, 2L)
  expect_s3_class(out$patches$plots[[2]], "ggplot")
})

test_that("add_split_table ggplot add syntax preserves a forest-plot center panel", {
  out <- ggforestplot(make_contract_data(), n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = c("term", "n", "events"), right_columns = c("estimate", "p"))

  center_plot <- out$patches$plots[[2]]

  expect_s3_class(out, "patchwork")
  expect_s3_class(center_plot, "ggplot")
  expect_true(!is.null(center_plot$ggforestplotR_state))
})

make_logistic_example_data <- function() {
  set.seed(123)

  logit_data <- data.frame(
    age = rnorm(250, mean = 62, sd = 8),
    bmi = rnorm(250, mean = 28, sd = 4),
    treatment = factor(rbinom(250, 1, 0.45), labels = c("Control", "Treatment"))
  )

  linpred <- -9 +
    0.09 * logit_data$age +
    0.11 * logit_data$bmi +
    0.9 * (logit_data$treatment == "Treatment")

  logit_data$event <- rbinom(250, 1, plogis(linpred))
  logit_data
}

test_that("mixed-model tidy output is filtered to fixed effects", {
  mixed_tidy <- data.frame(
    effect = c("fixed", "fixed", "ran_pars"),
    term = c("(Intercept)", "Days", "sd__(Intercept)"),
    estimate = c(251.4, 10.5, 24.7),
    conf.low = c(238.0, 8.9, NA),
    conf.high = c(264.8, 12.1, NA),
    stringsAsFactors = FALSE
  )

  out <- keep_fixed_effects(mixed_tidy)

  expect_equal(out$term, c("(Intercept)", "Days"))
  expect_true(all(as.character(out$effect) == "fixed"))
})

test_that("tidy_forest_model exponentiates logistic regression coefficients", {
  fit <- glm(
    event ~ age + bmi + treatment,
    data = make_logistic_example_data(),
    family = binomial()
  )

  out <- tidy_forest_model(fit, exponentiate = TRUE)

  expect_true(all(c("term", "estimate", "conf.low", "conf.high", "p.value") %in% names(out)))
  expect_true(all(out$estimate > 0))
  expect_true(all(out$conf.low > 0))
  expect_true(all(out$conf.high > 0))
  expect_true(all(!is.na(out$p.value)))
})

test_that("tidy_forest_model supports lme4 mixed models", {
  skip_if_not_installed("broom.mixed")
  skip_if_not_installed("lme4")

  fit <- lme4::lmer(Reaction ~ Days + (Days | Subject), data = lme4::sleepstudy)
  out <- tidy_forest_model(fit)

  expect_equal(out$term, "Days")
  expect_true("effect" %in% names(out))
  expect_true(all(as.character(out$effect) == "fixed"))
  expect_true(all(!is.na(out$estimate)))
  expect_true(all(!is.na(out$conf.low)))
  expect_true(all(!is.na(out$conf.high)))

  p <- ggforestplot(fit)

  expect_equal(p$ggforestplotR_state$forest_data$term, "Days")
  expect_s3_class(p, "ggplot")
})

test_that("ggforestplot labels logistic mixed models as odds ratios", {
  skip_if_not_installed("broom.mixed")
  skip_if_not_installed("lme4")

  fit <- lme4::glmer(
    cbind(incidence, size - incidence) ~ period + (1 | herd),
    data = lme4::cbpp,
    family = binomial()
  )

  p <- ggforestplot(fit)

  expect_equal(p$labels$x, "OR (95% CI)")
  expect_equal(p$ggforestplotR_state$defaults$estimate_label, "OR")
  expect_true(all(p$ggforestplotR_state$forest_data$estimate > 0))
  expect_true(all(as.character(p$ggforestplotR_state$forest_data$effect) == "fixed"))
})

test_that("ggforestplot supports exponentiated logistic regression models", {
  fit <- glm(
    event ~ age + bmi + treatment,
    data = make_logistic_example_data(),
    family = binomial()
  )

  p <- ggforestplot(fit, exponentiate = TRUE) +
    ggplot2::labs(x = "Odds ratio")

  expect_equal(p$labels$x, "Odds ratio")
  expect_equal(p$scales$get_scales("x")$trans$name, "log-10")
  expect_true(all(p$ggforestplotR_state$forest_data$estimate > 0))
  expect_true(all(p$ggforestplotR_state$forest_data$conf.low > 0))
  expect_true(all(p$ggforestplotR_state$forest_data$conf.high > 0))
})

test_that("ggforestplot uses canonical labels for model objects", {
  fit <- glm(
    event ~ age + bmi + treatment,
    data = make_logistic_example_data(),
    family = binomial()
  )

  p <- ggforestplot(fit)

  expect_equal(p$labels$x, "OR (95% CI)")
  expect_equal(p$ggforestplotR_state$defaults$estimate_label, "OR")
  expect_true(all(p$ggforestplotR_state$forest_data$estimate > 0))
})

test_that("ggforestplot labels Cox models as hazard ratios", {
  skip_if_not_installed("survival")

  fit <- survival::coxph(
    survival::Surv(time, status) ~ age + sex,
    data = survival::lung
  )

  p <- ggforestplot(fit)

  expect_equal(p$labels$x, "HR (95% CI)")
  expect_equal(p$ggforestplotR_state$defaults$estimate_label, "HR")
  expect_equal(p$ggforestplotR_state$defaults$ref_line, 1)
  expect_true(all(p$ggforestplotR_state$forest_data$estimate > 0))
})

test_that("add_forest_table works for exponentiated logistic regression output", {
  fit <- glm(
    event ~ age + bmi + treatment,
    data = make_logistic_example_data(),
    family = binomial()
  )

  out <- ggforestplot(fit, exponentiate = TRUE) +
    ggplot2::labs(x = "Odds ratio") +
    add_forest_table(position = "right", estimate_label = "OR", columns = c("term", "estimate", "p"))

  expect_s3_class(out, "patchwork")
})

test_that("ggforestplot defaults staple width to 0.2", {
  raw <- make_simple_forest_data()

  built <- ggplot2::ggplot_build(ggforestplot(raw))

  expect_true(all(built$data[[1]]$width == 0.2))
})

test_that("ggforestplot can facet grouped rows and add stripes", {
  raw <- data.frame(
    term = c("Age", "BMI", "Smoking", "Stage II", "Stage III", "Nodes"),
    estimate = c(0.3, -0.2, 0.4, 0.5, 0.8, 0.4),
    conf.low = c(0.1, -0.4, 0.2, 0.2, 0.4, 0.1),
    conf.high = c(0.5, 0.0, 0.6, 0.8, 1.2, 0.7),
    section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor", "Tumor")
  )

  p <- ggforestplot(raw, facet = "section", striped_rows = TRUE, stripe_alpha = 0.35)
  built <- ggplot2::ggplot_build(p)
  panel_rows <- lapply(split(as.numeric(built$data[[2]]$y), built$data[[2]]$PANEL), unique)

  expect_equal(nrow(built$data[[1]]), 4L)
  expect_true(all(built$data[[1]]$alpha == 0.35))
  expect_equal(p$ggforestplotR_state$defaults$stripe_alpha, 0.35)
  expect_equal(length(panel_rows), 2L)
  expect_equal(unname(panel_rows[[1]]), c(1, 2, 3))
  expect_equal(unname(panel_rows[[2]]), c(1, 2, 3))
})

test_that("ggforestplot respects factor level order for facets", {
  raw <- data.frame(
    term = c("Age", "BMI", "Stage II", "Stage III"),
    estimate = c(0.3, -0.2, 0.5, 0.8),
    conf.low = c(0.1, -0.4, 0.2, 0.4),
    conf.high = c(0.5, 0.0, 0.8, 1.2),
    section = factor(
      c("Clinical", "Clinical", "Tumor", "Tumor"),
      levels = c("Tumor", "Clinical")
    )
  )

  p <- ggforestplot(raw, facet = "section")
  built <- ggplot2::ggplot_build(p)
  panel_order <- as.character(built$layout$layout$grouping_panel)
  out <- p + add_forest_table()
  table_plot <- out$patches$plots[[1]]

  expect_equal(panel_order, c("Tumor", "Clinical"))
  expect_equal(levels(p$ggforestplotR_state$forest_data$grouping_panel), c("Tumor", "Clinical"))
  expect_equal(levels(p$ggforestplotR_state$stripe_data$grouping_panel), c("Tumor", "Clinical"))
  expect_equal(levels(table_plot$data$grouping_panel), c("Tumor", "Clinical"))
})

test_that("faceted ggforestplot supports visible labels in scale_y_discrete limits", {
  raw <- data.frame(
    term = c("Age", "BMI", "Smoking", "Stage II", "Stage III", "Nodes"),
    estimate = c(0.3, -0.2, 0.4, 0.5, 0.8, 0.4),
    conf.low = c(0.1, -0.4, 0.2, 0.2, 0.4, 0.1),
    conf.high = c(0.5, 0.0, 0.6, 0.8, 1.2, 0.7),
    section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor", "Tumor")
  )

  p <- suppressMessages(
    ggforestplot(raw, facet = "section") +
      ggplot2::scale_y_discrete(limits = c("Smoking", "Age", "Stage II"))
  )
  built <- ggplot2::ggplot_build(p)
  panel_limits <- lapply(built$layout$panel_params, function(panel) panel$y$get_limits())
  aligned_state <- align_forest_state_to_plot_y_scale(p$ggforestplotR_state, p)
  out <- p + add_forest_table()
  table_plot <- out$patches$plots[[1]]

  expect_equal(sum(!is.na(built$data[[1]]$y)), 3L)
  expect_equal(sum(!is.na(built$data[[2]]$y)), 3L)
  expect_equal(panel_limits[[1]], c("Smoking", "Age"))
  expect_equal(panel_limits[[2]], "Stage II")
  expect_equal(
    levels(aligned_state$forest_data$row_key),
    c("Smoking", "Age", "Stage II")
  )
  expect_equal(levels(table_plot$data$row_key), c("Smoking", "Age", "Stage II"))
})

test_that("ggforestplot supports point and interval geometry controls", {
  raw <- make_simple_forest_data()

  built <- ggplot2::ggplot_build(
    ggforestplot(raw, point_shape = 17, linewidth = 0.8, staple_width = 0.25)
  )

  expect_true(all(built$data[[2]]$shape == 17))
  expect_true(all(built$data[[1]]$linewidth == 0.8))
  expect_true(all(built$data[[1]]$width == 0.25))
})

test_that("ggforestplot truncates confidence intervals and draws arrows", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(-2.0, -0.4, 0.2),
    conf.high = c(0.5, 2.0, 1.8)
  )

  p <- ggforestplot(raw, ci_limits = c(-1, 1), ci_arrow_length = 0.05)
  built <- ggplot2::ggplot_build(p)
  errorbar_layers <- Filter(
    function(layer) all(c("xmin", "xmax", "width") %in% names(layer)),
    built$data
  )
  segment_layers <- Filter(
    function(layer) all(c("x", "xend", "y", "yend") %in% names(layer)),
    built$data
  )
  staple_layers <- Filter(
    function(layer) all(layer$width == 0.2) && all(layer$xmin == layer$xmax),
    errorbar_layers
  )
  staple_positions <- unlist(lapply(staple_layers, function(layer) layer$xmin), use.names = FALSE)

  expect_equal(p$scales$get_scales("x")$limits, c(-1, 1))
  expect_equal(built$data[[1]]$xmin, c(-1, -0.4, 0.2))
  expect_equal(built$data[[1]]$xmax, c(0.5, 1, 1))
  expect_true(all(built$data[[1]]$width == 0))
  expect_equal(sort(staple_positions), c(-0.4, 0.2, 0.5))
  expect_false(any(staple_positions %in% c(-1, 1)))
  expect_length(segment_layers, 2L)
  expect_true(any(segment_layers[[1]]$xend == -1))
  expect_true(all(segment_layers[[2]]$xend == 1))
  expect_equal(p$ggforestplotR_state$forest_data$conf.low, raw$conf.low)
  expect_equal(p$ggforestplotR_state$forest_data$conf.high, raw$conf.high)
  expect_equal(p$ggforestplotR_state$defaults$ci_limits, c(-1, 1))
})

test_that("ggforestplot can truncate confidence intervals without arrows", {
  raw <- data.frame(
    term = "Age",
    estimate = 0.3,
    conf.low = -2,
    conf.high = 2
  )

  built <- ggplot2::ggplot_build(
    ggforestplot(raw, ci_limits = c(-1, 1), ci_arrows = FALSE)
  )
  segment_layers <- Filter(
    function(layer) all(c("x", "xend", "y", "yend") %in% names(layer)),
    built$data
  )

  expect_equal(built$data[[1]]$xmin, -1)
  expect_equal(built$data[[1]]$xmax, 1)
  expect_length(segment_layers, 0L)
})

test_that("ggforestplot validates confidence interval truncation limits", {
  raw <- data.frame(
    term = "Age",
    estimate = 1.2,
    conf.low = 0.1,
    conf.high = 3
  )

  expect_error(
    ggforestplot(raw, ci_limits = 1),
    "`ci_limits` must be `NULL` or a numeric vector of length 2."
  )
  expect_error(
    ggforestplot(raw, ci_limits = c(1, 1)),
    "`ci_limits` must contain two distinct values."
  )
  expect_error(
    ggforestplot(raw, exponentiate = TRUE, ci_limits = c(0, 2)),
    "`ci_limits` must be positive for exponentiated plots."
  )
  expect_error(
    ggforestplot(raw, ci_limits = c(0, 2), ci_arrows = NA),
    "`ci_arrows` must be `TRUE` or `FALSE`."
  )
  expect_error(
    ggforestplot(raw, ci_limits = c(0, 2), ci_arrow_length = 0),
    "`ci_arrow_length` must be a single positive number."
  )
})

test_that("ggforestplot can draw separator lines for each labeled variable block", {
  raw <- data.frame(
    term = c("race_black", "race_white", "race_other", "age"),
    label = c("Black", "White", "Other", "Age"),
    estimate = c(0.3, 0.1, 0.05, -0.2),
    conf.low = c(0.1, -0.1, -0.15, -0.4),
    conf.high = c(0.5, 0.3, 0.25, 0.0),
    block = c("Race", "Race", "Race", "Age")
  )

  built <- ggplot2::ggplot_build(
    ggforestplot(
      raw,
      label = "label",
      separate_groups = "block",
      separate_lines = TRUE,
      ref_line = NULL
    )
  )

  line_layers <- Filter(function(x) "yintercept" %in% names(x), built$data)

  expect_length(line_layers, 1L)
  expect_equal(line_layers[[1]]$yintercept, c(0.5, 3.5, 4.5))
  expect_true(all(line_layers[[1]]$linetype == 2))
  expect_true(all(line_layers[[1]]$colour == "black"))
})
test_that("ggforestplot prefixes labels for multi-level separator groups", {
  raw <- data.frame(
    term = c("race_black", "race_white", "race_other", "age"),
    label = c("Black", "White", "Other", "Age"),
    estimate = c(0.3, 0.1, 0.05, -0.2),
    conf.low = c(0.1, -0.1, -0.15, -0.4),
    conf.high = c(0.5, 0.3, 0.25, 0.0),
    block = c("Race", "Race", "Race", "Age")
  )

  p <- ggforestplot(raw, label = "label", separate_groups = "block")

  expect_equal(
    as.character(p$ggforestplotR_state$forest_data$label),
    c("Race: Black", "Race: White", "Race: Other", "Age")
  )
})

.compose_split_table <- function(plot,
                                 show_terms = TRUE,
                                 show_n = NULL,
                                 show_events = NULL,
                                 show_estimate = TRUE,
                                 show_p = FALSE,
                                 left_columns = NULL,
                                  right_columns = NULL,
                                 term_header = "Term",
                                 n_header = "N",
                                 events_header = "Events",
                                 estimate_label = "Estimate",
                                 p_header = "P-value",
                                 digits = NULL,
                                 text_size = NULL,
                                 header_text_size = NULL,
                                 header_fontface = "bold",
                                 header_family = NULL,
                                 striped_rows = NULL,
                                 stripe_fill = NULL,
                                 stripe_colour = NULL,
                                 left_width = NULL,
                                 plot_width = NULL,
                                 right_width = NULL) {
  if (!inherits(plot, "ggplot")) {
    stop("`plot` must be a ggplot object created by `ggforestplot()`.", call. = FALSE)
  }

  state <- plot$ggforestplotR_state

  if (is.null(state)) {
    stop("`plot` must be created by `ggforestplot()` before calling `add_split_table()`.", call. = FALSE)
  }

  if (is.null(show_n)) {
    show_n <- any(!is.na(state$forest_data$n) & nzchar(state$forest_data$n))
  }

  if (is.null(show_events)) {
    show_events <- any(!is.na(state$forest_data$events) & nzchar(state$forest_data$events))
  }

  if (is.null(digits)) {
    digits <- 2
  }

  if (is.null(text_size)) {
    text_size <- 3.2
  }

  if (is.null(header_text_size)) {
    header_text_size <- 11
  }

  if (is.null(striped_rows)) {
    striped_rows <- isTRUE(state$defaults$striped_rows)
  }

  if (is.null(stripe_fill)) {
    stripe_fill <- state$defaults$stripe_fill
  }

  if (is.null(stripe_colour)) {
    stripe_colour <- state$defaults$stripe_colour
  }

  default_left <- c(if (isTRUE(show_terms)) "term", if (isTRUE(show_n)) "n", if (isTRUE(show_events)) "events")
  default_right <- c(if (isTRUE(show_estimate)) "estimate", if (isTRUE(show_p)) "p")
  resolved_left <- if (is.null(left_columns)) default_left else normalize_table_columns(left_columns)
  resolved_right <- if (is.null(right_columns)) default_right else normalize_table_columns(right_columns)

  if (length(resolved_left) == 0L) {
    stop(
      "`add_split_table()` requires at least one left-side column. Supply `left_columns` or enable `show_terms`/`show_n`.",
      call. = FALSE
    )
  }

  if (length(resolved_right) == 0L) {
    stop(
      "`add_split_table()` requires at least one right-side column. Supply `right_columns` or enable `show_estimate`/`show_p`.",
      call. = FALSE
    )
  }

  overlap <- intersect(resolved_left, resolved_right)

  if (length(overlap) > 0L) {
    stop(
      sprintf("Split table columns cannot appear on both sides: %s", paste(overlap, collapse = ", ")),
      call. = FALSE
    )
  }

  if ("n" %in% c(resolved_left, resolved_right) && all(is.na(state$forest_data$n) | !nzchar(state$forest_data$n))) {
    stop("An `n` column is required when split table columns include `n`.", call. = FALSE)
  }

  if ("events" %in% c(resolved_left, resolved_right) &&
      all(is.na(state$forest_data$events) | !nzchar(state$forest_data$events))) {
    stop("An `events` column is required when split table columns include `events`.", call. = FALSE)
  }

  if ("p" %in% c(resolved_left, resolved_right) && all(is.na(state$forest_data$p.value))) {
    stop("A `p.value` column is required when split table columns include `p`.", call. = FALSE)
  }

  left_spec <- build_forest_table_data(
    state$forest_data,
    show_terms = FALSE,
    show_n = FALSE,
    show_estimate = FALSE,
    show_p = FALSE,
    term_header = term_header,
    n_header = n_header,
    events_header = events_header,
    estimate_label = estimate_label,
    p_header = p_header,
    digits = digits,
    columns = resolved_left
  )

  right_spec <- build_forest_table_data(
    state$forest_data,
    show_terms = FALSE,
    show_n = FALSE,
    show_estimate = FALSE,
    show_p = FALSE,
    term_header = term_header,
    n_header = n_header,
    events_header = events_header,
    estimate_label = estimate_label,
    p_header = p_header,
    digits = digits,
    columns = resolved_right
  )

  left_spec <- layout_split_table_spec(
    left_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = if (is.null(header_family)) "" else header_family,
    alignment = "left"
  )
  right_spec <- layout_split_table_spec(
    right_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = if (is.null(header_family)) "" else header_family,
    alignment = "right"
  )

  if (is.null(plot_width)) {
    plot_width <- 2.5
  }

  if (is.null(left_width)) {
    left_width <- plot_width * split_table_width_multiplier(length(left_spec$column_keys))
  }

  if (is.null(right_width)) {
    right_width <- plot_width * split_table_width_multiplier(length(right_spec$column_keys))
  }

  left_plot <- build_forest_table_plot(
    table_spec = left_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "left",
    striped_rows = striped_rows,
    stripe_fill = stripe_fill,
    stripe_colour = stripe_colour,
    text_size = text_size,
    grid_lines = FALSE,
    plot_margin = ggplot2::margin(5.5, 0, 5.5, 5.5),
    text_hjust = 0,
    header_hjust = 0,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family
  )

  right_plot <- build_forest_table_plot(
    table_spec = right_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "right",
    striped_rows = striped_rows,
    stripe_fill = stripe_fill,
    stripe_colour = stripe_colour,
    text_size = text_size,
    grid_lines = FALSE,
    plot_margin = ggplot2::margin(5.5, 5.5, 5.5, 0),
    text_hjust = 1,
    header_hjust = 1,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family
  )

  plot_theme_args <- list(
    axis.text.y = ggplot2::element_blank(),
    axis.ticks.y = ggplot2::element_blank(),
    axis.title.y = ggplot2::element_blank(),
    panel.border = ggplot2::element_blank(),
    axis.line.y = ggplot2::element_blank(),
    panel.grid.major = ggplot2::element_blank(),
    panel.grid.minor = ggplot2::element_blank()
  )

  if (is.null(plot$theme$axis.line.x)) {
    plot_theme_args$axis.line.x <- ggplot2::element_line(colour = "black")
  }

  if (is.null(plot$theme$plot.margin)) {
    plot_theme_args$plot.margin <- ggplot2::margin(5.5, 0, 5.5, 0)
  }

  plot_out <- plot + do.call(ggplot2::theme, plot_theme_args)

  left_spec$content_width <- left_width
  right_spec$content_width <- right_width

  combine_split_forest_plot(
    plot = plot_out,
    left_table = left_plot,
    right_table = right_plot,
    left_spec = left_spec,
    right_spec = right_spec,
    plot_width = plot_width
  )
}

#' Add split tables around a forest plot
#'
#' Compose split table blocks around a forest plot so that summary data appear
#' on both sides of the plotting panel.
#'
#' @param plot A plot created by [ggforestplot()]. Leave as `NULL` to use
#'   `+ add_split_table(...)` syntax.
#' @param show_terms Whether to include the term column in the default
#'   left-side selection when `left_columns` is not supplied.
#' @param show_n Whether to include the `N` column in the default left-side
#'   selection when `left_columns` is not supplied. Defaults to `TRUE` when
#'   the underlying plot data include an `n` column.
#' @param show_events Whether to include the `Events` column in the default
#'   left-side selection when `left_columns` is not supplied. Defaults to
#'   `TRUE` when the underlying plot data include an `events` column.
#' @param show_estimate Whether to include the formatted estimate and
#'   confidence interval column in the default right-side selection when
#'   `right_columns` is not supplied.
#' @param show_p Whether to include the p-value column in the default
#'   right-side selection when `right_columns` is not supplied.
#' @param left_columns Optional explicit columns to place on the left side of
#'   the forest plot. Accepts names such as `"term"`, `"n"`, and `"events"`,
#'   or positions `1:5` corresponding to `term`, `n`, `events`, `estimate`,
#'   and `p`.
#' @param right_columns Optional explicit columns to place on the right side
#'   of the forest plot. Accepts names such as `"estimate"` and `"p"`, or
#'   positions `1:5` corresponding to `term`, `n`, `events`, `estimate`,
#'   and `p`.
#' @param term_header Header text for the term column.
#' @param n_header Header text for the `N` column.
#' @param events_header Header text for the `Events` column.
#' @param estimate_label Header label for the estimate column.
#' @param p_header Header text for the p-value column.
#' @param digits Number of digits used when formatting estimates and p-values.
#'   Defaults to `2`.
#' @param text_size Text size for table contents. Defaults to `3.2`.
#' @param header_text_size Header text size for table column labels. Defaults
#'   to `11`.
#' @param header_fontface Font face used for table column labels. Defaults to
#'   `"bold"`.
#' @param header_family Optional font family used for table column labels.
#' @param striped_rows Whether to draw alternating row stripes behind the
#'   split table layout. Defaults to the stripe setting used in
#'   [ggforestplot()].
#' @param stripe_fill Fill colour used for striped rows. Defaults to the
#'   stripe fill used in [ggforestplot()].
#' @param stripe_colour Outline colour for striped rows. Defaults to the
#'   stripe outline used in [ggforestplot()].
#' @param left_width Optional width allocated to the left table block. By
#'   default this is derived from the number of displayed left-side columns
#'   relative to `plot_width`.
#' @param plot_width Optional width allocated to the forest plot panel.
#'   Defaults to `2.5`.
#' @param right_width Optional width allocated to the right table block. By
#'   default this is derived from the number of displayed right-side columns
#'   relative to `plot_width`.
#'
#' @return A patchwork-composed plot containing a left table, the forest plot,
#'   and a right table, or a ggplot add-on object when `plot = NULL`.
#' @export
#'
#' @examples
#' coefs <- data.frame(
#'   term = c("Age", "BMI", "Treatment"),
#'   estimate = c(0.3, -0.2, 0.4),
#'   conf.low = c(0.1, -0.4, 0.2),
#'   conf.high = c(0.5, 0.0, 0.6),
#'   sample_size = c(120, 115, 98),
#'   p_value = c(0.012, 0.031, 0.004)
#' )
#'
#' p <- ggforestplot(coefs, n = "sample_size", p.value = "p_value")
#' add_split_table(
#'   p,
#'   left_columns = c("term", "n"),
#'   right_columns = c("estimate", "p"),
#'   estimate_label = "HR"
#' )
#'
#' ggforestplot(coefs, n = "sample_size", p.value = "p_value") +
#'   add_split_table(
#'     left_columns = c(1, 2),
#'     right_columns = c(4, 5),
#'     estimate_label = "HR"
#'   )
add_split_table <- function(plot = NULL,
                            show_terms = TRUE,
                            show_n = NULL,
                            show_events = NULL,
                            show_estimate = TRUE,
                            show_p = FALSE,
                            left_columns = NULL,
                            right_columns = NULL,
                            term_header = "Term",
                            n_header = "N",
                            events_header = "Events",
                            estimate_label = "Estimate",
                            p_header = "P-value",
                            digits = NULL,
                            text_size = NULL,
                            header_text_size = NULL,
                            header_fontface = "bold",
                            header_family = NULL,
                            striped_rows = NULL,
                            stripe_fill = NULL,
                            stripe_colour = NULL,
                            left_width = NULL,
                            plot_width = NULL,
                            right_width = NULL) {
  if (is.null(plot)) {
    return(structure(
      list(
        show_terms = show_terms,
        show_n = show_n,
        show_events = show_events,
        show_estimate = show_estimate,
        show_p = show_p,
        left_columns = left_columns,
        right_columns = right_columns,
        term_header = term_header,
        n_header = n_header,
        events_header = events_header,
        estimate_label = estimate_label,
        p_header = p_header,
        digits = digits,
        text_size = text_size,
        header_text_size = header_text_size,
        header_fontface = header_fontface,
        header_family = header_family,
        striped_rows = striped_rows,
        stripe_fill = stripe_fill,
        stripe_colour = stripe_colour,
        left_width = left_width,
        plot_width = plot_width,
        right_width = right_width
      ),
      class = "ggforestplot_split_table_adder"
    ))
  }

  .compose_split_table(
    plot = plot,
    show_terms = show_terms,
    show_n = show_n,
    show_events = show_events,
    show_estimate = show_estimate,
    show_p = show_p,
    left_columns = left_columns,
    right_columns = right_columns,
    term_header = term_header,
    n_header = n_header,
    events_header = events_header,
    estimate_label = estimate_label,
    p_header = p_header,
    digits = digits,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family,
    striped_rows = striped_rows,
    stripe_fill = stripe_fill,
    stripe_colour = stripe_colour,
    left_width = left_width,
    plot_width = plot_width,
    right_width = right_width
  )
}

#' @export
#' @keywords internal
ggplot_add.ggforestplot_split_table_adder <- function(object, plot, ...) {
  do.call(
    .compose_split_table,
    c(list(plot = plot), object)
  )
}

bind_model_frames <- function(parts) {
  all_names <- unique(unlist(lapply(parts, names), use.names = FALSE))

  aligned <- lapply(parts, function(part) {
    part <- as.data.frame(part, stringsAsFactors = FALSE)
    missing <- setdiff(all_names, names(part))

    for (column in missing) {
      part[[column]] <- NA
    }

    part[all_names]
  })

  out <- do.call(rbind, aligned)
  rownames(out) <- NULL
  out
}

resolve_model_labels <- function(models, model_labels = NULL) {
  n_models <- length(models)

  if (is.null(model_labels)) {
    model_labels <- names(models)

    if (is.null(model_labels)) {
      model_labels <- rep("", n_models)
    }

    missing_labels <- is.na(model_labels) | !nzchar(model_labels)
    model_labels[missing_labels] <- paste("Model", which(missing_labels))
  }

  if (!is.atomic(model_labels) || length(model_labels) != n_models) {
    stop("`model_labels` must have one label per model.", call. = FALSE)
  }

  model_labels <- as.character(model_labels)

  if (anyNA(model_labels) || any(!nzchar(model_labels))) {
    stop("`model_labels` cannot contain missing or empty values.", call. = FALSE)
  }

  if (anyDuplicated(model_labels)) {
    stop("`model_labels` must be unique.", call. = FALSE)
  }

  model_labels
}

resolve_model_exponentiate <- function(exponentiate = NULL, n_models) {
  if (is.null(exponentiate)) {
    return(rep(list(NULL), n_models))
  }

  if (!is.logical(exponentiate) || !length(exponentiate) %in% c(1L, n_models) ||
      anyNA(exponentiate)) {
    stop("`exponentiate` must be `NULL`, a single logical value, or one logical value per model.", call. = FALSE)
  }

  as.list(rep(exponentiate, length.out = n_models))
}

bind_forest_source_columns <- function(part, label) {
  source_columns <- attr(part, "source_columns")

  if (is.null(source_columns)) {
    source_columns <- part
  }

  source_columns <- as.data.frame(source_columns, stringsAsFactors = FALSE)
  source_columns$group <- label
  source_columns
}

#' Bind multiple model summaries for a grouped forest plot
#'
#' Tidies multiple fitted models and stacks their fixed-effect coefficient
#' tables into a single forest-plot data frame. The resulting data can be
#' passed directly to [ggforestplot()], where model labels are used as the
#' grouping variable for dodged, color-coded estimates.
#'
#' @param models A non-empty list of fitted model objects supported by
#'   [tidy_forest_model()].
#' @param model_labels Optional labels used to identify each model in the
#'   forest plot. Defaults to list names when present, otherwise `"Model 1"`,
#'   `"Model 2"`, and so on.
#' @param exponentiate `NULL`, a single logical value, or one logical value per
#'   model. `NULL` uses the canonical scale inferred by [tidy_forest_model()]
#'   for each model.
#' @param ... Additional arguments passed to [tidy_forest_model()], such as
#'   `conf.level`, `intercept`, `term_labels`, or `sort_terms`.
#'
#' @return A standardized forest-plot data frame with one row per model term
#'   and a `group` column containing the model labels.
#' @export
#'
#' @examples
#' if (requireNamespace("broom", quietly = TRUE)) {
#'   fit1 <- lm(mpg ~ wt + hp, data = mtcars)
#'   fit2 <- lm(mpg ~ wt + qsec, data = mtcars)
#'
#'   bound <- bind_forest_models(
#'     list(Base = fit1, Adjusted = fit2)
#'   )
#'
#'   ggforestplot(bound)
#' }
bind_forest_models <- function(models,
                               model_labels = NULL,
                               exponentiate = NULL,
                               ...) {
  if (!is.list(models) || length(models) == 0L || !is.null(attr(models, "class"))) {
    stop("`models` must be a non-empty list of fitted model objects.", call. = FALSE)
  }

  model_labels <- resolve_model_labels(models, model_labels = model_labels)
  exponentiate <- resolve_model_exponentiate(exponentiate, length(models))

  parts <- vector("list", length(models))
  source_parts <- vector("list", length(models))

  for (i in seq_along(models)) {
    part <- tidy_forest_model(
      models[[i]],
      exponentiate = exponentiate[[i]],
      ...
    )
    part$group <- model_labels[[i]]

    parts[[i]] <- part
    source_parts[[i]] <- bind_forest_source_columns(part, model_labels[[i]])
  }

  scales <- vapply(parts, function(part) isTRUE(attr(part, "exponentiate")), logical(1))

  if (length(unique(scales)) > 1L) {
    stop(
      "All bound models must use the same estimate scale. ",
      "Set `exponentiate` to a single logical value to force a common scale.",
      call. = FALSE
    )
  }

  estimate_labels <- vapply(parts, function(part) {
    label <- attr(part, "estimate_label")
    if (is.null(label)) "Estimate" else label
  }, character(1))
  axis_labels <- vapply(parts, function(part) {
    label <- attr(part, "axis_label")
    if (is.null(label)) "Estimate" else label
  }, character(1))

  out <- bind_model_frames(parts)
  source_columns <- bind_model_frames(source_parts)

  attr(out, "source_columns") <- source_columns
  attr(out, "exponentiate") <- scales[[1]]
  attr(out, "estimate_label") <- if (length(unique(estimate_labels)) == 1L) {
    estimate_labels[[1]]
  } else if (isTRUE(scales[[1]])) {
    "Ratio"
  } else {
    "Estimate"
  }
  attr(out, "axis_label") <- if (length(unique(axis_labels)) == 1L) {
    axis_labels[[1]]
  } else {
    attr(out, "estimate_label")
  }
  attr(out, "conf.level") <- attr(parts[[1]], "conf.level")

  class(out) <- c("ggforestplot_bound_models", class(out))
  out
}

test_that("bind_forest_models stacks model terms with model labels", {
  skip_if_not_installed("broom")

  fit1 <- lm(mpg ~ wt + hp, data = mtcars)
  fit2 <- lm(mpg ~ wt + qsec, data = mtcars)

  out <- bind_forest_models(list(Base = fit1, Adjusted = fit2))

  expect_s3_class(out, "ggforestplot_bound_models")
  expect_equal(unique(out$group), c("Base", "Adjusted"))
  expect_false("(Intercept)" %in% out$term)
  expect_true(all(c("term", "estimate", "conf.low", "conf.high", "group") %in% names(out)))
  expect_equal(attr(out, "estimate_label"), "Estimate")
  expect_false(isTRUE(attr(out, "exponentiate")))
})

test_that("ggforestplot uses bound model labels as groups", {
  skip_if_not_installed("broom")

  fit1 <- lm(mpg ~ wt + hp, data = mtcars)
  fit2 <- lm(mpg ~ wt + qsec, data = mtcars)
  bound <- bind_forest_models(
    list(fit1, fit2),
    model_labels = c("Clinical", "Adjusted")
  )

  p <- ggforestplot(bound)

  expect_equal(p$labels$colour, "Group")
  expect_equal(unique(p$ggforestplotR_state$forest_data$group), c("Clinical", "Adjusted"))
  expect_s3_class(p, "ggplot")
})

test_that("bound model tables label terms that appear in only one model", {
  skip_if_not_installed("broom")

  fit1 <- lm(mpg ~ cyl, data = mtcars)
  fit2 <- lm(mpg ~ cyl + disp, data = mtcars)
  fit3 <- lm(mpg ~ cyl + disp + wt, data = mtcars)
  bound <- bind_forest_models(
    list(fit1, fit2, fit3),
    model_labels = c("Unadjusted", "Adjusted", "Fully Adjusted")
  )
  p <- ggforestplot(bound)
  table_spec <- build_forest_table_data(p$ggforestplotR_state$forest_data)
  estimate_text <- table_spec$table_data$text[
    table_spec$table_data$column_key == "estimate" &
      as.character(table_spec$table_data$row_key) == "wt"
  ]

  expect_match(estimate_text, "^Fully Adjusted: ")
})

test_that("bound model tables format p.value aliases with p_digits", {
  skip_if_not_installed("broom")

  fit1 <- lm(mpg ~ cyl, data = mtcars)
  fit2 <- lm(mpg ~ cyl + disp, data = mtcars)
  fit3 <- lm(mpg ~ cyl + disp + wt, data = mtcars)
  bound <- bind_forest_models(
    list(fit1, fit2, fit3),
    model_labels = c("Unadjusted", "Adjusted", "Fully Adjusted")
  )
  p <- ggforestplot(bound, p.value = "p.value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("term", "estimate", "p.value"),
    p_digits = 3
  )
  p_text <- table_spec$table_data$text[
    table_spec$table_data$column_key == "p" &
      as.character(table_spec$table_data$row_key) == "wt"
  ]

  expect_true("p" %in% table_spec$column_keys)
  expect_false("p.value" %in% table_spec$column_keys)
  expect_match(p_text, "^Fully Adjusted: ")
  expect_equal(p_text, "Fully Adjusted: 0.002")
})

test_that("bind_forest_models supports common exponentiated scales", {
  skip_if_not_installed("broom")

  set.seed(123)
  dat <- data.frame(
    x = rnorm(120),
    z = rnorm(120),
    group = factor(rep(c("A", "B"), length.out = 120))
  )
  dat$y <- rbinom(120, 1, stats::plogis(-0.2 + 0.8 * dat$x - 0.4 * dat$z))

  fit1 <- glm(y ~ x, data = dat, family = binomial())
  fit2 <- glm(y ~ x + z, data = dat, family = binomial())

  out <- bind_forest_models(
    list(Unadjusted = fit1, Adjusted = fit2),
    exponentiate = TRUE
  )

  expect_true(isTRUE(attr(out, "exponentiate")))
  expect_equal(attr(out, "estimate_label"), "OR")
  expect_true(all(out$estimate > 0))
  expect_true(all(out$conf.low > 0))
  expect_true(all(out$conf.high > 0))
})

test_that("bind_forest_models validates model labels and scales", {
  skip_if_not_installed("broom")

  fit1 <- lm(mpg ~ wt, data = mtcars)
  fit2 <- lm(mpg ~ hp, data = mtcars)

  expect_error(
    bind_forest_models(fit1),
    "`models` must be a non-empty list"
  )
  expect_error(
    bind_forest_models(list(fit1, fit2), model_labels = "Only one"),
    "`model_labels` must have one label per model"
  )
  expect_error(
    bind_forest_models(list(fit1, fit2), model_labels = c("A", "A")),
    "`model_labels` must be unique"
  )
  expect_error(
    bind_forest_models(list(fit1, fit2), exponentiate = c(TRUE, FALSE, TRUE)),
    "`exponentiate` must be `NULL`, a single logical value, or one logical value per model"
  )
})

test_that("bind_forest_models rejects mixed estimate scales", {
  skip_if_not_installed("broom")

  set.seed(123)
  dat <- data.frame(x = rnorm(80))
  dat$y <- rbinom(80, 1, stats::plogis(0.2 + dat$x))

  linear <- lm(mpg ~ wt, data = mtcars)
  logistic <- glm(y ~ x, data = dat, family = binomial())

  expect_error(
    bind_forest_models(list(linear, logistic)),
    "same estimate scale"
  )
})

test_that("ggforestplot rejects exponentiate overrides for bound models", {
  skip_if_not_installed("broom")

  bound <- bind_forest_models(list(Base = lm(mpg ~ wt, data = mtcars)))

  expect_error(
    ggforestplot(bound, exponentiate = TRUE),
    "`exponentiate` is set by `bind_forest_models\\(\\)`"
  )
})

#' Tidy a model object for forest plotting
#'
#' Uses [broom::tidy()] to convert a fitted model into forest-plot data.
#'
#' @param model A fitted model object supported by [broom::tidy()].
#' @param conf.int Logical; if `TRUE`, request confidence intervals from
#'   [broom::tidy()].
#' @param conf.level Confidence level for intervals.
#' @param exponentiate Logical; passed through to [broom::tidy()].
#' @param intercept Logical; if `FALSE`, drop the intercept term.
#' @param sort_terms How to sort rows: `"none"`, `"descending"`, or
#'   `"ascending"`.
#'
#' @return A standardized coefficient data frame ready for [ggforestplot()].
#' @export
#'
#' @examples
#' if (requireNamespace("broom", quietly = TRUE)) {
#'   fit <- lm(mpg ~ wt + hp + qsec, data = mtcars)
#'   tidy_forest_model(fit)
#'
#'   set.seed(123)
#'   logit_data <- data.frame(
#'     age = rnorm(250, mean = 62, sd = 8),
#'     bmi = rnorm(250, mean = 28, sd = 4),
#'     treatment = factor(rbinom(250, 1, 0.45), labels = c("Control", "Treatment"))
#'   )
#'   linpred <- -9 + 0.09 * logit_data$age + 0.11 * logit_data$bmi +
#'     0.9 * (logit_data$treatment == "Treatment")
#'   logit_data$event <- rbinom(250, 1, plogis(linpred))
#'   logit_fit <- glm(event ~ age + bmi + treatment, data = logit_data, family = binomial())
#'
#'   tidy_forest_model(logit_fit, exponentiate = TRUE)
#' }
tidy_forest_model <- function(model,
                              conf.int = TRUE,
                              conf.level = 0.95,
                              exponentiate = FALSE,
                              intercept = FALSE,
                              sort_terms = c("none", "descending", "ascending")) {
  sort_terms <- match.arg(sort_terms)

  if (!requireNamespace("broom", quietly = TRUE)) {
    stop(
      "The `broom` package is required to tidy model objects. ",
      "Install it or pass a coefficient data frame to `ggforestplot()` instead.",
      call. = FALSE
    )
  }

  out <- broom::tidy(
    x = model,
    conf.int = conf.int,
    conf.level = conf.level,
    exponentiate = exponentiate
  )

  if (!"term" %in% names(out) || !"estimate" %in% names(out)) {
    stop(
      "`broom::tidy()` returned an unsupported result. ",
      "Expected at least `term` and `estimate` columns.",
      call. = FALSE
    )
  }

  if (conf.int && (!"conf.low" %in% names(out) || !"conf.high" %in% names(out))) {
    stop(
      "`broom::tidy()` did not return confidence interval columns.",
      call. = FALSE
    )
  }

  if (!isTRUE(intercept)) {
    out <- out[out$term != "(Intercept)", , drop = FALSE]
  }

  as_forest_data(
    data = out,
    term = "term",
    estimate = "estimate",
    conf.low = "conf.low",
    conf.high = "conf.high",
    n = NULL,
    p.value = if ("p.value" %in% names(out)) "p.value" else NULL,
    exponentiate = exponentiate,
    sort_terms = sort_terms
  )
}

#' Draw a ggplot2 forest plot
#'
#' Builds a forest plot from standardized coefficient data or directly from a
#' fitted model.
#'
#' @param data Either a tidy coefficient data frame or a model object
#'   supported by [broom::tidy()].
#' @param term Column name holding the model term identifiers.
#' @param estimate Column name holding the point estimates.
#' @param conf.low Column name holding the lower confidence bounds.
#' @param conf.high Column name holding the upper confidence bounds.
#' @param label Optional column name used for the displayed row labels.
#' @param term_labels Optional named vector used to relabel displayed terms.
#'   Names should match values in the term column and values are the labels to
#'   display.
#' @param group Optional column name used for color-grouping estimates.
#' @param facet Optional column name used to split rows into faceted plot
#'   sections. If this column is a factor, its levels control facet order.
#' @param facet_strip_position Positioning for facet strip labels.
#' @param grouping Deprecated. Use `facet` instead.
#' @param grouping_strip_position Deprecated. Use `facet_strip_position`
#'   instead.
#' @param separate_groups Optional column name used to identify labeled
#'   variable blocks that can be outlined with grid lines.
#' @param n Optional column name holding sample sizes or other N labels for
#'   table helpers.
#' @param events Optional column name holding event counts or event labels for
#'   table helpers.
#' @param p.value Optional column name holding p-values.
#' @param exponentiate Logical; if `TRUE`, transform the estimates and draw the
#'   axis on the log scale with the reference line at 1. For model objects,
#'   `NULL` uses the canonical scale when it can be inferred, such as hazard
#'   ratios for Cox models.
#' @param sort_terms How to sort rows: `"none"`, `"descending"`, or
#'   `"ascending"`.
#' @param point_size Point size for coefficient markers.
#' @param point_shape Shape used for coefficient markers.
#' @param linewidth Line width for confidence intervals.
#' @param line_size Deprecated. Use `linewidth` instead.
#' @param staple_width Width of the terminal staples on confidence interval
#'   lines.
#' @param ci_limits Optional numeric vector of length 2 used to truncate
#'   displayed confidence intervals. Intervals extending beyond these limits
#'   are clipped to the boundary and marked with arrows when `ci_arrows` is
#'   `TRUE`.
#' @param ci_arrows Logical; if `TRUE`, draw outward-facing arrows for
#'   confidence intervals truncated by `ci_limits`.
#' @param ci_arrow_length Length of CI truncation arrows in inches.
#' @param ci_arrow_type Arrowhead type for truncated confidence intervals.
#'   Passed to [grid::arrow()].
#' @param dodge_width Horizontal dodging used for grouped estimates.
#' @param separate_lines Logical; if `TRUE`, draw grid lines
#'   around each labeled block identified by `separate_groups`.
#' @param separator_line_linetype Line type used for separator lines.
#' @param separator_line_colour Colour used for separator lines.
#' @param separator_line_size Line width used for separator lines.
#' @param striped_rows Logical; if `TRUE`, shade alternating rows.
#' @param stripe_fill Fill color used for shaded rows.
#' @param stripe_colour Border color for shaded rows.
#' @param stripe_alpha Transparency for shaded rows.
#' @param ref_line Numeric x-value where the reference line is drawn, or
#'   `NULL` to hide it. When omitted, defaults to `0` for additive effects and
#'   `1` for exponentiated effects.
#' @param ref_label Optional label drawn alongside the reference line.
#' @param ref_linetype Line type for the reference line.
#' @param ref_color Color for the reference line.
#'
#' @return A `ggplot` object. Use standard `ggplot2` functions such as
#'   [ggplot2::labs()] for plot labels, and add composition helpers after
#'   styling the main plot.
#' @export
#'
#' @examples
#' coefs <- data.frame(
#'   term = c("Age", "BMI", "Treatment"),
#'   estimate = c(0.10, -0.08, 0.34),
#'   conf.low = c(0.02, -0.16, 0.12),
#'   conf.high = c(0.18, 0.00, 0.56)
#' )
#'
#' ggforestplot(coefs)
#'
#' ggforestplot(coefs, striped_rows = TRUE, point_shape = 17)
ggforestplot <- function(data,
                         term = "term",
                         estimate = "estimate",
                         conf.low = "conf.low",
                         conf.high = "conf.high",
                         label = term,
                         term_labels = NULL,
                         group = NULL,
                         facet = NULL,
                         facet_strip_position = c("left", "right"),
                         grouping = NULL,
                         grouping_strip_position = NULL,
                         separate_groups = NULL,
                         n = NULL,
                         events = NULL,
                         p.value = NULL,
                         exponentiate = NULL,
                         sort_terms = c("none", "descending", "ascending"),
                         point_size = 2.3,
                         point_shape = 19,
                         linewidth = 0.5,
                         line_size = NULL,
                         staple_width = 0.2,
                         ci_limits = NULL,
                         ci_arrows = TRUE,
                         ci_arrow_length = 0.08,
                         ci_arrow_type = c("closed", "open"),
                         dodge_width = 0.6,
                         separate_lines = FALSE,
                         separator_line_linetype = 2,
                         separator_line_colour = "black",
                         separator_line_size = 0.4,
                         striped_rows = FALSE,
                         stripe_fill = "grey95",
                         stripe_colour = NA,
                         stripe_alpha = 1,
                         ref_line = NULL,
                         ref_label = NULL,
                         ref_linetype = 2,
                         ref_color = "grey60") {
  ref_line_missing <- missing(ref_line)

  if (!missing(line_size)) {
    if (!missing(linewidth)) {
      stop("Use only one of `linewidth` or deprecated `line_size`.", call. = FALSE)
    }

    warn_deprecated_argument("line_size", "`linewidth`")
    linewidth <- line_size
  }

  if (!missing(grouping)) {
    if (!is.null(facet)) {
      stop("Use only one of `facet` or deprecated `grouping`.", call. = FALSE)
    }

    warn_deprecated_argument("grouping", "`facet`")
    facet <- grouping
  }

  if (!missing(grouping_strip_position)) {
    if (!missing(facet_strip_position)) {
      stop(
        "Use only one of `facet_strip_position` or deprecated `grouping_strip_position`.",
        call. = FALSE
      )
    }

    warn_deprecated_argument("grouping_strip_position", "`facet_strip_position`")
    facet_strip_position <- grouping_strip_position
  }

  sort_terms <- match.arg(sort_terms)
  facet_strip_position <- match.arg(facet_strip_position)
  ci_arrow_type <- match.arg(ci_arrow_type)

  forest_data <- if (inherits(data, "ggforestplot_bound_models")) {
    if (!is.null(exponentiate)) {
      stop("`exponentiate` is set by `bind_forest_models()`; pass it there instead.", call. = FALSE)
    }

    bound_data <- data
    bound_data$label <- apply_term_labels(bound_data$term, bound_data$label, term_labels)
    validate_forest_data(bound_data, exponentiate = isTRUE(attr(bound_data, "exponentiate")))

    source_columns <- attr(bound_data, "source_columns")
    bound_data$.source_row <- seq_len(nrow(bound_data))
    bound_data <- sort_forest_data(bound_data, sort_terms = sort_terms)

    if (!is.null(source_columns)) {
      attr(bound_data, "source_columns") <- source_columns[bound_data$.source_row, , drop = FALSE]
    }

    bound_data$.source_row <- NULL
    attr(bound_data, "exponentiate") <- isTRUE(attr(data, "exponentiate"))
    attr(bound_data, "estimate_label") <- attr(data, "estimate_label")
    attr(bound_data, "axis_label") <- attr(data, "axis_label")
    attr(bound_data, "conf.level") <- attr(data, "conf.level")
    attr(bound_data, "grouping_levels") <- attr(data, "grouping_levels")
    bound_data
  } else if (is.data.frame(data)) {
    as_forest_data(
      data = data,
      term = term,
      estimate = estimate,
      conf.low = conf.low,
      conf.high = conf.high,
      label = label,
      term_labels = term_labels,
      group = group,
      grouping = facet,
      separate_groups = separate_groups,
      n = n,
      events = events,
      p.value = p.value,
      exponentiate = isTRUE(exponentiate),
      sort_terms = sort_terms
    )
  } else {
    tidy_forest_model(
      model = data,
      exponentiate = exponentiate,
      term_labels = term_labels,
      sort_terms = sort_terms
    )
  }
  plot_exponentiate <- isTRUE(attr(forest_data, "exponentiate"))
  estimate_label <- attr(forest_data, "estimate_label")
  axis_label <- attr(forest_data, "axis_label")
  column_mapping <- attr(forest_data, "column_mapping")

  if (is.null(estimate_label)) {
    estimate_label <- "Estimate"
  }

  if (is.null(axis_label)) {
    axis_label <- if (isTRUE(plot_exponentiate)) "Estimate (log scale)" else "Estimate"
  }

  default_ref_line <- if (isTRUE(plot_exponentiate)) 1 else 0

  if (ref_line_missing) {
    ref_line <- default_ref_line
  }

  draw_ref_line <- !is.null(ref_line)

  if (isTRUE(draw_ref_line) && (!is.numeric(ref_line) || length(ref_line) != 1L || is.na(ref_line))) {
    stop("`ref_line` must be a single numeric value or `NULL`.", call. = FALSE)
  }

  if (isTRUE(plot_exponentiate) && isTRUE(draw_ref_line) && ref_line <= 0) {
    stop("`ref_line` must be positive for exponentiated plots.", call. = FALSE)
  }

  ci_limits <- validate_ci_limits(ci_limits, exponentiate = plot_exponentiate)

  if (!is.logical(ci_arrows) || length(ci_arrows) != 1L || is.na(ci_arrows)) {
    stop("`ci_arrows` must be `TRUE` or `FALSE`.", call. = FALSE)
  }

  if (!is.numeric(ci_arrow_length) || length(ci_arrow_length) != 1L ||
      is.na(ci_arrow_length) || ci_arrow_length <= 0) {
    stop("`ci_arrow_length` must be a single positive number.", call. = FALSE)
  }

  display_data <- build_forest_plot_data(forest_data)
  forest_data <- display_data$plot_data
  stripe_data <- display_data$stripe_data
  separator_data <- display_data$separator_data
  plot_stripe_data <- stripe_data
  plot_x_limits <- NULL
  stripe_layer_index <- NULL

  if (!is.null(ci_limits)) {
    plot_x_limits <- ci_limits
  } else if (isTRUE(plot_exponentiate)) {
    plot_x_limits <- default_plot_background_limits(
      forest_data,
      exponentiate = plot_exponentiate,
      include_zero = draw_ref_line,
      ref_line = ref_line
    )
  }

  if (isTRUE(plot_exponentiate)) {
    plot_stripe_data$xmin <- plot_x_limits[1]
    plot_stripe_data$xmax <- plot_x_limits[2]
  }

  ci_plot_data <- build_ci_plot_data(
    forest_data,
    ci_limits = ci_limits,
    exponentiate = plot_exponentiate
  )
  has_groups <- any(!is.na(forest_data$group) & nzchar(forest_data$group))
  dodge <- ggplot2::position_dodge(width = dodge_width)
  point_mapping <- if (has_groups) {
    ggplot2::aes(
      x = .data$estimate,
      y = .data$row_key,
      colour = .data$group
    )
  } else {
    ggplot2::aes(
      x = .data$estimate,
      y = .data$row_key
    )
  }
  ci_mapping <- if (has_groups) {
    ggplot2::aes(
      x = .data$ci_estimate,
      y = .data$row_key,
      xmin = .data$ci_low,
      xmax = .data$ci_high,
      colour = .data$group
    )
  } else {
    ggplot2::aes(
      x = .data$ci_estimate,
      y = .data$row_key,
      xmin = .data$ci_low,
      xmax = .data$ci_high
    )
  }

  p <- ggplot2::ggplot(ci_plot_data, point_mapping)

  if (isTRUE(striped_rows)) {
    p <- p + ggplot2::geom_rect(
      data = plot_stripe_data[plot_stripe_data$fill_key == "stripe", , drop = FALSE],
      mapping = ggplot2::aes(
        xmin = .data$xmin,
        xmax = .data$xmax,
        ymin = .data$ymin,
        ymax = .data$ymax
      ),
      inherit.aes = FALSE,
      fill = stripe_fill,
      colour = stripe_colour,
      alpha = stripe_alpha
    )
    stripe_layer_index <- length(p$layers)
  }

  if (isTRUE(separate_lines) && nrow(separator_data) > 0L) {
    p <- p + ggplot2::geom_hline(
      data = separator_data,
      mapping = ggplot2::aes(yintercept = .data$yintercept),
      inherit.aes = FALSE,
      linetype = separator_line_linetype,
      colour = separator_line_colour,
      linewidth = separator_line_size
    )
  }

  left_arrow_data <- ci_plot_data[ci_plot_data$ci_truncated_left, , drop = FALSE]
  right_arrow_data <- ci_plot_data[ci_plot_data$ci_truncated_right, , drop = FALSE]
  arrow_caps <- isTRUE(ci_arrows) &&
    any(ci_plot_data$ci_truncated_left | ci_plot_data$ci_truncated_right)
  arrow_spec <- grid::arrow(
    length = grid::unit(ci_arrow_length, "inches"),
    type = ci_arrow_type
  )

  if (isTRUE(arrow_caps)) {
    complete_ci_data <- ci_plot_data[
      !ci_plot_data$ci_truncated_left & !ci_plot_data$ci_truncated_right,
      ,
      drop = FALSE
    ]
    truncated_ci_data <- ci_plot_data[
      ci_plot_data$ci_truncated_left | ci_plot_data$ci_truncated_right,
      ,
      drop = FALSE
    ]

    if (nrow(complete_ci_data) > 0L) {
      p <- p + ggplot2::geom_errorbar(
        data = complete_ci_data,
        mapping = ci_mapping,
        width = staple_width,
        linewidth = linewidth,
        position = dodge,
        orientation = "y"
      )
    }

    if (nrow(truncated_ci_data) > 0L) {
      p <- p + ggplot2::geom_errorbar(
        data = truncated_ci_data,
        mapping = ci_mapping,
        width = 0,
        linewidth = linewidth,
        position = dodge,
        orientation = "y"
      )
    }

    low_staple_data <- truncated_ci_data[!truncated_ci_data$ci_truncated_left, , drop = FALSE]
    high_staple_data <- truncated_ci_data[!truncated_ci_data$ci_truncated_right, , drop = FALSE]
    low_staple_mapping <- if (has_groups) {
      ggplot2::aes(
        x = .data$ci_low,
        y = .data$row_key,
        xmin = .data$ci_low,
        xmax = .data$ci_low,
        colour = .data$group
      )
    } else {
      ggplot2::aes(
        x = .data$ci_low,
        y = .data$row_key,
        xmin = .data$ci_low,
        xmax = .data$ci_low
      )
    }
    high_staple_mapping <- if (has_groups) {
      ggplot2::aes(
        x = .data$ci_high,
        y = .data$row_key,
        xmin = .data$ci_high,
        xmax = .data$ci_high,
        colour = .data$group
      )
    } else {
      ggplot2::aes(
        x = .data$ci_high,
        y = .data$row_key,
        xmin = .data$ci_high,
        xmax = .data$ci_high
      )
    }

    if (nrow(low_staple_data) > 0L) {
      p <- p + ggplot2::geom_errorbar(
        data = low_staple_data,
        mapping = low_staple_mapping,
        width = staple_width,
        linewidth = linewidth,
        position = dodge,
        orientation = "y"
      )
    }

    if (nrow(high_staple_data) > 0L) {
      p <- p + ggplot2::geom_errorbar(
        data = high_staple_data,
        mapping = high_staple_mapping,
        width = staple_width,
        linewidth = linewidth,
        position = dodge,
        orientation = "y"
      )
    }
  } else {
    p <- p + ggplot2::geom_errorbar(
      mapping = ci_mapping,
      width = staple_width,
      linewidth = linewidth,
      position = dodge,
      orientation = "y"
    )
  }

  if (isTRUE(ci_arrows) && nrow(left_arrow_data) > 0L) {
    left_arrow_mapping <- if (has_groups) {
      ggplot2::aes(
        x = .data$ci_arrow_left_start,
        xend = .data$ci_low,
        y = .data$row_key,
        yend = .data$row_key,
        colour = .data$group
      )
    } else {
      ggplot2::aes(
        x = .data$ci_arrow_left_start,
        xend = .data$ci_low,
        y = .data$row_key,
        yend = .data$row_key
      )
    }
    p <- p + ggplot2::geom_segment(
      data = left_arrow_data,
      mapping = left_arrow_mapping,
      inherit.aes = FALSE,
      linewidth = linewidth,
      arrow = arrow_spec,
      position = dodge
    )
  }

  if (isTRUE(ci_arrows) && nrow(right_arrow_data) > 0L) {
    right_arrow_mapping <- if (has_groups) {
      ggplot2::aes(
        x = .data$ci_arrow_right_start,
        xend = .data$ci_high,
        y = .data$row_key,
        yend = .data$row_key,
        colour = .data$group
      )
    } else {
      ggplot2::aes(
        x = .data$ci_arrow_right_start,
        xend = .data$ci_high,
        y = .data$row_key,
        yend = .data$row_key
      )
    }
    p <- p + ggplot2::geom_segment(
      data = right_arrow_data,
      mapping = right_arrow_mapping,
      inherit.aes = FALSE,
      linewidth = linewidth,
      arrow = arrow_spec,
      position = dodge
    )
  }

  p <- p +
    ggplot2::geom_point(
      size = point_size,
      shape = point_shape,
      position = dodge
    ) +
    ggplot2::scale_y_discrete(
      labels = display_data$axis_labels,
      drop = TRUE
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      panel.grid.minor = ggplot2::element_blank(),
      strip.placement = "outside"
    )

  if (isTRUE(draw_ref_line)) {
    p <- p + ggplot2::geom_vline(
      xintercept = ref_line,
      linetype = ref_linetype,
      colour = ref_color
    )

    if (!is.null(ref_label)) {
      p <- p + ggplot2::annotate(
        "text",
        x = ref_line,
        y = Inf,
        label = ref_label,
        angle = 90,
        hjust = 1.1,
        vjust = -0.4,
        colour = ref_color
      )
    }
  }

  if (isTRUE(plot_exponentiate)) {
    p <- p + ggplot2::scale_x_log10(
      limits = plot_x_limits,
      expand = ggplot2::expansion(mult = 0)
    )
  } else if (!is.null(plot_x_limits)) {
    p <- p + ggplot2::scale_x_continuous(
      limits = plot_x_limits,
      expand = ggplot2::expansion(mult = 0)
    )
  }

  if (isTRUE(display_data$has_groupings)) {
    p <- p + ggplot2::facet_wrap(
      ggplot2::vars(grouping_panel),
      ncol = 1,
      scales = "free_y",
      strip.position = facet_strip_position
    )
  }

  p <- p + ggplot2::labs(
    x = axis_label,
    y = NULL,
    colour = if (has_groups) "Group" else NULL
  )

  p$ggforestplotR_state <- list(
    forest_data = forest_data,
    column_mapping = column_mapping,
    stripe_data = stripe_data,
    stripe_layer_index = stripe_layer_index,
    has_groupings = display_data$has_groupings,
    facet_strip_position = facet_strip_position,
    grouping_strip_position = facet_strip_position,
    defaults = list(
      striped_rows = striped_rows,
      stripe_fill = stripe_fill,
      stripe_colour = stripe_colour,
      stripe_alpha = stripe_alpha,
      exponentiate = plot_exponentiate,
      estimate_label = estimate_label,
      axis_label = axis_label,
      ref_line = ref_line,
      ref_label = ref_label,
      ci_limits = ci_limits,
      ci_arrows = ci_arrows
    )
  )

  p
}

# ─── Column resolution ────────────────────────────────────────────────────────

resolve_column <- function(data, column, arg, required = TRUE) {
  if (is.null(column)) {
    if (required) {
      stop(sprintf("`%s` must be supplied.", arg), call. = FALSE)
    }
    return(NULL)
  }
  
  if (!is.character(column) || length(column) != 1L) {
    stop(sprintf("`%s` must be a single column name (string).", arg), call. = FALSE)
  }
  
  if (!column %in% names(data)) {
    stop(sprintf("Column `%s` was not found in `data`.", column), call. = FALSE)
  }
  
  column
}

# ─── Data validation ──────────────────────────────────────────────────────────

validate_forest_data <- function(data, exponentiate = FALSE) {
  required <- c(
    "term", "estimate", "conf.low", "conf.high",
    "label", "group", "grouping", "separate_groups", "n", "events", "p.value"
  )
  missing <- setdiff(required, names(data))
  
  if (length(missing) > 0L) {
    stop(
      sprintf("Missing required forest data columns: %s", paste(missing, collapse = ", ")),
      call. = FALSE
    )
  }
  
  numeric_cols <- c("estimate", "conf.low", "conf.high")
  
  for (col in numeric_cols) {
    if (anyNA(data[[col]])) {
      stop(sprintf("Column `%s` cannot contain missing values.", col), call. = FALSE)
    }
    
    if (isTRUE(exponentiate) && any(data[[col]] <= 0)) {
      stop(
        "Exponentiated forest plots require strictly positive `estimate`, `conf.low`, and `conf.high` values.",
        call. = FALSE
      )
    }
  }
  
  if (any(data$conf.low > data$conf.high)) {
    stop("`conf.low` cannot be greater than `conf.high`.", call. = FALSE)
  }
  
  invisible(data)
}

# ─── Column normalisation ────────────────────────────────────────────────────

normalize_table_columns <- function(columns) {
  default_order <- c("term", "n", "events", "estimate", "p")
  
  if (is.null(columns)) {
    return(NULL)
  }
  
  if (is.numeric(columns)) {
    idx <- as.integer(columns)
    
    if (anyNA(idx) || any(idx < 1L | idx > length(default_order))) {
      stop("Numeric table columns must be between 1 and 4.", call. = FALSE)
    }
    
    return(unique(default_order[idx]))
  }
  
  if (!is.character(columns)) {
    stop("Table columns must be specified by name or position.", call. = FALSE)
  }
  
  aliases <- c(
    term = "term", terms = "term", label = "term", labels = "term",
    subgroup = "term", subgroups = "term",
    n = "n", samplesize = "n", sample_size = "n",
    events = "events", event = "events", cases = "events", count = "events",
    estimate = "estimate", estimates = "estimate",
    effect = "estimate", effects = "estimate",
    p = "p", pvalue = "p", p.value = "p", p_value = "p", pvalues = "p"
  )
  
  normalized <- gsub("\\s+", "", tolower(columns))
  resolved <- unname(aliases[normalized])
  
  if (anyNA(resolved)) {
    bad <- unique(columns[is.na(resolved)])
    stop(
      sprintf("Unsupported table columns: %s", paste(bad, collapse = ", ")),
      call. = FALSE
    )
  }
  
  unique(resolved)
}

# ─── Formatting helpers ──────────────────────────────────────────────────────

#' Shared logic for collapsing formatted values with optional group labels.
#'
#' All three public formatters (p-values, estimates, n-values) produce a
#' character vector of per-row formatted strings, then need identical
#' deduplication / group-prefixing / newline-collapsing.  This function
#' handles that single concern.
#' @keywords internal
#' @noRd
collapse_grouped_values <- function(formatted, group = NULL) {
  keep <- !is.na(formatted) & nzchar(formatted)
  
  if (!any(keep)) {
    return("")
  }
  
  non_empty <- formatted[keep]
  
  if (length(unique(non_empty)) == 1L) {
    return(non_empty[1L])
  }
  
  if (all(is.na(group) | !nzchar(group))) {
    return(paste(non_empty, collapse = "\n"))
  }
  
  group_labels <- ifelse(
    is.na(group) | !nzchar(group),
    paste0("Series ", seq_along(formatted)),
    group
  )
  paste(paste0(group_labels[keep], ": ", formatted[keep]), collapse = "\n")
}

format_forest_p_values <- function(values, group = NULL, digits = 2) {
  d <- max(3L, digits)
  values <- as.numeric(values)
  formatted <- ifelse(is.na(values), "", format.pval(values, digits = d, eps = 10^(-d)))
  collapse_grouped_values(formatted, group)
}

format_forest_estimates <- function(estimate, conf.low, conf.high,
                                    group = NULL, digits = 2) {
  fmt <- paste0("%.", digits, "f (%.", digits, "f, %.", digits, "f)")
  formatted <- sprintf(fmt, estimate, conf.low, conf.high)
  collapse_grouped_values(formatted, group)
}

format_forest_table_values <- function(values, group = NULL) {
  formatted <- as.character(values)
  formatted[is.na(formatted)] <- ""
  collapse_grouped_values(formatted, group)
}

# ─── Plot data construction (decomposed into single-purpose passes) ──────────

assign_grouping_panels <- function(data, has_groupings) {
  if (has_groupings) {
    ifelse(
      is.na(data$grouping) | !nzchar(data$grouping),
      "(Ungrouped)",
      data$grouping
    )
  } else {
    rep(NA_character_, nrow(data))
  }
}

#' Within each panel, if a separate_groups value appears more than once the
#' labels are ambiguous.  Prefix them with "group: label" so the axis is
#' readable.
#' @keywords internal
#' @noRd
prefix_ambiguous_labels <- function(data, has_groupings) {
  panel_values <- if (has_groupings) unique(data$grouping_panel) else "__all__"
  
  for (pv in panel_values) {
    idx <- if (has_groupings) which(data$grouping_panel == pv) else seq_len(nrow(data))
    sep_vals <- data$separate_groups[idx]
    counts <- table(sep_vals[!is.na(sep_vals) & nzchar(sep_vals)])
    prefix_groups <- names(counts[counts > 1L])
    
    if (length(prefix_groups) > 0L) {
      prefix_idx <- idx[!is.na(sep_vals) & sep_vals %in% prefix_groups]
      data$label[prefix_idx] <- paste0(data$separate_groups[prefix_idx], ": ", data$label[prefix_idx])
    }
  }
  
  data
}

#' Assign a unique row_key per label within each panel and set factor levels
#' in display order.
#' @keywords internal
#' @noRd
assign_row_keys <- function(data, has_groupings) {
  panel_values <- if (has_groupings) unique(data$grouping_panel) else "__all__"
  data$row_key <- NA_character_
  all_levels <- character()
  
  for (pv in panel_values) {
    idx <- if (has_groupings) which(data$grouping_panel == pv) else seq_len(nrow(data))
    panel_labels <- unique(data$label[idx])
    
    keys <- if (has_groupings) paste(pv, panel_labels, sep = "___") else panel_labels
    row_map <- stats::setNames(keys, panel_labels)
    
    data$row_key[idx] <- unname(row_map[data$label[idx]])
    all_levels <- c(all_levels, keys)
  }
  
  data$row_key <- factor(data$row_key, levels = all_levels)
  data
}

#' Build axis label lookup: row_key -> display label.
#' @keywords internal
#' @noRd
build_axis_labels <- function(data, has_groupings) {
  panel_values <- if (has_groupings) unique(data$grouping_panel) else "__all__"
  labels <- character()
  
  for (pv in panel_values) {
    idx <- if (has_groupings) which(data$grouping_panel == pv) else seq_len(nrow(data))
    panel_labels <- unique(data$label[idx])
    keys <- if (has_groupings) paste(pv, panel_labels, sep = "___") else panel_labels
    labels <- c(labels, stats::setNames(panel_labels, keys))
  }
  
  labels
}

#' Build a data frame of alternating stripe rectangles for each panel.
#' @keywords internal
#' @noRd
build_stripe_rectangles <- function(data, has_groupings) {
  panel_values <- if (has_groupings) unique(data$grouping_panel) else "__all__"
  parts <- vector("list", length(panel_values))
  
  for (i in seq_along(panel_values)) {
    pv <- panel_values[[i]]
    idx <- if (has_groupings) which(data$grouping_panel == pv) else seq_len(nrow(data))
    n_rows <- length(unique(data$row_key[idx]))
    
    parts[[i]] <- data.frame(
      grouping_panel = if (has_groupings) pv else NA_character_,
      stripe_id = seq_len(n_rows),
      xmin = -Inf,
      xmax = Inf,
      ymin = seq_len(n_rows) - 0.5,
      ymax = seq_len(n_rows) + 0.5,
      stringsAsFactors = FALSE
    )
  }
  
  stripe_data <- do.call(rbind, parts)
  stripe_data$fill_key <- ifelse(stripe_data$stripe_id %% 2 == 1, "stripe", "base")
  stripe_data
}

#' Detect runs of identical separate_groups values within each panel and
#' return a data frame of horizontal separator positions.
#' @keywords internal
#' @noRd
build_separate_lines <- function(data, has_groupings) {
  panel_values <- if (has_groupings) unique(data$grouping_panel) else "__all__"
  parts <- vector("list", length(panel_values))
  
  for (i in seq_along(panel_values)) {
    pv <- panel_values[[i]]
    idx <- if (has_groupings) which(data$grouping_panel == pv) else seq_len(nrow(data))
    
    row_keys <- levels(data$row_key)[levels(data$row_key) %in% data$row_key[idx]]
    
    # Map each row_key to its separate_groups value
    sep_groups <- vapply(row_keys, function(rk) {
      row_idx <- idx[as.character(data$row_key[idx]) == rk]
      vals <- unique(data$separate_groups[row_idx])
      vals <- vals[!is.na(vals) & nzchar(vals)]
      if (length(vals) == 0L) NA_character_ else vals[1L]
    }, character(1))
    
    separator_rows <- list()
    run_start <- 1L
    
    while (run_start <= length(sep_groups)) {
      current <- sep_groups[run_start]
      run_end <- run_start
      
      if (!is.na(current) && nzchar(current)) {
        while (run_end < length(sep_groups) &&
               !is.na(sep_groups[run_end + 1L]) &&
               sep_groups[run_end + 1L] == current) {
          run_end <- run_end + 1L
        }
        
        separator_rows[[length(separator_rows) + 1L]] <- data.frame(
          grouping_panel = if (has_groupings) pv else NA_character_,
          separate_groups = unname(current),
          yintercept = c(run_start - 0.5, run_end + 0.5),
          stringsAsFactors = FALSE
        )
      }
      
      run_start <- run_end + 1L
    }
    
    parts[[i]] <- if (length(separator_rows) > 0L) do.call(rbind, separator_rows) else NULL
  }
  
  separator_data <- do.call(rbind, parts)
  
  if (is.null(separator_data)) {
    data.frame(
      grouping_panel = character(),
      yintercept = numeric(),
      stringsAsFactors = FALSE
    )
  } else {
    unique(separator_data[c("grouping_panel", "yintercept")])
  }
}

#' Main entry point.  Orchestrates the four passes defined above.
#' @keywords internal
#' @noRd
build_forest_plot_data <- function(data) {
  has_groupings <- any(!is.na(data$grouping) & nzchar(data$grouping))
  plot_data <- data
  plot_data$grouping_panel <- assign_grouping_panels(plot_data, has_groupings)
  
  plot_data <- prefix_ambiguous_labels(plot_data, has_groupings)
  plot_data <- assign_row_keys(plot_data, has_groupings)
  
  stripe_data <- build_stripe_rectangles(plot_data, has_groupings)
  separator_data <- build_separate_lines(plot_data, has_groupings)
  axis_labels <- build_axis_labels(plot_data, has_groupings)
  
  list(
    plot_data = plot_data,
    stripe_data = stripe_data,
    separator_data = separator_data,
    axis_labels = axis_labels,
    has_groupings = has_groupings && length(unique(plot_data$grouping_panel)) > 1
  )
}

# ─── Table data construction ─────────────────────────────────────────────────

build_forest_table_data <- function(data,
                                    show_terms = TRUE,
                                    show_n = FALSE,
                                    show_events = FALSE,
                                    show_estimate = TRUE,
                                    show_p = FALSE,
                                    term_header = "Term",
                                    n_header = "N",
                                    events_header = "Events",
                                    estimate_label = "Estimate",
                                    p_header = "P-value",
                                    digits = 2,
                                    columns = NULL) {
  row_levels <- levels(data$row_key)
  row_parts <- vector("list", length(row_levels))
  
  for (i in seq_along(row_levels)) {
    row_key <- row_levels[[i]]
    idx <- which(as.character(data$row_key) == row_key)
    
    if (length(idx) == 0L) next
    
    rd <- data[idx, , drop = FALSE]
    row_parts[[i]] <- data.frame(
      row_key = row_key,
      grouping_panel = rd$grouping_panel[1L],
      term_text = rd$label[1L],
      n_text = format_forest_table_values(rd$n, rd$group),
      events_text = format_forest_table_values(rd$events, rd$group),
      estimate_text = format_forest_estimates(
        rd$estimate, rd$conf.low, rd$conf.high, rd$group, digits = digits
      ),
      p_text = format_forest_p_values(rd$p.value, rd$group, digits = digits),
      stringsAsFactors = FALSE
    )
  }
  
  table_rows <- do.call(rbind, row_parts)
  table_rows$row_key <- factor(table_rows$row_key, levels = row_levels)
  
  # Determine which columns to show
  if (is.null(columns)) {
    column_keys <- character()
    if (isTRUE(show_terms))    column_keys <- c(column_keys, "term")
    if (isTRUE(show_n))        column_keys <- c(column_keys, "n")
    if (isTRUE(show_events))   column_keys <- c(column_keys, "events")
    if (isTRUE(show_estimate)) column_keys <- c(column_keys, "estimate")
    if (isTRUE(show_p))        column_keys <- c(column_keys, "p")
  } else {
    column_keys <- normalize_table_columns(columns)
  }
  
  if (length(column_keys) == 0L) {
    stop("Select at least one table column to display.", call. = FALSE)
  }
  
  column_field_lookup <- c(
    term = "term_text",
    n = "n_text",
    events = "events_text",
    estimate = "estimate_text",
    p = "p_text"
  )
  header_lookup <- c(
    term = term_header,
    n = n_header,
    events = events_header,
    estimate = sprintf("%s (95%% CI)", estimate_label),
    p = p_header
  )
  
  # Build long-format table.  column_position is left as NA — the layout
  
  # functions (layout_split_table_spec / layout_center_table_spec) are
  # responsible for setting real positions.
  long_parts <- vector("list", length(column_keys))
  
  for (i in seq_along(column_keys)) {
    key <- column_keys[[i]]
    long_parts[[i]] <- data.frame(
      row_key = table_rows$row_key,
      grouping_panel = table_rows$grouping_panel,
      column_key = key,
      column_position = NA_real_,
      text = table_rows[[column_field_lookup[[key]]]],
      stringsAsFactors = FALSE
    )
  }
  
  table_data <- do.call(rbind, long_parts)
  table_data$row_key <- factor(table_data$row_key, levels = row_levels)
  
  list(
    table_data = table_data,
    positions = rep(NA_real_, length(column_keys)),
    header_positions = rep(NA_real_, length(column_keys)),
    headers = unname(header_lookup[column_keys]),
    column_keys = column_keys
  )
}

# ─── Grid line data ──────────────────────────────────────────────────────────

build_table_line_data <- function(stripe_data, has_groupings = FALSE) {
  if (isTRUE(has_groupings)) {
    line_parts <- lapply(
      split(stripe_data, stripe_data$grouping_panel, drop = TRUE),
      function(panel_data) {
        boundary_values <- unique(c(
          min(panel_data$ymin),
          panel_data$ymin[panel_data$stripe_id > 1L],
          max(panel_data$ymax)
        ))
        data.frame(
          grouping_panel = panel_data$grouping_panel[1L],
          yintercept = boundary_values,
          stringsAsFactors = FALSE
        )
      }
    )
    do.call(rbind, line_parts)
  } else {
    data.frame(
      yintercept = unique(c(
        min(stripe_data$ymin),
        stripe_data$ymin[stripe_data$stripe_id > 1L],
        max(stripe_data$ymax)
      )),
      stringsAsFactors = FALSE
    )
  }
}

# ─── Text measurement ────────────────────────────────────────────────────────

#' Ground-truth width measurement via grid graphics.  Handles multi-line
#' strings by splitting on newlines and returning the widest line.
#' @keywords internal
#' @noRd
measure_max_grob_width <- function(text,
                                   fontsize_pt,
                                   fontface = "plain",
                                   fontfamily = "") {
  text <- as.character(text)
  text[is.na(text)] <- ""
  
  if (length(text) == 0L) return(0)
  
  widths <- vapply(text, function(value) {
    lines <- strsplit(value, "\n", fixed = TRUE)[[1L]]
    if (length(lines) == 0L) return(0)
    
    max(vapply(lines, function(line) {
      if (!nzchar(line)) return(0)
      grid::convertWidth(
        grid::grobWidth(
          grid::textGrob(
            line,
            gp = grid::gpar(
              fontsize = fontsize_pt,
              fontface = fontface,
              fontfamily = fontfamily
            )
          )
        ),
        "inches",
        valueOnly = TRUE
      )
    }, numeric(1)), 0)
  }, numeric(1))
  
  max(widths, 0)
}

#' Measure the displayed text width (in inches) for each column, taking the
#' max of the header and all cell values.
#' @keywords internal
#' @noRd
measure_table_text_widths <- function(table_spec,
                                      text_size = 3.2,
                                      header_text_size = 11,
                                      header_fontface = "bold",
                                      header_family = "") {
  text_size_pt <- text_size * (72.27 / 25.4)
  
  stats::setNames(vapply(seq_along(table_spec$column_keys), function(i) {
    key <- table_spec$column_keys[[i]]
    values <- table_spec$table_data$text[table_spec$table_data$column_key == key]
    max(
      measure_max_grob_width(
        table_spec$headers[[i]],
        fontsize_pt = header_text_size,
        fontface = header_fontface,
        fontfamily = header_family
      ),
      measure_max_grob_width(values, fontsize_pt = text_size_pt, fontface = "plain")
    )
  }, numeric(1)), table_spec$column_keys)
}

# ─── Column width estimation ─────────────────────────────────────────────────

#' Default per-column base padding.  Uses a known lookup for the four standard
#' keys and a sensible fallback for anything else.
#' @keywords internal
#' @noRd
column_base_padding <- function(column_key) {
  known <- c(term = 0.16, n = 0.10, events = 0.12, estimate = 0.18, p = 0.12)
  pad <- known[[column_key]]
  if (is.null(pad)) 0.14 else unname(pad)
}

#' Total column width = measured text width + base padding + alignment padding.
#' @keywords internal
#' @noRd
estimate_split_column_widths <- function(table_spec,
                                         text_size = 3.2,
                                         header_text_size = 11,
                                         header_fontface = "bold",
                                         header_family = "",
                                         alignment = c("left", "center", "right")) {
  alignment <- match.arg(alignment)
  text_widths <- measure_table_text_widths(
    table_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family
  )
  alignment_padding <- switch(alignment, left = 0.06, right = 0.06, center = 0.05)
  
  stats::setNames(vapply(seq_along(table_spec$column_keys), function(i) {
    key <- table_spec$column_keys[[i]]
    unname(text_widths[[key]]) + column_base_padding(key) + alignment_padding
  }, numeric(1)), table_spec$column_keys)
}

# ─── Layout engines ──────────────────────────────────────────────────────────

#' Lay out a table spec for a split forest plot.
#'
#' Computes column positions and a content width.  Does NOT try to account for
#' which side of the plot the table will sit on — that's handled at assembly
#' time by equalising the two table widths.
#' @keywords internal
#' @noRd
layout_split_table_spec <- function(table_spec,
                                    text_size = 3.2,
                                    header_text_size = 11,
                                    header_fontface = "bold",
                                    header_family = "",
                                    alignment = c("left", "right")) {
  alignment <- match.arg(alignment)
  column_widths <- estimate_split_column_widths(
    table_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family,
    alignment = alignment
  )
  text_widths <- measure_table_text_widths(
    table_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family
  )
  gap <- 0.2
  
  positions <- if (alignment == "left") {
    starts <- c(0, utils::head(cumsum(column_widths + gap), -1))
    starts + 0.02
  } else {
    cumsum(column_widths + c(rep(gap, length(column_widths) - 1L), 0))
  }
  
  table_spec$table_data$column_position <- unname(
    positions[match(table_spec$table_data$column_key, table_spec$column_keys)]
  )
  table_spec$alignment               <- alignment
  table_spec$positions               <- unname(positions)
  table_spec$header_positions        <- unname(positions)
  table_spec$estimated_column_widths <- unname(column_widths)
  table_spec$displayed_column_widths <- unname(text_widths)
  table_spec$content_width <- sum(column_widths) +
    gap * max(length(column_widths) - 1L, 0)
  table_spec
}

layout_center_table_spec <- function(table_spec,
                                     text_size = 3.2,
                                     header_text_size = 11,
                                     header_fontface = "bold",
                                     header_family = "") {
  column_widths <- estimate_split_column_widths(
    table_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family,
    alignment = "center"
  )
  text_widths <- measure_table_text_widths(
    table_spec,
    text_size = text_size,
    header_text_size = header_text_size,
    header_fontface = header_fontface,
    header_family = header_family
  )
  gap <- 0.55
  left_edges <- cumsum(c(0, utils::head(column_widths + gap, -1)))
  positions <- left_edges + column_widths / 2
  
  table_spec$table_data$column_position <- unname(
    positions[match(table_spec$table_data$column_key, table_spec$column_keys)]
  )
  table_spec$positions               <- unname(positions)
  table_spec$header_positions        <- unname(positions)
  table_spec$estimated_column_widths <- unname(column_widths)
  table_spec$displayed_column_widths <- unname(text_widths)
  table_spec$content_width <- sum(column_widths) +
    gap * max(length(column_widths) - 1L, 0)
  table_spec
}

compute_table_x_limits <- function(table_spec, pad = 0.03) {
  widths <- if (!is.null(table_spec$displayed_column_widths)) {
    table_spec$displayed_column_widths
  } else {
    table_spec$estimated_column_widths
  }
  positions <- table_spec$positions
  alignment <- if (!is.null(table_spec$alignment)) table_spec$alignment else "center"

  if (alignment == "left") {
    xmin <- min(positions) - pad
    xmax <- max(positions + widths) + pad
  } else if (alignment == "right") {
    xmin <- min(positions - widths) - pad
    xmax <- max(positions) + pad
  } else {
    xmin <- min(positions - widths / 2) - pad
    xmax <- max(positions + widths / 2) + pad
  }

  c(xmin, xmax)
}

split_table_width_multiplier <- function(n_columns) {
  n_columns <- as.integer(n_columns[[1]])

  if (is.na(n_columns) || n_columns <= 0L) {
    return(0)
  }

  if (n_columns == 1L) {
    return(0.5)
  }

  1 + (n_columns - 2L) / 3
}

# ─── Plot limits ─────────────────────────────────────────────────────────────

default_plot_background_limits <- function(forest_data,
                                           exponentiate = FALSE,
                                           include_zero = TRUE) {
  xmin <- min(forest_data$conf.low, na.rm = TRUE)
  xmax <- max(forest_data$conf.high, na.rm = TRUE)
  
  if (isTRUE(include_zero)) {
    null_value <- if (isTRUE(exponentiate)) 1 else 0
    xmin <- min(xmin, null_value)
    xmax <- max(xmax, null_value)
  }
  
  if (isTRUE(exponentiate)) {
    pad_mult <- 1.08
    c(xmin / pad_mult, xmax * pad_mult)
  } else {
    span <- xmax - xmin
    pad <- if (is.finite(span) && span > 0) span * 0.08 else max(abs(xmax), 1) * 0.08
    c(xmin - pad, xmax + pad)
  }
}


# ─── ggplot2 table panel ─────────────────────────────────────────────────────

#' Build a ggplot2 table panel for one side of a split forest plot.
#'
#' Uses symmetric expansion and uniform margins.  The "equal spacing"
#' guarantee comes from `combine_split_forest_plot()` giving both table
#' panels the same patchwork width — not from per-side padding hacks here.
#' @keywords internal
#' @noRd
build_forest_table_plot <- function(table_spec,
                                    stripe_data,
                                    has_groupings = FALSE,
                                    grouping_strip_position = c("left", "right"),
                                    table_position = c("left", "right"),
                                    striped_rows = FALSE,
                                    stripe_fill = "grey95",
                                    stripe_colour = NA,
                                    text_size = 3.2,
                                    grid_lines = FALSE,
                                    grid_line_colour = "black",
                                    grid_line_size = 0.3,
                                    grid_line_linetype = 1,
                                    x_expand = ggplot2::expansion(mult = 0.05),
                                    x_limits = NULL,
                                    plot_margin = ggplot2::margin(5.5, 4, 5.5, 4),
                                    text_hjust = 0.5,
                                    header_hjust = 0.5,
                                    header_text_size = 11,
                                    header_fontface = "bold",
                                    header_family = NULL) {
  grouping_strip_position <- match.arg(grouping_strip_position)
  table_position <- match.arg(table_position)

  if (all(is.na(table_spec$positions))) {
    table_spec <- layout_center_table_spec(table_spec, text_size = text_size)
  }
  
  p <- ggplot2::ggplot(
    table_spec$table_data,
    ggplot2::aes(x = .data$column_position, y = .data$row_key, label = .data$text)
  )

  if (is.null(x_limits)) {
    x_limits <- compute_table_x_limits(table_spec)
  }
  
  if (isTRUE(striped_rows)) {
    p <- p + ggplot2::geom_rect(
      data = stripe_data[stripe_data$fill_key == "stripe", , drop = FALSE],
      mapping = ggplot2::aes(
        xmin = .data$xmin, xmax = .data$xmax,
        ymin = .data$ymin, ymax = .data$ymax
      ),
      inherit.aes = FALSE,
      fill = stripe_fill,
      colour = stripe_colour
    )
  }
  
  if (isTRUE(grid_lines)) {
    line_data <- build_table_line_data(stripe_data, has_groupings = has_groupings)
    
    if (nrow(line_data) > 0L) {
      p <- p + ggplot2::geom_hline(
        data = line_data,
        mapping = ggplot2::aes(yintercept = .data$yintercept),
        inherit.aes = FALSE,
        colour = grid_line_colour,
        linewidth = grid_line_size,
        linetype = grid_line_linetype
      )
    }
  }
  
  p <- p +
    ggplot2::geom_text(hjust = text_hjust, size = text_size, lineheight = 0.95) +
    ggplot2::scale_x_continuous(
      breaks = table_spec$header_positions,
      labels = table_spec$headers,
      position = "top",
      expand = x_expand,
      limits = x_limits
    ) +
    ggplot2::coord_cartesian(clip = "off") +
    ggplot2::scale_y_discrete(labels = function(x) rep("", length(x)), drop = TRUE) +
    ggplot2::theme_void() +
    ggplot2::theme(
      axis.title            = ggplot2::element_blank(),
      axis.text.y           = ggplot2::element_blank(),
      axis.text.x.bottom    = ggplot2::element_blank(),
      axis.text.x.top       = ggplot2::element_text(
        face = header_fontface,
        family = header_family,
        size = header_text_size,
        colour = "black",
        hjust = header_hjust,
        margin = ggplot2::margin(b = 0)
      ),
      axis.ticks            = ggplot2::element_blank(),
      panel.grid.major.y    = ggplot2::element_blank(),
      panel.grid.major.x    = ggplot2::element_blank(),
      panel.grid.minor      = ggplot2::element_blank(),
      strip.background      = ggplot2::element_blank(),
      strip.text.y          = ggplot2::element_blank(),
      strip.text.y.left     = ggplot2::element_blank(),
      strip.text.y.right    = ggplot2::element_blank(),
      strip.placement       = "outside",
      plot.margin           = plot_margin
    )
  
  if (isTRUE(has_groupings)) {
    p <- p + ggplot2::facet_wrap(
      ggplot2::vars(grouping_panel),
      ncol = 1, scales = "free_y",
      strip.position = grouping_strip_position
    )
  }
  
  p
}

# ─── Final assembly ──────────────────────────────────────────────────────────

#' Combine a forest plot with one table on either side.
#'
#' The key idea: both table panels are given the same patchwork width (the max
#' of the two measured content widths).  This guarantees the forest plot is
#' centred and both sides have equal spacing, with no per-side padding logic.
#'
#' @param plot The forest plot (ggplot2 object).
#' @param left_table Left-side table plot, or NULL if none.
#' @param right_table Right-side table plot, or NULL if none.
#' @param left_spec Left table_spec (needs `content_width`), or NULL.
#' @param right_spec Right table_spec (needs `content_width`), or NULL.
#' @param plot_width Width ratio for the forest plot panel.
#' @keywords internal
#' @noRd
combine_split_forest_plot <- function(plot,
                                      left_table = NULL,
                                      right_table = NULL,
                                      left_spec = NULL,
                                      right_spec = NULL,
                                      plot_width = 2.5) {
  left_w  <- if (!is.null(left_spec))  left_spec$content_width  else 0
  right_w <- if (!is.null(right_spec)) right_spec$content_width else 0
  
  panels <- list()
  widths <- numeric()
  
  if (!is.null(left_table)) {
    panels <- c(panels, list(left_table))
    widths <- c(widths, left_w)
  }
  
  panels <- c(panels, list(plot))
  widths <- c(widths, plot_width)
  
  if (!is.null(right_table)) {
    panels <- c(panels, list(right_table))
    widths <- c(widths, right_w)
  }
  
  patchwork::wrap_plots(panels, nrow = 1, widths = widths)
}

#' Convenience wrapper for single-table layouts (table on one side only).
#' @keywords internal
#' @noRd
combine_forest_plot_and_table <- function(plot, table_plot,
                                          table_position = c("left", "right"),
                                          table_width = 2.2,
                                          plot_width = 2.4) {
  table_position <- match.arg(table_position)
  widths <- c(table_width, plot_width)
  
  if (table_position == "left") {
    patchwork::wrap_plots(table_plot, plot, nrow = 1, widths = widths)
  } else {
    patchwork::wrap_plots(plot, table_plot, nrow = 1, widths = rev(widths))
  }
}

test_that("add_forest_table validates N table requests", {
  raw <- data.frame(
    term = "Treatment",
    estimate = 1.2,
    conf.low = 0.9,
    conf.high = 1.6
  )

  expect_error(
    add_forest_table(ggforestplot(raw), position = "left", columns = "n"),
    "requires an `n` column"
  )
})

test_that("deprecated ggforestplot facet arguments warn", {
  raw <- make_simple_forest_data(
    section = c("Clinical", "Tumor")
  )

  expect_warning(
    ggforestplot(raw, grouping = "section"),
    "`grouping` is deprecated"
  )
  expect_warning(
    ggforestplot(raw, facet = "section", grouping_strip_position = "right"),
    "`grouping_strip_position` is deprecated"
  )
  expect_error(
    ggforestplot(raw, facet = "section", grouping = "section"),
    "Use only one of"
  )
  expect_error(
    ggforestplot(raw, facet_strip_position = "right", grouping_strip_position = "right"),
    "Use only one of"
  )
})

test_that("deprecated ggforestplot line_size argument warns", {
  raw <- make_simple_forest_data()

  expect_warning(
    ggforestplot(raw, line_size = 0.8),
    "`line_size` is deprecated"
  )
  expect_error(
    ggforestplot(raw, linewidth = 0.8, line_size = 0.6),
    "Use only one of"
  )
})

test_that("table helpers use stripe alpha from plots and overrides", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6)
  )

  p <- ggforestplot(raw, striped_rows = TRUE, stripe_alpha = 0.35)
  table_out <- add_forest_table(p, position = "left")
  table_plot <- table_out$patches$plots[[1]]
  split_out <- add_split_table(p, stripe_alpha = 0.6)
  left_table <- split_out$patches$plots[[1]]

  expect_true(all(ggplot2::ggplot_build(table_plot)$data[[1]]$alpha == 0.35))
  expect_true(all(ggplot2::ggplot_build(left_table)$data[[1]]$alpha == 0.6))
})

test_that("add_forest_table requires a ggforestplot object", {
  raw <- data.frame(x = 1:2, y = 1:2)
  p <- ggplot2::ggplot(raw, ggplot2::aes(x, y)) + ggplot2::geom_point()

  expect_error(
    add_forest_table(p),
    "must be created by"
  )
})

test_that("forest tables inherit y-axis order from the plot scale", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6)
  )

  p <- suppressMessages(
    ggforestplot(raw) +
      ggplot2::scale_y_discrete(limits = c("Treatment", "Age"))
  )
  aligned_state <- align_forest_state_to_plot_y_scale(p$ggforestplotR_state, p)
  table_spec <- build_forest_table_data(aligned_state$forest_data)
  out <- p + add_forest_table()
  table_plot <- out$patches$plots[[1]]

  expect_equal(
    levels(aligned_state$forest_data$row_key),
    c("Treatment", "Age")
  )
  expect_equal(
    levels(table_spec$table_data$row_key),
    c("Treatment", "Age")
  )
  expect_false(any(table_spec$table_data$text == "BMI"))
  expect_equal(levels(table_plot$data$row_key), c("Treatment", "Age"))
})

test_that("ggforestplot can draw striped rows on exponentiated plots", {
  raw <- data.frame(
    term = c("Treatment", "Biomarker"),
    estimate = c(1.2, 0.8),
    conf.low = c(0.9, 0.6),
    conf.high = c(1.6, 1.1)
  )

  expect_no_warning({
    built <- ggplot2::ggplot_build(
      ggforestplot(raw, exponentiate = TRUE, striped_rows = TRUE)
    )
  })

  stripe_layers <- Filter(function(x) all(c("xmin", "xmax", "ymin", "ymax") %in% names(x)), built$data)

  expect_true(length(stripe_layers) >= 1L)
  expect_true(all(is.finite(stripe_layers[[1]]$xmin)))
  expect_true(all(is.finite(stripe_layers[[1]]$xmax)))

  p <- ggforestplot(raw, exponentiate = TRUE, striped_rows = TRUE)
  expected_limits <- default_plot_background_limits(
    p$ggforestplotR_state$forest_data,
    exponentiate = TRUE,
    include_zero = TRUE
  )

  expect_equal(p$scales$get_scales("x")$limits, log10(expected_limits))

  p_custom <- suppressMessages(
    ggforestplot(raw, exponentiate = TRUE, striped_rows = TRUE) +
      ggplot2::scale_x_log10(limits = c(0.5, 2), breaks = c(0.5, 1, 2))
  )
  custom_stripe_data <- p_custom$layers[[p_custom$ggforestplotR_state$stripe_layer_index]]$data

  expect_equal(p_custom$scales$get_scales("x")$limits, log10(c(0.5, 2)))
  expect_equal(p_custom$scales$get_scales("x")$breaks, c(0.5, 1, 2))
  expect_equal(unique(custom_stripe_data$xmin), 0.5)
  expect_equal(unique(custom_stripe_data$xmax), 2)

  p_partial <- suppressMessages(
    p + ggplot2::scale_x_log10(limits = c(NA, 2))
  )
  partial_stripe_data <- p_partial$layers[[p_partial$ggforestplotR_state$stripe_layer_index]]$data

  expect_equal(p_partial$scales$get_scales("x")$limits[2], log10(2))
  expect_equal(partial_stripe_data$xmin, p$layers[[p$ggforestplotR_state$stripe_layer_index]]$data$xmin)
  expect_equal(partial_stripe_data$xmax, 2)

  p_breaks <- suppressMessages(
    p + ggplot2::scale_x_log10(breaks = c(0.5, 1, 2))
  )

  expect_no_warning(
    ggplot2::ggplot_build(p_breaks)
  )
})

test_that("ggforestplot truncates confidence intervals on exponentiated plots", {
  raw <- data.frame(
    term = c("Treatment", "Biomarker"),
    estimate = c(1.2, 0.8),
    conf.low = c(0.2, 0.6),
    conf.high = c(4.0, 1.1)
  )

  p <- ggforestplot(raw, exponentiate = TRUE, ci_limits = c(0.5, 2))
  built <- ggplot2::ggplot_build(p)
  errorbar_layers <- Filter(
    function(layer) all(c("xmin", "xmax", "width") %in% names(layer)),
    built$data
  )
  truncated_interval <- Filter(
    function(layer) all(layer$width == 0),
    errorbar_layers
  )[[1]]
  complete_interval <- Filter(
    function(layer) all(layer$width == 0.2) && any(layer$xmin != layer$xmax),
    errorbar_layers
  )[[1]]

  expect_equal(p$scales$get_scales("x")$limits, log10(c(0.5, 2)))
  expect_equal(truncated_interval$xmin, log10(0.5))
  expect_equal(truncated_interval$xmax, log10(2.0))
  expect_equal(complete_interval$xmin, log10(0.6))
  expect_equal(complete_interval$xmax, log10(1.1))
})

test_that("ggforestplot allows facet strip labels on the right", {
  raw <- data.frame(
    term = c("Age", "BMI", "Stage II", "Stage III"),
    estimate = c(0.3, -0.2, 0.5, 0.8),
    conf.low = c(0.1, -0.4, 0.2, 0.4),
    conf.high = c(0.5, 0.0, 0.8, 1.2),
    section = c("Clinical", "Clinical", "Tumor", "Tumor")
  )

  p <- ggforestplot(raw, facet = "section", facet_strip_position = "right")
  table_spec <- build_forest_table_data(p$ggforestplotR_state$forest_data)
  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = p$ggforestplotR_state$stripe_data,
    has_groupings = p$ggforestplotR_state$has_groupings,
    grouping_strip_position = p$ggforestplotR_state$facet_strip_position
  )

  expect_equal(p$facet$params$strip.position, "right")
  expect_equal(p$ggforestplotR_state$facet_strip_position, "right")
  expect_equal(table_plot$facet$params$strip.position, "right")
})

test_that("ggforestplot can sort terms with grouped sections", {
  raw <- data.frame(
    term = c("Age", "BMI", "Stage II", "Stage III"),
    estimate = c(0.3, -0.2, 0.5, 0.8),
    conf.low = c(0.1, -0.4, 0.2, 0.4),
    conf.high = c(0.5, 0.0, 0.8, 1.2),
    section = c("Clinical", "Clinical", "Tumor", "Tumor")
  )

  p <- ggforestplot(
    raw,
    facet = "section",
    striped_rows = TRUE,
    stripe_fill = "grey94",
    facet_strip_position = "right",
    sort_terms = "descending"
  )

  expect_equal(
    as.character(p$ggforestplotR_state$forest_data$term),
    c("Age", "BMI", "Stage III", "Stage II")
  )
  expect_equal(p$facet$params$strip.position, "right")
})

test_that("forest table centers the Term header and text", {
  raw <- make_simple_forest_data()

  p <- ggforestplot(raw)
  table_spec <- layout_center_table_spec(
    build_forest_table_data(p$ggforestplotR_state$forest_data)
  )
  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = p$ggforestplotR_state$stripe_data
  )

  expect_equal(table_spec$header_positions[1], table_spec$positions[1])
  expect_equal(table_plot$layers[[1]]$aes_params$hjust, 0.5)
  expect_equal(table_plot$theme$axis.text.x.top$hjust, 0.5)
})

test_that("forest table supports header size and font customization", {
  raw <- make_simple_forest_data()

  p <- ggforestplot(raw)
  out <- add_forest_table(
    p,
    position = "left",
    header_text_size = 14,
    header_fontface = "italic",
    header_family = "mono"
  )

  table_plot <- out$patches$plots[[1]]

  expect_equal(table_plot$theme$axis.text.x.top$size, 14)
  expect_equal(table_plot$theme$axis.text.x.top$face, "italic")
  expect_equal(table_plot$theme$axis.text.x.top$family, "mono")
})

test_that("add_forest_table supports explicit side-table column order", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    term_header = "Term",
    n_header = "N",
    events_header = "Events",
    estimate_label = "Beta",
    p_header = "P-value",
    columns = c("n", "events", "term", "estimate", "p")
  )

  expect_equal(table_spec$column_keys, c("n", "events", "term", "estimate", "p"))
  expect_equal(table_spec$headers, c("N", "Events", "Term", "Beta (95% CI)", "P-value"))
  expect_s3_class(
    add_forest_table(
      p,
      position = "left",
      columns = c("n", "events", "term", "estimate", "p")
    ),
    "patchwork"
  )
})

test_that("mapped row-label columns use the term header and suppress duplicate y-axis labels", {
  raw <- data.frame(
    name = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6)
  )

  p <- suppressMessages(
    ggforestplot(raw, term = "name") +
      ggplot2::scale_y_discrete(limits = c("Treatment", "BMI", "Age"))
  )
  out <- p + add_forest_table(
    columns = c("name", "estimate"),
    term_header = "Variable"
  )
  table_plot <- out$patches$plots[[1]]

  expect_s3_class(out$theme$axis.text.y, "element_blank")
  expect_s3_class(out$theme$axis.ticks.y, "element_blank")
  expect_equal(
    table_plot$scales$get_scales("x")$labels,
    c("Variable", "Estimate (95% CI)")
  )
  expect_equal(
    out$scales$get_scales("y")$limits,
    c("Treatment", "BMI", "Age")
  )
})

test_that("ggforestplot relabels terms with a named vector", {
  raw <- data.frame(
    term = c("age", "bmi", "treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6)
  )

  p <- ggforestplot(
    raw,
    term_labels = c(age = "Age, years", treatment = "Treatment arm")
  )

  expect_equal(
    as.character(p$ggforestplotR_state$forest_data$label),
    c("Age, years", "bmi", "Treatment arm")
  )
})

test_that("ggforestplot supports reference line naming and values", {
  raw <- make_simple_forest_data()

  p <- ggforestplot(
    raw,
    ref_line = 0.25,
    ref_label = "Null",
    ref_linetype = 3,
    ref_color = "red"
  )
  built <- ggplot2::ggplot_build(p)
  vline_layers <- Filter(function(x) "xintercept" %in% names(x), built$data)
  label_layers <- Filter(function(x) "label" %in% names(x) && any(x$label == "Null"), built$data)

  expect_equal(vline_layers[[1]]$xintercept, 0.25)
  expect_equal(vline_layers[[1]]$linetype, 3)
  expect_equal(vline_layers[[1]]$colour, "red")
  expect_equal(label_layers[[1]]$label, "Null")
  expect_equal(p$ggforestplotR_state$defaults$ref_line, 0.25)

  hidden <- ggplot2::ggplot_build(ggforestplot(raw, ref_line = NULL))
  hidden_vline_layers <- Filter(function(x) "xintercept" %in% names(x), hidden$data)

  expect_length(hidden_vline_layers, 0L)
  expect_error(
    ggforestplot(raw, ref_line = "Null"),
    "`ref_line` must be a single numeric value or `NULL`."
  )
})

test_that("add_forest_table supports arbitrary preserved columns", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.345, -0.234),
    conf.low = c(0.12, -0.43),
    conf.high = c(0.57, -0.04),
    upper_bound = c(0.57, -0.04),
    adjustment = c("Clinical", "Clinical")
  )

  p <- ggforestplot(raw)
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("term", "adjustment", "upper_bound", "estimate")
  )

  expect_equal(table_spec$column_keys, c("term", "adjustment", "upper_bound", "estimate"))
  expect_equal(table_spec$headers, c("Term", "adjustment", "upper_bound", "Estimate (95% CI)"))
  expect_true(any(table_spec$table_data$column_key == "adjustment"))
  expect_true(any(table_spec$table_data$text == "Clinical"))
  expect_true(any(table_spec$table_data$column_key == "upper_bound"))
  expect_true(any(table_spec$table_data$text == "0.57"))
  positioned_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c(1, 5, 6)
  )

  expect_equal(positioned_spec$column_keys, c("term", "upper_bound", "adjustment"))
  expect_s3_class(
    add_forest_table(p, columns = c("term", "upper_bound", "adjustment")),
    "patchwork"
  )
  expect_s3_class(
    add_forest_table(p, columns = c(1, 5, 6)),
    "patchwork"
  )
})

test_that("forest table supports custom column labels", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.345, -0.234),
    conf.low = c(0.12, -0.43),
    conf.high = c(0.57, -0.04),
    adjustment = c("Clinical", "Demographic"),
    p_value = c(0.012, 0.031)
  )

  p <- ggforestplot(raw, p.value = "p_value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("term", "adjustment", "estimate", "p"),
    column_labels = c(
      term = "Variable",
      adjustment = "Adjusted for",
      estimate = "Beta (95% CI)",
      p = "P"
    )
  )

  expect_equal(table_spec$headers, c("Variable", "Adjusted for", "Beta (95% CI)", "P"))
  expect_s3_class(
    add_forest_table(
      p,
      columns = c("term", "adjustment", "estimate", "p"),
      column_labels = c(adjustment = "Adjusted for", estimate = "Beta (95% CI)")
    ),
    "patchwork"
  )
})

test_that("forest table validates column labels", {
  raw <- data.frame(
    term = "Age",
    estimate = 0.345,
    conf.low = 0.12,
    conf.high = 0.57
  )

  p <- ggforestplot(raw)

  expect_error(
    build_forest_table_data(
      p$ggforestplotR_state$forest_data,
      columns = "term",
      column_labels = c("Variable")
    ),
    "`column_labels` must be a named vector."
  )
  expect_error(
    build_forest_table_data(
      p$ggforestplotR_state$forest_data,
      columns = "term",
      column_labels = c(missing_column = "Missing")
    ),
    "Unsupported table columns"
  )
})

test_that("forest table columns can use original dataframe vectors", {
  raw <- data.frame(
    variable = c("Age", "BMI"),
    beta = c(0.345, -0.234),
    lower = c(0.12, -0.43),
    upper = c(0.57, -0.04),
    adjustment = factor(c("Clinical", "Demographic")),
    review_date = as.Date(c("2026-01-15", "2026-02-20")),
    selected = c(TRUE, FALSE),
    group = c("Reviewer A", "Reviewer B")
  )

  p <- ggforestplot(
    raw,
    term = "variable",
    estimate = "beta",
    conf.low = "lower",
    conf.high = "upper"
  )
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("variable", "beta", "lower", "upper", "adjustment", "review_date", "selected", "group")
  )

  expect_equal(
    table_spec$column_keys,
    c("variable", "beta", "lower", "upper", "adjustment", "review_date", "selected", "group")
  )
  expect_true(any(table_spec$table_data$text == "Age"))
  expect_true(any(table_spec$table_data$text == "0.345"))
  expect_true(any(table_spec$table_data$text == "0.12"))
  expect_true(any(table_spec$table_data$text == "0.57"))
  expect_true(any(table_spec$table_data$text == "Clinical"))
  expect_true(any(table_spec$table_data$text == "2026-01-15"))
  expect_true(any(table_spec$table_data$text == "TRUE"))
  expect_true(any(table_spec$table_data$text == "Reviewer A"))
  expect_s3_class(
    add_forest_table(
      p,
      columns = c("variable", "beta", "lower", "upper", "adjustment", "review_date", "selected", "group")
    ),
    "patchwork"
  )
})

test_that("forest table formats estimates, intervals, and p-values separately", {
  raw <- data.frame(
    term = "Age",
    estimate = 0.3456,
    conf.low = 0.1234,
    conf.high = 0.5678,
    p_value = 0.01234
  )

  p <- ggforestplot(raw, p.value = "p_value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("estimate", "p"),
    estimate_digits = 1,
    interval_digits = 3,
    p_digits = 4
  )

  expect_true(any(table_spec$table_data$text == "0.3 (0.123, 0.568)"))
  expect_true(any(table_spec$table_data$text == "0.0123"))
})

test_that("forest table supports custom estimate format strings", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3456, -0.2345),
    conf.low = c(0.1234, -0.4321),
    conf.high = c(0.5678, -0.0432)
  )

  p <- ggforestplot(raw)
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = "estimate",
    estimate_digits = 1,
    interval_digits = 3,
    estimate_fmt = "{estimate} [{conf.low}, {conf.high}]"
  )
  shorthand_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = "estimate",
    estimate_digits = 1,
    interval_digits = 3,
    estimate_fmt = "{estimate} ({conf.low, conf.high})"
  )

  expect_true(any(table_spec$table_data$text == "0.3 [0.123, 0.568]"))
  expect_true(any(shorthand_spec$table_data$text == "0.3 (0.123, 0.568)"))
  expect_s3_class(
    add_forest_table(
      p,
      columns = "estimate",
      estimate_digits = 1,
      interval_digits = 3,
      estimate_fmt = "{estimate} [{conf.low}, {conf.high}]"
    ),
    "patchwork"
  )
})

test_that("forest table can split estimates and confidence intervals", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3456, -0.2345),
    conf.low = c(0.1234, -0.4321),
    conf.high = c(0.5678, -0.0432)
  )

  p <- ggforestplot(raw)
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("estimate", "ci"),
    estimate_digits = 1,
    interval_digits = 3
  )
  custom_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("estimate", "ci"),
    estimate_digits = 1,
    interval_digits = 3,
    ci_fmt = "{conf.low} to {conf.high}"
  )

  expect_equal(table_spec$column_keys, c("estimate", "ci"))
  expect_equal(table_spec$headers, c("Estimate", "95% CI"))
  expect_true(any(table_spec$table_data$column_key == "estimate" & table_spec$table_data$text == "0.3"))
  expect_true(any(table_spec$table_data$column_key == "ci" & table_spec$table_data$text == "(0.123, 0.568)"))
  expect_true(any(custom_spec$table_data$column_key == "ci" & custom_spec$table_data$text == "0.123 to 0.568"))
  expect_s3_class(
    add_forest_table(
      p,
      columns = c("term", "estimate", "ci"),
      estimate_digits = 1,
      interval_digits = 3,
      column_labels = c(estimate = "Beta", ci = "95% CI")
    ),
    "patchwork"
  )
})

test_that("confidence bound column names alias to the CI table column", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3456, -0.2345),
    conf.low = c(0.1234, -0.4321),
    conf.high = c(0.5678, -0.0432)
  )

  p <- ggforestplot(raw)
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("estimate", "conf.low", "conf.high"),
    estimate_digits = 1,
    interval_digits = 3,
    column_labels = c(conf.high = "CI")
  )

  expect_equal(table_spec$column_keys, c("estimate", "ci"))
  expect_equal(table_spec$headers, c("Estimate", "CI"))
  expect_true(any(table_spec$table_data$column_key == "estimate" & table_spec$table_data$text == "0.3"))
  expect_true(any(table_spec$table_data$column_key == "ci" & table_spec$table_data$text == "(0.123, 0.568)"))
  expect_s3_class(
    add_forest_table(
      p,
      columns = c("term", "estimate", "conf.low", "conf.high"),
      column_labels = c(conf.high = "CI")
    ),
    "patchwork"
  )
})

test_that("forest table validates confidence interval format strings", {
  raw <- data.frame(
    term = "Age",
    estimate = 0.345,
    conf.low = 0.12,
    conf.high = 0.57
  )

  p <- ggforestplot(raw)

  expect_error(
    build_forest_table_data(
      p$ggforestplotR_state$forest_data,
      columns = "ci",
      ci_fmt = c("{conf.low}", "{conf.high}")
    ),
    "`ci_fmt` must be a single character string."
  )
})

test_that("add_forest_table can show an events column", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    event_count = c(42, 39, 31)
  )

  p <- ggforestplot(raw, events = "event_count")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("term", "events", "estimate")
  )

  expect_true(any(table_spec$column_keys == "events"))
  expect_true(any(table_spec$table_data$column_key == "events"))
  expect_true(any(table_spec$table_data$text == "42"))
})

test_that("add_forest_table validates events table requests", {
  raw <- data.frame(
    term = "Treatment",
    estimate = 1.2,
    conf.low = 0.9,
    conf.high = 1.6
  )

  expect_error(
    add_forest_table(ggforestplot(raw), position = "left", columns = "events"),
    "requires an `events` column"
  )
})

test_that("forest table can draw horizontal separator lines only", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98)
  )

  p <- ggforestplot(raw, n = "sample_size")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("term", "n", "estimate")
  )
  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = p$ggforestplotR_state$stripe_data,
    grid_lines = TRUE,
    grid_line_size = 0.4,
    grid_line_linetype = 2
  )

  built <- ggplot2::ggplot_build(table_plot)
  hline_layers <- Filter(function(x) "yintercept" %in% names(x), built$data)
  segment_layers <- Filter(function(x) all(c("x", "xend", "y", "yend") %in% names(x)), built$data)
  vline_layers <- Filter(function(x) "xintercept" %in% names(x), built$data)

  expect_equal(length(hline_layers), 1L)
  expect_equal(length(segment_layers), 0L)
  expect_equal(length(vline_layers), 0L)
  expect_equal(hline_layers[[1]]$yintercept, c(0.5, 1.5, 2.5, 3.5))
  expect_true(all(hline_layers[[1]]$colour == "black"))
  expect_true(all(hline_layers[[1]]$linetype == 2))
})






test_that("add_forest_table can show p-values to the right of estimates", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, p.value = "p_value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("term", "estimate", "p"),
    estimate_label = "Beta"
  )

  expect_equal(table_spec$headers, c("Term", "Beta (95% CI)", "P-value"))
  expect_true(any(table_spec$table_data$column_key == "p"))
  expect_true(any(grepl("0\\.012", table_spec$table_data$text, fixed = FALSE)))
})

test_that("add_forest_table validates p-value table requests", {
  raw <- data.frame(
    term = "Treatment",
    estimate = 1.2,
    conf.low = 0.9,
    conf.high = 1.6
  )

  expect_error(
    add_forest_table(ggforestplot(raw), position = "left", columns = "p"),
    "requires a `p.value` column"
  )
})

test_that("deprecated table digit argument warns", {
  raw <- make_simple_forest_data(
    p_value = c(0.012, 0.031)
  )

  p <- ggforestplot(raw, p.value = "p_value")

  expect_warning(
    add_forest_table(p, digits = 3),
    "`digits` is deprecated"
  )
  expect_warning(
    add_split_table(p, digits = 3),
    "`digits` is deprecated"
  )
})

test_that("add_split_table requires left and right table columns", {
  raw <- make_simple_forest_data(
    sample_size = c(120, 115)
  )

  p <- ggforestplot(raw, n = "sample_size")

  expect_error(
    add_split_table(p, left_columns = character()),
    "left-side column"
  )

  expect_error(
    add_split_table(p, right_columns = character()),
    "right-side column"
  )
})

test_that("add_split_table accepts explicit left and right columns by name", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(
      left_columns = c("term", "n", "events"),
      right_columns = c("estimate", "p")
    )

  expect_s3_class(out, "patchwork")

  out_with_preserved_columns <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(
      left_columns = c(1, 5, 6),
      right_columns = c(2, 7)
    )

  expect_s3_class(out_with_preserved_columns, "patchwork")
})

test_that("add_split_table supports custom column labels", {
  raw <- make_simple_forest_data(
    sample_size = c(120, 115),
    adjustment = c("Clinical", "Demographic")
  )

  p <- ggforestplot(raw, n = "sample_size")
  state <- p$ggforestplotR_state
  left_spec <- build_forest_table_data(
    state$forest_data,
    columns = c("term", "adjustment"),
    column_labels = c(term = "Variable", adjustment = "Adjusted for")
  )
  right_spec <- build_forest_table_data(
    state$forest_data,
    columns = "estimate",
    column_labels = c(estimate = "Beta (95% CI)")
  )

  expect_equal(left_spec$headers, c("Variable", "Adjusted for"))
  expect_equal(right_spec$headers, "Beta (95% CI)")
  expect_s3_class(
    add_split_table(
      p,
      left_columns = c("term", "adjustment"),
      right_columns = "estimate",
      column_labels = c(term = "Variable", adjustment = "Adjusted for", estimate = "Beta (95% CI)")
    ),
    "patchwork"
  )
})

test_that("add_split_table can split estimates and confidence intervals", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3456, -0.2345),
    conf.low = c(0.1234, -0.4321),
    conf.high = c(0.5678, -0.0432),
    sample_size = c(120, 115)
  )

  p <- ggforestplot(raw, n = "sample_size")
  state <- p$ggforestplotR_state
  right_spec <- build_forest_table_data(
    state$forest_data,
    columns = c("estimate", "ci"),
    estimate_digits = 1,
    interval_digits = 3,
    ci_fmt = "{conf.low} to {conf.high}"
  )

  expect_equal(right_spec$headers, c("Estimate", "95% CI"))
  expect_true(any(right_spec$table_data$column_key == "estimate" & right_spec$table_data$text == "0.3"))
  expect_true(any(right_spec$table_data$column_key == "ci" & right_spec$table_data$text == "0.123 to 0.568"))
  expect_s3_class(
    add_split_table(
      p,
      left_columns = c("term", "n"),
      right_columns = c("estimate", "ci"),
      estimate_digits = 1,
      interval_digits = 3,
      ci_fmt = "{conf.low} to {conf.high}",
      column_labels = c(estimate = "Beta", ci = "95% CI")
    ),
    "patchwork"
  )
})

test_that("add_split_table accepts confidence bound names as CI aliases", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3456, -0.2345),
    conf.low = c(0.1234, -0.4321),
    conf.high = c(0.5678, -0.0432),
    sample_size = c(120, 115)
  )

  p <- ggforestplot(raw, n = "sample_size")
  right_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    columns = c("estimate", "conf.low", "conf.high"),
    estimate_digits = 1,
    interval_digits = 3
  )

  expect_equal(right_spec$column_keys, c("estimate", "ci"))
  expect_true(any(right_spec$table_data$column_key == "ci" & right_spec$table_data$text == "(0.123, 0.568)"))
  expect_s3_class(
    add_split_table(
      p,
      left_columns = c("term", "n"),
      right_columns = c("estimate", "conf.low", "conf.high"),
      estimate_digits = 1,
      interval_digits = 3
    ),
    "patchwork"
  )
})

test_that("add_split_table accepts explicit left and right columns by position", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(
      left_columns = c(1, 5, 6),
      right_columns = c(2, 7)
    )

  expect_s3_class(out, "patchwork")
})

test_that("add_split_table removes panel border and keeps x-axis line", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out <- add_split_table(
    ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") + ggplot2::labs(title = "Split"),
    left_columns = c("term", "n", "events"),
    right_columns = c("estimate", "p")
  )

  center_plot <- out$patches$plots[[2]]

  expect_s3_class(center_plot$theme$panel.border, "element_blank")
  expect_s3_class(center_plot$theme$axis.line.x, "element_line")
  expect_s3_class(center_plot$theme$axis.line.y, "element_blank")
  expect_s3_class(center_plot$theme$panel.grid.major, "element_blank")
  expect_s3_class(center_plot$theme$panel.grid.minor, "element_blank")
})

test_that("add_split_table uses split-specific alignment and no grid lines", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value")
  state <- p$ggforestplotR_state
  left_spec <- build_forest_table_data(state$forest_data, columns = c("term", "n", "events"))
  right_spec <- build_forest_table_data(state$forest_data, columns = c("estimate", "p"))
  left_spec <- layout_split_table_spec(left_spec, text_size = 3.2, alignment = "left")
  right_spec <- layout_split_table_spec(right_spec, text_size = 3.2, alignment = "right")
  left_plot <- build_forest_table_plot(
    table_spec = left_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "left",
    striped_rows = TRUE,
    grid_lines = FALSE,
    x_expand = c(0.05, 0.35),
    plot_margin = ggplot2::margin(5.5, 0, 5.5, 5.5),
    text_hjust = 0,
    header_hjust = 0
  )
  right_plot <- build_forest_table_plot(
    table_spec = right_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "right",
    striped_rows = TRUE,
    grid_lines = FALSE,
    x_expand = c(0.1, 0.35),
    plot_margin = ggplot2::margin(5.5, 5.5, 5.5, 0),
    text_hjust = 1,
    header_hjust = 1
  )

  left_built <- ggplot2::ggplot_build(left_plot)
  right_built <- ggplot2::ggplot_build(right_plot)

  expect_equal(left_plot$layers[[2]]$aes_params$hjust, 0)
  expect_equal(right_plot$layers[[2]]$aes_params$hjust, 1)
  expect_equal(left_plot$theme$axis.text.x.top$hjust, 0)
  expect_equal(right_plot$theme$axis.text.x.top$hjust, 1)
  expect_length(Filter(function(x) "yintercept" %in% names(x), left_built$data), 0L)
  expect_length(Filter(function(x) "yintercept" %in% names(x), right_built$data), 0L)
})

test_that("add_split_table supports header size and font customization", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value")
  state <- p$ggforestplotR_state
  left_spec <- build_forest_table_data(state$forest_data, columns = c("term", "n", "events"))
  right_spec <- build_forest_table_data(state$forest_data, columns = c("estimate", "p"))
  left_spec <- layout_split_table_spec(
    left_spec,
    text_size = 3.2,
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif",
    alignment = "left"
  )
  right_spec <- layout_split_table_spec(
    right_spec,
    text_size = 3.2,
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif",
    alignment = "right"
  )
  left_plot <- build_forest_table_plot(
    table_spec = left_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "left",
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif"
  )
  right_plot <- build_forest_table_plot(
    table_spec = right_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "right",
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif"
  )

  expect_equal(left_plot$theme$axis.text.x.top$size, 13)
  expect_equal(left_plot$theme$axis.text.x.top$face, "plain")
  expect_equal(left_plot$theme$axis.text.x.top$family, "serif")
  expect_equal(right_plot$theme$axis.text.x.top$size, 13)
  expect_equal(right_plot$theme$axis.text.x.top$face, "plain")
  expect_equal(right_plot$theme$axis.text.x.top$family, "serif")
})


test_that("add_split_table sizes panels from split column counts", {
  raw <- data.frame(
    term = c("Very long predictor name", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out_equal <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = c("term", "events"), right_columns = c("estimate", "p"))
  widths_equal <- out_equal$patches$layout$widths

  out_unequal <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = "term", right_columns = c("estimate", "p"))
  widths_unequal <- out_unequal$patches$layout$widths

  out_three <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = c("term", "n", "events"), right_columns = "estimate")
  widths_three <- out_three$patches$layout$widths

  expect_equal(widths_equal, c(2.5, 2.5, 2.5))
  expect_equal(widths_unequal, c(1.25, 2.5, 2.5))
  expect_equal(widths_three, c(2.5 * (4 / 3), 2.5, 1.25))
  expect_equal(split_table_width_multiplier(1), 0.5)
  expect_equal(split_table_width_multiplier(2), 1)
  expect_equal(split_table_width_multiplier(3), 4 / 3)
})

test_that("add_split_table validates events requests", {
  raw <- make_simple_forest_data()

  p <- ggforestplot(raw)

  expect_error(
    add_split_table(p, left_columns = c("term", "events"), right_columns = "estimate"),
    "An `events` column is required"
  )
})

test_that("ggforestplot defaults staple width to 0.2", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3, -0.2),
    conf.low = c(0.1, -0.4),
    conf.high = c(0.5, 0.0)
  )

  built <- ggplot2::ggplot_build(ggforestplot(raw))

  expect_true(all(built$data[[1]]$width == 0.2))
})

test_that("ggforestplot can facet grouped rows and add stripes", {
  raw <- data.frame(
    term = c("Age", "BMI", "Smoking", "Stage II", "Stage III", "Nodes"),
    estimate = c(0.3, -0.2, 0.4, 0.5, 0.8, 0.4),
    conf.low = c(0.1, -0.4, 0.2, 0.2, 0.4, 0.1),
    conf.high = c(0.5, 0.0, 0.6, 0.8, 1.2, 0.7),
    section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor", "Tumor")
  )

  p <- ggforestplot(raw, grouping = "section", striped_rows = TRUE)
  built <- ggplot2::ggplot_build(p)
  panel_rows <- lapply(split(as.numeric(built$data[[2]]$y), built$data[[2]]$PANEL), unique)

  expect_s3_class(p, "ggplot")
  expect_equal(nrow(built$data[[1]]), 4L)
  expect_equal(length(panel_rows), 2L)
  expect_equal(unname(panel_rows[[1]]), c(1, 2, 3))
  expect_equal(unname(panel_rows[[2]]), c(1, 2, 3))
})

test_that("ggforestplot supports shape and staple width controls", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3, -0.2),
    conf.low = c(0.1, -0.4),
    conf.high = c(0.5, 0.0)
  )

  built <- ggplot2::ggplot_build(
    ggforestplot(raw, point_shape = 17, staple_width = 0.25)
  )

  expect_true(all(built$data[[2]]$shape == 17))
  expect_true(all(built$data[[1]]$width == 0.25))
})

test_that("ggforestplot can draw separator lines for each labeled variable block", {
  raw <- data.frame(
    term = c("race_black", "race_white", "race_other", "age"),
    label = c("Black", "White", "Other", "Age"),
    estimate = c(0.3, 0.1, 0.05, -0.2),
    conf.low = c(0.1, -0.1, -0.15, -0.4),
    conf.high = c(0.5, 0.3, 0.25, 0.0),
    block = c("Race", "Race", "Race", "Age")
  )

  built <- ggplot2::ggplot_build(
    ggforestplot(
      raw,
      label = "label",
      separate_groups = "block",
      separate_lines = TRUE,
      zero_line = FALSE
    )
  )

  line_layers <- Filter(function(x) "yintercept" %in% names(x), built$data)

  expect_length(line_layers, 1L)
  expect_equal(line_layers[[1]]$yintercept, c(0.5, 3.5, 4.5))
  expect_true(all(line_layers[[1]]$linetype == 2))
  expect_true(all(line_layers[[1]]$colour == "black"))
})
test_that("ggforestplot prefixes labels for multi-level separator groups", {
  raw <- data.frame(
    term = c("race_black", "race_white", "race_other", "age"),
    label = c("Black", "White", "Other", "Age"),
    estimate = c(0.3, 0.1, 0.05, -0.2),
    conf.low = c(0.1, -0.1, -0.15, -0.4),
    conf.high = c(0.5, 0.3, 0.25, 0.0),
    block = c("Race", "Race", "Race", "Age")
  )

  p <- ggforestplot(raw, label = "label", separate_groups = "block")

  expect_equal(
    as.character(p$ggforestplotR_state$forest_data$label),
    c("Race: Black", "Race: White", "Race: Other", "Age")
  )
})

test_that("add_forest_table validates N table requests", {
  raw <- data.frame(
    term = "Treatment",
    estimate = 1.2,
    conf.low = 0.9,
    conf.high = 1.6
  )

  expect_error(
    add_forest_table(ggforestplot(raw), position = "left", show_n = TRUE),
    "requires an `n` column"
  )
})

test_that("add_forest_table requires a ggforestplot object", {
  raw <- data.frame(x = 1:2, y = 1:2)
  p <- ggplot2::ggplot(raw, ggplot2::aes(x, y)) + ggplot2::geom_point()

  expect_error(
    add_forest_table(p),
    "must be created by"
  )
})

test_that("ggforestplot can draw striped rows on exponentiated plots", {
  raw <- data.frame(
    term = c("Treatment", "Biomarker"),
    estimate = c(1.2, 0.8),
    conf.low = c(0.9, 0.6),
    conf.high = c(1.6, 1.1)
  )

  expect_no_warning({
    built <- ggplot2::ggplot_build(
      ggforestplot(raw, exponentiate = TRUE, striped_rows = TRUE)
    )
  })

  stripe_layers <- Filter(function(x) all(c("xmin", "xmax", "ymin", "ymax") %in% names(x)), built$data)

  expect_true(length(stripe_layers) >= 1L)
  expect_true(all(is.finite(stripe_layers[[1]]$xmin)))
  expect_true(all(is.finite(stripe_layers[[1]]$xmax)))

  p <- ggforestplot(raw, exponentiate = TRUE, striped_rows = TRUE)
  expected_limits <- default_plot_background_limits(
    p$ggforestplotR_state$forest_data,
    exponentiate = TRUE,
    include_zero = TRUE
  )

  expect_equal(p$scales$get_scales("x")$limits, log10(expected_limits))
})

test_that("ggforestplot allows grouping strip labels on the right", {
  raw <- data.frame(
    term = c("Age", "BMI", "Stage II", "Stage III"),
    estimate = c(0.3, -0.2, 0.5, 0.8),
    conf.low = c(0.1, -0.4, 0.2, 0.4),
    conf.high = c(0.5, 0.0, 0.8, 1.2),
    section = c("Clinical", "Clinical", "Tumor", "Tumor")
  )

  p <- ggforestplot(raw, grouping = "section", grouping_strip_position = "right")
  table_spec <- build_forest_table_data(p$ggforestplotR_state$forest_data)
  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = p$ggforestplotR_state$stripe_data,
    has_groupings = p$ggforestplotR_state$has_groupings,
    grouping_strip_position = p$ggforestplotR_state$grouping_strip_position
  )

  expect_equal(p$facet$params$strip.position, "right")
  expect_equal(p$ggforestplotR_state$grouping_strip_position, "right")
  expect_equal(table_plot$facet$params$strip.position, "right")
})

test_that("forest table centers the Term header and text", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3, -0.2),
    conf.low = c(0.1, -0.4),
    conf.high = c(0.5, 0.0)
  )

  p <- ggforestplot(raw)
  table_spec <- build_forest_table_data(p$ggforestplotR_state$forest_data)
  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = p$ggforestplotR_state$stripe_data
  )

  expect_equal(table_spec$header_positions[1], table_spec$positions[1])
  expect_equal(table_plot$layers[[1]]$aes_params$hjust, 0.5)
  expect_equal(table_plot$theme$axis.text.x.top$hjust, 0.5)
})

test_that("forest table supports header size and font customization", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3, -0.2),
    conf.low = c(0.1, -0.4),
    conf.high = c(0.5, 0.0)
  )

  p <- ggforestplot(raw)
  out <- add_forest_table(
    p,
    position = "left",
    header_text_size = 14,
    header_fontface = "italic",
    header_family = "mono"
  )

  table_plot <- out$patches$plots[[1]]

  expect_equal(table_plot$theme$axis.text.x.top$size, 14)
  expect_equal(table_plot$theme$axis.text.x.top$face, "italic")
  expect_equal(table_plot$theme$axis.text.x.top$family, "mono")
})

test_that("add_forest_table supports explicit side-table column order", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    term_header = "Term",
    n_header = "N",
    events_header = "Events",
    estimate_label = "Beta",
    p_header = "P-value",
    columns = c("n", "events", "term", "estimate", "p")
  )

  expect_equal(table_spec$column_keys, c("n", "events", "term", "estimate", "p"))
  expect_equal(table_spec$headers, c("N", "Events", "Term", "Beta (95% CI)", "P-value"))
  expect_s3_class(
    add_forest_table(
      p,
      position = "left",
      columns = c("n", "events", "term", "estimate", "p")
    ),
    "patchwork"
  )
})

test_that("add_forest_table can show an events column", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    event_count = c(42, 39, 31)
  )

  p <- ggforestplot(raw, events = "event_count")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    show_events = TRUE
  )

  expect_true(any(table_spec$column_keys == "events"))
  expect_true(any(table_spec$table_data$column_key == "events"))
  expect_true(any(table_spec$table_data$text == "42"))
})

test_that("add_forest_table validates events table requests", {
  raw <- data.frame(
    term = "Treatment",
    estimate = 1.2,
    conf.low = 0.9,
    conf.high = 1.6
  )

  expect_error(
    add_forest_table(ggforestplot(raw), position = "left", show_events = TRUE),
    "requires an `events` column"
  )
})

test_that("forest table can draw horizontal separator lines only", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98)
  )

  p <- ggforestplot(raw, n = "sample_size")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    show_n = TRUE
  )
  table_plot <- build_forest_table_plot(
    table_spec = table_spec,
    stripe_data = p$ggforestplotR_state$stripe_data,
    grid_lines = TRUE,
    grid_line_size = 0.4,
    grid_line_linetype = 2
  )

  built <- ggplot2::ggplot_build(table_plot)
  hline_layers <- Filter(function(x) "yintercept" %in% names(x), built$data)
  segment_layers <- Filter(function(x) all(c("x", "xend", "y", "yend") %in% names(x)), built$data)
  vline_layers <- Filter(function(x) "xintercept" %in% names(x), built$data)

  expect_equal(length(hline_layers), 1L)
  expect_equal(length(segment_layers), 0L)
  expect_equal(length(vline_layers), 0L)
  expect_equal(hline_layers[[1]]$yintercept, c(0.5, 1.5, 2.5, 3.5))
  expect_true(all(hline_layers[[1]]$colour == "black"))
  expect_true(all(hline_layers[[1]]$linetype == 2))
})






test_that("add_forest_table can show p-values to the right of estimates", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, p.value = "p_value")
  table_spec <- build_forest_table_data(
    p$ggforestplotR_state$forest_data,
    show_p = TRUE,
    estimate_label = "Beta"
  )

  expect_equal(table_spec$headers, c("Term", "Beta (95% CI)", "P-value"))
  expect_true(any(table_spec$table_data$column_key == "p"))
  expect_true(any(grepl("0\\.012", table_spec$table_data$text, fixed = FALSE)))
})

test_that("add_forest_table validates p-value table requests", {
  raw <- data.frame(
    term = "Treatment",
    estimate = 1.2,
    conf.low = 0.9,
    conf.high = 1.6
  )

  expect_error(
    add_forest_table(ggforestplot(raw), position = "left", show_p = TRUE),
    "requires a `p.value` column"
  )
})

test_that("add_split_table requires left and right table columns", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3, -0.2),
    conf.low = c(0.1, -0.4),
    conf.high = c(0.5, 0.0),
    sample_size = c(120, 115)
  )

  p <- ggforestplot(raw, n = "sample_size")

  expect_error(
    add_split_table(p, show_terms = FALSE, show_n = FALSE),
    "left-side column"
  )

  expect_error(
    add_split_table(p, show_estimate = FALSE, show_p = FALSE),
    "right-side column"
  )
})

test_that("add_split_table accepts explicit left and right columns by name", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(
      left_columns = c("term", "n", "events"),
      right_columns = c("estimate", "p")
    )

  expect_s3_class(out, "patchwork")
  expect_s3_class(out, "ggplot")
})

test_that("add_split_table accepts explicit left and right columns by position", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(
      left_columns = c(1, 2, 3),
      right_columns = c(4, 5)
    )

  expect_s3_class(out, "patchwork")
  expect_s3_class(out, "ggplot")
})

test_that("add_split_table removes panel border and keeps x-axis line", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out <- add_split_table(
    ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") + ggplot2::labs(title = "Split"),
    left_columns = c("term", "n", "events"),
    right_columns = c("estimate", "p")
  )

  center_plot <- out$patches$plots[[2]]

  expect_s3_class(center_plot$theme$panel.border, "element_blank")
  expect_s3_class(center_plot$theme$axis.line.x, "element_line")
  expect_s3_class(center_plot$theme$axis.line.y, "element_blank")
  expect_s3_class(center_plot$theme$panel.grid.major, "element_blank")
  expect_s3_class(center_plot$theme$panel.grid.minor, "element_blank")
})

test_that("add_split_table uses split-specific alignment and no grid lines", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value")
  state <- p$ggforestplotR_state
  left_spec <- build_forest_table_data(state$forest_data, columns = c("term", "n", "events"))
  right_spec <- build_forest_table_data(state$forest_data, columns = c("estimate", "p"))
  left_spec <- layout_split_table_spec(left_spec, text_size = 3.2, alignment = "left")
  right_spec <- layout_split_table_spec(right_spec, text_size = 3.2, alignment = "right")
  left_plot <- build_forest_table_plot(
    table_spec = left_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "left",
    striped_rows = TRUE,
    grid_lines = FALSE,
    x_expand = c(0.05, 0.35),
    plot_margin = ggplot2::margin(5.5, 0, 5.5, 5.5),
    text_hjust = 0,
    header_hjust = 0
  )
  right_plot <- build_forest_table_plot(
    table_spec = right_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "right",
    striped_rows = TRUE,
    grid_lines = FALSE,
    x_expand = c(0.1, 0.35),
    plot_margin = ggplot2::margin(5.5, 5.5, 5.5, 0),
    text_hjust = 1,
    header_hjust = 1
  )

  left_built <- ggplot2::ggplot_build(left_plot)
  right_built <- ggplot2::ggplot_build(right_plot)

  expect_equal(left_plot$layers[[2]]$aes_params$hjust, 0)
  expect_equal(right_plot$layers[[2]]$aes_params$hjust, 1)
  expect_equal(left_plot$theme$axis.text.x.top$hjust, 0)
  expect_equal(right_plot$theme$axis.text.x.top$hjust, 1)
  expect_length(Filter(function(x) "yintercept" %in% names(x), left_built$data), 0L)
  expect_length(Filter(function(x) "yintercept" %in% names(x), right_built$data), 0L)
})

test_that("add_split_table supports header size and font customization", {
  raw <- data.frame(
    term = c("Age", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  p <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value")
  state <- p$ggforestplotR_state
  left_spec <- build_forest_table_data(state$forest_data, columns = c("term", "n", "events"))
  right_spec <- build_forest_table_data(state$forest_data, columns = c("estimate", "p"))
  left_spec <- layout_split_table_spec(
    left_spec,
    text_size = 3.2,
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif",
    alignment = "left"
  )
  right_spec <- layout_split_table_spec(
    right_spec,
    text_size = 3.2,
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif",
    alignment = "right"
  )
  left_plot <- build_forest_table_plot(
    table_spec = left_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "left",
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif"
  )
  right_plot <- build_forest_table_plot(
    table_spec = right_spec,
    stripe_data = state$stripe_data,
    has_groupings = state$has_groupings,
    grouping_strip_position = state$grouping_strip_position,
    table_position = "right",
    header_text_size = 13,
    header_fontface = "plain",
    header_family = "serif"
  )

  expect_equal(left_plot$theme$axis.text.x.top$size, 13)
  expect_equal(left_plot$theme$axis.text.x.top$face, "plain")
  expect_equal(left_plot$theme$axis.text.x.top$family, "serif")
  expect_equal(right_plot$theme$axis.text.x.top$size, 13)
  expect_equal(right_plot$theme$axis.text.x.top$face, "plain")
  expect_equal(right_plot$theme$axis.text.x.top$family, "serif")
})


test_that("add_split_table sizes panels from split column counts", {
  raw <- data.frame(
    term = c("Very long predictor name", "BMI", "Treatment"),
    estimate = c(0.3, -0.2, 0.4),
    conf.low = c(0.1, -0.4, 0.2),
    conf.high = c(0.5, 0.0, 0.6),
    sample_size = c(120, 115, 98),
    event_count = c(42, 39, 31),
    p_value = c(0.012, 0.031, 0.004)
  )

  out_equal <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = c("term", "events"), right_columns = c("estimate", "p"))
  widths_equal <- out_equal$patches$layout$widths

  out_unequal <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = "term", right_columns = c("estimate", "p"))
  widths_unequal <- out_unequal$patches$layout$widths

  out_three <- ggforestplot(raw, n = "sample_size", events = "event_count", p.value = "p_value") +
    add_split_table(left_columns = c("term", "n", "events"), right_columns = "estimate")
  widths_three <- out_three$patches$layout$widths

  expect_equal(widths_equal, c(2.5, 2.5, 2.5))
  expect_equal(widths_unequal, c(1.25, 2.5, 2.5))
  expect_equal(widths_three, c(2.5 * (4 / 3), 2.5, 1.25))
  expect_equal(split_table_width_multiplier(1), 0.5)
  expect_equal(split_table_width_multiplier(2), 1)
  expect_equal(split_table_width_multiplier(3), 4 / 3)
})

test_that("add_split_table validates events requests", {
  raw <- data.frame(
    term = c("Age", "BMI"),
    estimate = c(0.3, -0.2),
    conf.low = c(0.1, -0.4),
    conf.high = c(0.5, 0.0)
  )

  p <- ggforestplot(raw)

  expect_error(
    add_split_table(p, left_columns = c("term", "events"), right_columns = "estimate"),
    "An `events` column is required"
  )
})

#' ggforestplotR: Forest plots from model coefficients with ggplot2
#'
#' Transform model coefficients into flexible forest plots using
#' `ggplot2`. Provides helpers to standardize coefficient data
#' from a range of modelling workflows and render publication-ready
#' coefficient plots with a consistent interface.
#' 
#'
#' @keywords internal
#' @importFrom ggplot2 ggplot_add
#' @importFrom rlang .data
"_PACKAGE"

#' Standardize coefficient data for forest plots
#'
#' Standardizes a coefficient table into the internal forest-plot data
#' structure used throughout `ggforestplotR`.
#'
#' @param data A data frame containing coefficient estimates and intervals.
#' @param term Column name holding the model term identifier.
#' @param estimate Column name holding the point estimate.
#' @param conf.low Column name holding the lower confidence bound.
#' @param conf.high Column name holding the upper confidence bound.
#' @param label Optional column name used for the displayed row label.
#' @param group Optional column name used for color-grouping multiple
#'   estimates per row.
#' @param grouping Optional column name used to split rows into grouped plot
#'   sections.
#' @param separate_groups Optional column name used to identify labeled
#'   variable blocks that can be outlined with separator lines.
#' @param n Optional column name holding sample sizes or other N labels for
#'   table helpers.
#' @param events Optional column name holding event counts or event labels for
#'   table helpers.
#' @param p.value Optional column name holding p-values.
#' @param exponentiate Logical; if `TRUE`, require positive values for
#'   estimates and intervals.
#' @param sort_terms How to sort rows: `"none"`, `"descending"`, or
#'   `"ascending"`.
#'
#' @return A standardized data frame ready for [ggforestplot()] and the table
#'   composition helpers.
#' @export
#'
#' @examples
#' raw <- data.frame(
#'   variable = c("Age", "BMI", "Treatment"),
#'   beta = c(0.10, -0.08, 0.34),
#'   lower = c(0.02, -0.16, 0.12),
#'   upper = c(0.18, 0.00, 0.56)
#' )
#'
#' as_forest_data(
#'   data = raw,
#'   term = "variable",
#'   estimate = "beta",
#'   conf.low = "lower",
#'   conf.high = "upper"
#' )
as_forest_data <- function(data,
                           term,
                           estimate,
                           conf.low,
                           conf.high,
                           label = term,
                           group = NULL,
                           grouping = NULL,
                           separate_groups = NULL,
                           n = NULL,
                           events = NULL,
                           p.value = NULL,
                           exponentiate = FALSE,
                           sort_terms = c("none", "descending", "ascending")) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }

  sort_terms <- match.arg(sort_terms)

  cols <- list(
    term = resolve_column(data, term, "term"),
    estimate = resolve_column(data, estimate, "estimate"),
    conf.low = resolve_column(data, conf.low, "conf.low"),
    conf.high = resolve_column(data, conf.high, "conf.high"),
    label = resolve_column(data, label, "label", required = FALSE),
    group = resolve_column(data, group, "group", required = FALSE),
    grouping = resolve_column(data, grouping, "grouping", required = FALSE),
    separate_groups = resolve_column(data, separate_groups, "separate_groups", required = FALSE),
    n = resolve_column(data, n, "n", required = FALSE),
    events = resolve_column(data, events, "events", required = FALSE),
    p.value = resolve_column(data, p.value, "p.value", required = FALSE)
  )

  out <- data.frame(
    term = as.character(data[[cols$term]]),
    estimate = as.numeric(data[[cols$estimate]]),
    conf.low = as.numeric(data[[cols$conf.low]]),
    conf.high = as.numeric(data[[cols$conf.high]]),
    stringsAsFactors = FALSE
  )

  out$label <- if (is.null(cols$label)) {
    out$term
  } else {
    as.character(data[[cols$label]])
  }

  out$group <- if (is.null(cols$group)) {
    NA_character_
  } else {
    as.character(data[[cols$group]])
  }

  out$grouping <- if (is.null(cols$grouping)) {
    NA_character_
  } else {
    as.character(data[[cols$grouping]])
  }

  out$separate_groups <- if (is.null(cols$separate_groups)) {
    NA_character_
  } else {
    as.character(data[[cols$separate_groups]])
  }

  out$n <- if (is.null(cols$n)) {
    NA_character_
  } else {
    as.character(data[[cols$n]])
  }

  out$events <- if (is.null(cols$events)) {
    NA_character_
  } else {
    as.character(data[[cols$events]])
  }

  out$p.value <- if (is.null(cols$p.value)) {
    NA_real_
  } else {
    as.numeric(data[[cols$p.value]])
  }

  validate_forest_data(out, exponentiate = exponentiate)

  if (sort_terms == "descending") {
    if (any(!is.na(out$grouping))) {
      out <- out[order(out$grouping, out$estimate, decreasing = c(FALSE, TRUE)), , drop = FALSE]
    } else {
      out <- out[order(out$estimate, decreasing = TRUE), , drop = FALSE]
    }
  } else if (sort_terms == "ascending") {
    if (any(!is.na(out$grouping))) {
      out <- out[order(out$grouping, out$estimate, decreasing = c(FALSE, FALSE)), , drop = FALSE]
    } else {
      out <- out[order(out$estimate, decreasing = FALSE), , drop = FALSE]
    }
  }

  rownames(out) <- NULL

  out
}

test_that("as_forest_data standardizes coefficient columns", {
  raw <- data.frame(
    variable = c("Age", "BMI"),
    beta = c(0.3, -0.2),
    lower = c(0.1, -0.4),
    upper = c(0.5, 0.0),
    cohort = c("A", "A"),
    section = c("Clinical", "Clinical"),
    block = c("Anthropometrics", "Anthropometrics"),
    sample_size = c(120, 120),
    event_count = c(32, 28)
  )

  out <- as_forest_data(
    data = raw,
    term = "variable",
    estimate = "beta",
    conf.low = "lower",
    conf.high = "upper",
    group = "cohort",
    grouping = "section",
    separate_groups = "block",
    n = "sample_size",
    events = "event_count"
  )

  expect_equal(
    names(out)[seq_len(11)],
    c("term", "estimate", "conf.low", "conf.high", "label", "group", "grouping", "separate_groups", "n", "events", "p.value")
  )
  expect_true(all(c("variable", "beta", "lower", "upper", "cohort", "section", "block", "sample_size", "event_count") %in% names(out)))
  expect_equal(as.character(out$term), c("Age", "BMI"))
  expect_equal(as.character(out$group), c("A", "A"))
  expect_equal(as.character(out$grouping), c("Clinical", "Clinical"))
  expect_equal(as.character(out$separate_groups), c("Anthropometrics", "Anthropometrics"))
  expect_equal(as.character(out$n), c("120", "120"))
  expect_equal(as.character(out$events), c("32", "28"))
})

test_that("as_forest_data rejects reversed confidence intervals", {
  raw <- data.frame(
    term = "Age",
    estimate = 0.3,
    conf.low = 0.5,
    conf.high = 0.1
  )

  expect_error(
    as_forest_data(raw, "term", "estimate", "conf.low", "conf.high"),
    "conf.low"
  )
})

test_that("as_forest_data sorts terms within grouping sections", {
  raw <- data.frame(
    term = c("Age", "BMI", "Stage II", "Stage III"),
    estimate = c(0.3, -0.2, 0.5, 0.8),
    conf.low = c(0.1, -0.4, 0.2, 0.4),
    conf.high = c(0.5, 0.0, 0.8, 1.2),
    section = c("Clinical", "Clinical", "Tumor", "Tumor")
  )

  descending <- as_forest_data(
    raw,
    "term",
    "estimate",
    "conf.low",
    "conf.high",
    grouping = "section",
    sort_terms = "descending"
  )
  ascending <- as_forest_data(
    raw,
    "term",
    "estimate",
    "conf.low",
    "conf.high",
    grouping = "section",
    sort_terms = "ascending"
  )

  expect_equal(descending$term, c("Age", "BMI", "Stage III", "Stage II"))
  expect_equal(ascending$term, c("BMI", "Age", "Stage II", "Stage III"))
  expect_equal(descending$grouping, c("Clinical", "Clinical", "Tumor", "Tumor"))
})

#' @export
#' @keywords internal
ggplot_add.ScaleDiscretePosition <- function(object, plot, ...) {
  if (is_ggforestplot_faceted_y_scale(object, plot)) {
    object <- align_discrete_y_scale_to_forest_facets(object, plot)
  }

  NextMethod()
}

#' @export
#' @keywords internal
ggplot_add.ScaleContinuousPosition <- function(object, plot, ...) {
  if (is_ggforestplot_exponentiated_x_scale(object, plot)) {
    object <- zero_default_x_scale_expansion(object)
    plot <- align_striped_rows_to_x_scale(object, plot)
  }

  NextMethod()
}

is_ggforestplot_faceted_y_scale <- function(scale, plot) {
  state <- plot$ggforestplotR_state

  if (is.null(state) || !isTRUE(state$has_groupings)) {
    return(FALSE)
  }

  if (is.null(scale$aesthetics) || !"y" %in% scale$aesthetics) {
    return(FALSE)
  }

  is.character(scale$limits) && length(scale$limits) > 0L
}

align_discrete_y_scale_to_forest_facets <- function(scale, plot) {
  forest_data <- plot$ggforestplotR_state$forest_data
  row_lookup <- unique(data.frame(
    row_key = as.character(forest_data$row_key),
    label = as.character(forest_data$label),
    stringsAsFactors = FALSE
  ))
  requested_limits <- as.character(scale$limits)

  scale$limits <- function(x) {
    panel_row_keys <- as.character(x)
    panel_lookup <- row_lookup[row_lookup$row_key %in% panel_row_keys, , drop = FALSE]

    matched <- unlist(lapply(requested_limits, function(limit) {
      panel_lookup$row_key[
        panel_lookup$label == limit | panel_lookup$row_key == limit
      ]
    }), use.names = FALSE)

    unique(matched)
  }

  if (inherits(scale$labels, "waiver")) {
    scale$labels <- function(x) {
      matched_labels <- row_lookup$label[match(as.character(x), row_lookup$row_key)]
      ifelse(is.na(matched_labels), as.character(x), matched_labels)
    }
  }

  scale
}

is_ggforestplot_exponentiated_x_scale <- function(scale, plot) {
  state <- plot$ggforestplotR_state

  if (is.null(state) || !isTRUE(state$defaults$exponentiate)) {
    return(FALSE)
  }

  if (is.null(scale$aesthetics) || !"x" %in% scale$aesthetics) {
    return(FALSE)
  }

  TRUE
}

zero_default_x_scale_expansion <- function(scale) {
  if (inherits(scale$expand, "waiver")) {
    scale$expand <- ggplot2::expansion(mult = 0)
  }

  scale
}

align_striped_rows_to_x_scale <- function(scale, plot) {
  state <- plot$ggforestplotR_state

  if (!isTRUE(state$defaults$striped_rows) || is.null(state$stripe_layer_index)) {
    return(plot)
  }

  stripe_layer_index <- state$stripe_layer_index

  if (stripe_layer_index > length(plot$layers)) {
    return(plot)
  }

  stripe_data <- plot$layers[[stripe_layer_index]]$data
  stripe_limits <- x_scale_limits_for_stripes(scale, stripe_data)

  if (is.null(stripe_limits)) {
    return(plot)
  }

  plot$layers[[stripe_layer_index]]$data$xmin <- stripe_limits[1]
  plot$layers[[stripe_layer_index]]$data$xmax <- stripe_limits[2]

  plot
}

x_scale_limits_for_stripes <- function(scale, stripe_data) {
  scale_limits <- scale$limits

  if (is.null(scale_limits) || is.function(scale_limits) || inherits(scale_limits, "waiver")) {
    return(NULL)
  }

  if (!is.numeric(scale_limits) || length(scale_limits) != 2L) {
    return(NULL)
  }

  stripe_limits <- inverse_continuous_scale_limits(scale, scale_limits)

  current_limits <- c(
    min(stripe_data$xmin, na.rm = TRUE),
    max(stripe_data$xmax, na.rm = TRUE)
  )
  stripe_limits[is.na(stripe_limits)] <- current_limits[is.na(stripe_limits)]

  if (any(!is.finite(stripe_limits))) {
    return(NULL)
  }

  stripe_limits
}

inverse_continuous_scale_limits <- function(scale, limits) {
  transformation <- scale$trans

  if (is.null(transformation)) {
    transformation <- scale$transformation
  }

  if (!is.null(transformation) && is.function(transformation$inverse)) {
    return(transformation$inverse(limits))
  }

  limits
}

---
title: "Customize Forest Plots and Tables"
output:
  rmarkdown::html_vignette:
    highlight: pygments
vignette: >
  %\VignetteIndexEntry{Customize Forest Plots and Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8.5,
  fig.height = 5.5
)
```

```{r setup}
library(ggforestplotR)
library(ggplot2)
```

This article focuses on utility and enhanced customization of forest plots and accompanied tables.

## Base plotting data

```{r setup-data}
coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III"),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74),
  sample_size = c(120, 115, 98, 87, 83),
  p_value = c(0.04, 0.15, 0.29, 0.001, 0.75),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor")
)
```

## Group rows and control strip placement

`grouping` creates section panels, and `grouping_strip_position` controls which
side gets the strip labels.

```{r grouping-right}
ggforestplot(
  coefs,
  grouping = "section",
  grouping_strip_position = "right",
  striped_rows = TRUE
)
```

## Distinct variable separation

Use `separate_groups` and `separate_lines` when you want a more distinct visual separation between variables. This is especially useful for categorical variables with many levels. `separate_groups` automatically appends the variable name to the level.

```{r separators}
block_coefs <- data.frame(
  term = c("race_black", "race_white", "race_other", "age", "bmi"),
  label = c("Black", "White", "Other", "Age", "BMI"),
  estimate = c(0.24, 0.08, -0.04, 0.12, -0.09),
  conf.low = c(0.10, -0.04, -0.18, 0.03, -0.17),
  conf.high = c(0.38, 0.20, 0.10, 0.21, -0.01),
  variable_block = c("Race", "Race", "Race", "Age", "BMI")
)

ggforestplot(
  block_coefs,
  label = "label",
  separate_groups = "variable_block",
  separate_lines = TRUE,
  striped_rows = TRUE
) +
  scale_y_discrete(limits = rev(c("BMI", "Age", "Race: White", 
                                  "Race: Black", "Race: Other")))
```

## Add a side table

`add_forest_table()` allows you to attach model information to the coefficient plot. The table can be added to either the left or right side and allows for some customization. You should **always** add the table **LAST**, after styling your plot because the function calls on `patchwork` internally. `patchwork` requires specific syntax to customize plots and is generally more difficult to get working correctly.

```{r left-side-table}
ggforestplot(
  coefs,
  grouping = "section",
  grouping_strip_position = "right",
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  add_forest_table(
    show_n = TRUE,
    show_p = TRUE,
    estimate_label = "Beta"
  )
```

## Customize the table

`add_forest_table` also lets you change some minor styling elements of the forest table.

```{r}
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  add_forest_table(
    position = "left",
    show_n = TRUE,
    show_p = TRUE,
    estimate_label = "Beta",
    grid_lines = T,
    grid_line_linetype = 2,
    grid_line_colour = "red"
  )
```


## Split tables

`add_split_table()` can be used to create more traditional looking forest plots. You can choose which summary information goes to which side. Like `add_forest_table()`, it should be added after any plot-level styling. 

```{r split-table}
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  scale_x_continuous(limits = c(-.8,.8)) +
  add_split_table(
    left_columns = c("term","n"),
    right_columns = c("estimate","p"),
    estimate_label = "Beta"
  ) 
```

## Plotting other types of model coefficients

You can use `exponentiate = TRUE` for models on the log-odds scale (or similar).

```{r logistic-regression-data}
data(CO2)

l1 <- glm(Treatment ~ conc + uptake + Type, family = binomial(link = "logit"), 
    data = CO2)
```

```{r logistic-regression, warning=FALSE}

ggforestplot(l1, exponentiate = TRUE, striped_rows = T) +
  add_forest_table(position = "left", 
                   estimate_label = "OR", 
                   show_p = F)
```

We can do this for survival models as well.

```{r survival-analysis-data}
lung <- survival::lung

lung <- lung |>  
  dplyr::mutate(
    status = dplyr::recode(status, `1` = 0, `2` = 1)
  )

s1 <- survival::coxph(Surv(time, status) ~ sex + age + ph.karno + pat.karno, data = lung)
```

```{r survival-analysis-plot}
ggforestplot(s1, exponentiate = T, striped_rows = T) +
  add_forest_table(estimate_label = "HR")
```

## Compare multiple estimates

The `group` argument is handy when comparing estimates from several models.

```{r comparison}
comparison_coefs <- data.frame(
  term = rep(c("Age", "BMI", "Smoking", "Stage II", "Stage III"), 2),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46, 0.08, -0.05, 0.24, 0.40, 0.58),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18, 0.00, -0.13, 0.10, 0.20, 0.30),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74, 0.16, 0.03, 0.38, 0.60, 0.86),
  model = rep(c("Model A", "Model B"), each = 5),
  section = rep(c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor"), 2)
)

ggforestplot(
  comparison_coefs,
  group = "model",
  grouping = "section",
  striped_rows = TRUE,
  dodge_width = 0.5,
  grouping_strip_position = "right"
) +
  theme(legend.position = "bottom")
```

---
title: "Customize Forest Plots and Tables"
output:
  rmarkdown::html_vignette:
    highlight: pygments
vignette: >
  %\VignetteIndexEntry{Customize Forest Plots and Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8.5,
  fig.height = 5.5
)
```

```{r setup}
library(ggforestplotR)
library(ggplot2)
```

This article focuses on utility and enhanced customization of forest plots and accompanied tables.

## Group rows and control strip placement

`facet` creates section panels, and `facet_strip_position` controls which
side gets the strip labels.

```{r facet-right}
coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III"),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18),
  conf.high = c(0.21, 0.02, 0.32, 0.50, 0.74),
  sample_size = c(120, 115, 98, 87, 83),
  p_value = c(0.04, 0.15, 0.29, 0.001, 0.075),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor")
)


ggforestplot(
  coefs,
  facet = "section",
  facet_strip_position = "right",
  striped_rows = TRUE
)
```

## Distinct variable separation

Use `separate_groups` and `separate_lines` when you want a more distinct visual separation between variables. This is especially useful for categorical variables with many levels. `separate_groups` automatically appends the variable name to the level.

```{r separators}
block_coefs <- data.frame(
  term = c("race_black", "race_white", "race_other", "age", "bmi"),
  label = c("Black", "White", "Other", "Age", "BMI"),
  estimate = c(0.24, 0.08, -0.04, 0.12, -0.09),
  conf.low = c(0.10, -0.04, -0.18, 0.03, -0.17),
  conf.high = c(0.38, 0.20, 0.10, 0.21, -0.01),
  variable_block = c("Race", "Race", "Race", "Age", "BMI")
)

ggforestplot(
  block_coefs,
  label = "label",
  separate_groups = "variable_block",
  separate_lines = TRUE,
  striped_rows = TRUE
) +
  scale_y_discrete(limits = rev(c("BMI", "Age", "Race: White", 
                                  "Race: Black", "Race: Other")))
```

## Add a side table

`add_forest_table()` allows you to attach model information to the coefficient plot. The table can be added to either the left or right side and allows for some customization. You should **always** add the table **LAST**, after styling your plot because the function calls on `patchwork` internally. `patchwork` requires specific syntax to customize plots and is generally more difficult to get working correctly.

You can choose which columns from your dataframe to include in the table using the `columns` argument, and can change the labels using `column_labels`. If some of the term labels need to be changed, use `term_labels` to assign them new values. Some of the column labels are automatically assigned if no value is provided.

Notice how we are explicitly naming the *n* and *p.value* columns? This is necessary in most cases because aliases are not yet incorporated (but they will be...I promise I'm getting to it).

```{r left-side-table}
ggforestplot(
  coefs,
  facet = "section",
  facet_strip_position = "right",
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE,
  term_labels = c("Smoking" = "Smoking status")
) +
  add_forest_table(
    columns = c("term", "sample_size", "estimate", "p_value"),
    column_labels = c("term" = "Variable", "sample_size" = "N",
                      "estimate" = "Beta (95% CI)", "p_value" = "P-value")
  )
```

## Customize the table

`add_forest_table` also lets you change some minor styling elements of the forest table.

```{r}
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  add_forest_table(
    position = "left",
    grid_lines = T,
    grid_line_linetype = 2,
    grid_line_colour = "red"
  )
```


## Split tables

`add_split_table()` can be used to create more traditional looking forest plots. You can choose which summary information goes to which side. Like `add_forest_table()`, it should be added after any plot-level styling. 

Use the `estimate_fmt` argument to change how your estimates are displayed. You can also control digits via `estimate_digits` and `interval_digits`.

```{r split-table}
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  scale_x_continuous(limits = c(-.8,.8)) +
  add_split_table(
    left_columns = c("term","n"),
    right_columns = c("estimate","p"),
    column_labels = c("estimate" = "Beta [95% CI]"),
    estimate_fmt = "{estimate} [{conf.low}, {conf.high}]",
    estimate_digits = 2,
    interval_digits = 3
  ) 
```

## Plotting other types of model coefficients

You can use `exponentiate = TRUE` for models on the log-odds scale (or similar).

```{r logistic-regression-data}
data(CO2)

l1 <- glm(Treatment ~ conc + uptake + Type, family = binomial(link = "logit"), 
    data = CO2)
```

```{r logistic-regression, warning=FALSE}

ggforestplot(l1, exponentiate = TRUE, striped_rows = T, term_labels = c("TypeMississippi" = "Mississippi")) +
  add_forest_table(position = "left", 
                   columns = c("term", "estimate"))
```

We can do this for survival models as well.

```{r survival-analysis-data}
lung <- survival::lung

lung <- lung |>  
  dplyr::mutate(
    status = dplyr::recode(status, `1` = 0, `2` = 1)
  )

s1 <- survival::coxph(Surv(time, status) ~ sex + age + ph.karno + pat.karno, data = lung)
```

```{r survival-analysis-plot}
ggforestplot(s1, exponentiate = T, striped_rows = T) +
  add_forest_table()
```

## Compare multiple estimates

The `group` argument is handy when comparing estimates from several models.

```{r comparison}
comparison_coefs <- data.frame(
  term = rep(c("Age", "BMI", "Smoking", "Stage II", "Stage III"), 2),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46, 0.08, -0.05, 0.24, 0.40, 0.58),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18, 0.00, -0.13, 0.10, 0.20, 0.30),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74, 0.16, 0.03, 0.38, 0.60, 0.86),
  model = rep(c("Model A", "Model B"), each = 5),
  section = rep(c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor"), 2)
)

ggforestplot(
  comparison_coefs,
  group = "model",
  facet = "section",
  striped_rows = TRUE,
  dodge_width = 0.5,
  facet_strip_position = "right"
) +
  theme(legend.position = "top") +
  scale_color_manual(values = c("#1F968BFF", "#453781FF")) +
  add_forest_table()
```

You can also use `bind_forest_models()` to plot estimates from several fit models at once.

```{r bind-models, fig.width=8, fig.height=4.5, fig.dpi=300}
fit1 <- lm(mpg ~ cyl, data = mtcars)
fit2 <- lm(mpg ~ cyl + disp, data = mtcars)
fit3 <- lm(mpg ~ cyl + disp + wt, data = mtcars)

bound_models <- bind_forest_models(list(fit1,fit2,fit3), 
                                   model_labels = c("Unadjusted", "Adjusted", "Fully Adjusted"))

ggforestplot(bound_models, striped_rows = T, p.value = "p.value") +
  scale_x_continuous(limits = c(-6,1)) +
  theme(legend.position = "top") +
  scale_color_manual(values = c("#1F968BFF", "#453781FF", "#FDE725FF")) +
  add_forest_table(columns = c("term", "estimate", "p.value"),
                   p_digits = 4)
```

---
title: "Customize Forest Plots and Tables"
output:
  rmarkdown::html_vignette:
    highlight: pygments
vignette: >
  %\VignetteIndexEntry{Customize Forest Plots and Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8.5,
  fig.height = 5.5
)
```

```{r setup}
library(ggforestplotR)
library(ggplot2)
```

This article focuses on utility and enhanced customization of forest plots and accompanied tables.

## Group rows and control strip placement

`facet` creates section panels, and `facet_strip_position` controls which
side gets the strip labels.

```{r facet-right}
coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III"),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18),
  conf.high = c(0.21, 0.02, 0.32, 0.50, 0.74),
  sample_size = c(120, 115, 98, 87, 83),
  p_value = c(0.04, 0.15, 0.29, 0.001, 0.075),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor")
)


ggforestplot(
  coefs,
  facet = "section",
  facet_strip_position = "right",
  striped_rows = TRUE
)
```

## Distinct variable separation

Use `separate_groups` and `separate_lines` when you want a more distinct visual separation between variables. This is especially useful for categorical variables with many levels. `separate_groups` automatically appends the variable name to the level.

```{r separators}
block_coefs <- data.frame(
  term = c("race_black", "race_white", "race_other", "age", "bmi"),
  label = c("Black", "White", "Other", "Age", "BMI"),
  estimate = c(0.24, 0.08, -0.04, 0.12, -0.09),
  conf.low = c(0.10, -0.04, -0.18, 0.03, -0.17),
  conf.high = c(0.38, 0.20, 0.10, 0.21, -0.01),
  variable_block = c("Race", "Race", "Race", "Age", "BMI")
)

ggforestplot(
  block_coefs,
  label = "label",
  separate_groups = "variable_block",
  separate_lines = TRUE,
  striped_rows = TRUE
) +
  scale_y_discrete(limits = rev(c("BMI", "Age", "Race: White", 
                                  "Race: Black", "Race: Other")))
```

## Add a side table

`add_forest_table()` allows you to attach model information to the coefficient plot. The table can be added to either the left or right side and allows for some customization. You should **always** add the table **LAST**, after styling your plot because the function calls on `patchwork` internally. `patchwork` requires specific syntax to customize plots and is generally more difficult to get working correctly.

You can choose which columns from your dataframe to include in the table using the `columns` argument, and can change the labels using `column_labels`. If some of the term labels need to be changed, use `term_labels` to assign them new values. Some of the column labels are automatically assigned if no value is provided.

Notice how we are explicitly naming the *n* and *p.value* columns? This is necessary in most cases because aliases are not yet incorporated (but they will be...I promise I'm getting to it).

```{r left-side-table}
ggforestplot(
  coefs,
  facet = "section",
  facet_strip_position = "right",
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE,
  term_labels = c("Smoking" = "Smoking status")
) +
  add_forest_table(
    columns = c("term", "sample_size", "estimate", "p_value"),
    column_labels = c("term" = "Variable", "sample_size" = "N",
                      "estimate" = "Beta (95% CI)", "p_value" = "P-value")
  )
```

## Customize the table

`add_forest_table` also lets you change some minor styling elements of the forest table.

```{r}
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  add_forest_table(
    position = "left",
    grid_lines = T,
    grid_line_linetype = 2,
    grid_line_colour = "red"
  )
```


## Split tables

`add_split_table()` can be used to create more traditional looking forest plots. You can choose which summary information goes to which side. Like `add_forest_table()`, it should be added after any plot-level styling. 

Use the `estimate_fmt` argument to change how your estimates are displayed. You can also control digits via `estimate_digits` and `interval_digits`.

```{r split-table}
ggforestplot(
  coefs,
  n = "sample_size",
  p.value = "p_value",
  striped_rows = TRUE
) +
  scale_x_continuous(limits = c(-.8,.8)) +
  add_split_table(
    left_columns = c("term","n"),
    right_columns = c("estimate","p"),
    column_labels = c("estimate" = "Beta [95% CI]"),
    estimate_fmt = "{estimate} [{conf.low}, {conf.high}]",
    estimate_digits = 2,
    interval_digits = 3
  ) 
```

## Plotting other types of model coefficients

You can use `exponentiate = TRUE` for models on the log-odds scale (or similar).

```{r logistic-regression-data}
data(CO2)

l1 <- glm(Treatment ~ conc + uptake + Type, family = binomial(link = "logit"), 
    data = CO2)
```

```{r logistic-regression, warning=FALSE}

ggforestplot(l1, exponentiate = TRUE, striped_rows = T, term_labels = c("TypeMississippi" = "Mississippi")) +
  add_forest_table(position = "left", 
                   show_p = F)
```

We can do this for survival models as well.

```{r survival-analysis-data}
lung <- survival::lung

lung <- lung |>  
  dplyr::mutate(
    status = dplyr::recode(status, `1` = 0, `2` = 1)
  )

s1 <- survival::coxph(Surv(time, status) ~ sex + age + ph.karno + pat.karno, data = lung)
```

```{r survival-analysis-plot}
ggforestplot(s1, exponentiate = T, striped_rows = T) +
  add_forest_table()
```

## Compare multiple estimates

The `group` argument is handy when comparing estimates from several models.

```{r comparison}
comparison_coefs <- data.frame(
  term = rep(c("Age", "BMI", "Smoking", "Stage II", "Stage III"), 2),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46, 0.08, -0.05, 0.24, 0.40, 0.58),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18, 0.00, -0.13, 0.10, 0.20, 0.30),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74, 0.16, 0.03, 0.38, 0.60, 0.86),
  model = rep(c("Model A", "Model B"), each = 5),
  section = rep(c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor"), 2)
)

ggforestplot(
  comparison_coefs,
  group = "model",
  facet = "section",
  striped_rows = TRUE,
  dodge_width = 0.5,
  facet_strip_position = "right"
) +
  theme(legend.position = "top") +
  scale_color_manual(values = c("#1F968BFF", "#453781FF")) +
  add_forest_table()
```

---
title: "Get Started with ggforestplotR"
output:
  rmarkdown::html_vignette:
    highlight: pygments
vignette: >
  %\VignetteIndexEntry{Get Started with ggforestplotR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 5
)
```

```{r setup}
library(ggforestplotR)
library(ggplot2)
```

`ggforestplotR` is built for coefficient-driven forest plots that stay inside a
normal `ggplot2` workflow.

## Choose a workflow

Use the package in one of two ways:

1. Start from a coefficient table and map the required columns directly to the plot.
2. Start from a fitted model and let `tidy_forest_model()` or `ggforestplot()` call `broom::tidy()` and create the plot.

This article covers some basic examples and the minimum data you need.

## Start from a coefficient table

The simplest input is a data frame with a column for terms, estimates, and confidence limits. If your columns use different names, you can map them explicitly.

There is also functionality to rename term labels and sort terms to your liking, among other things.

```{r basic-plot}
basic_coefs <- data.frame(
  term = c("Age", "BMI", "Treatment"),
  estimate = c(0.10, -0.08, 0.34),
  conf.low = c(0.02, -0.16, 0.12),
  conf.high = c(0.18, 0.00, 0.56)
)

ggforestplot(basic_coefs,
             term_labels = c("Age" = "age", "BMI" = "bmi", "Treatment" = "trt"),
             sort_terms = "descending")
```

## Add grouped sections and row striping

Use `facet` when you want related variables separated into labeled panels.
Add `striped_rows = TRUE` to color alternating rows in the plot.

```{r grouped-striped}
sectioned_coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III", "Nodes"),
  estimate = c(0.10, -0.08, 0.20, 0.34, 0.52, 0.28),
  conf.low = c(0.02, -0.16, 0.05, 0.12, 0.20, 0.06),
  conf.high = c(0.18, 0.00, 0.35, 0.56, 0.84, 0.50),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor", "Tumor")
)

ggforestplot(
  sectioned_coefs,
  facet = "section",
  striped_rows = TRUE,
  stripe_fill = "grey94",
  facet_strip_position = "right",
  sort_terms = "ascending"
)
```

## Add a summary table

Use `add_forest_table()` to add a summary table to your forest plot.

```{r side-table}
tabled_coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III"),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74),
  sample_size = c(120, 115, 98, 87, 83)
)

ggforestplot(tabled_coefs, striped_rows = TRUE) +
  add_forest_table(
    position = "left",
    column_labels = c("term" = "Variable", "sample_size" = "N", "estimate" = "Beta (95% CI)"),
    columns = c("term", "sample_size", "estimate"),
    estimate_digits = 2,
    interval_digits = 3
  )
```

## Add split summary tables

Use `add_split_table()` to create a more traditional looking forest plot, with summary data on either side of the plot.

```{r split-table}
ggforestplot(tabled_coefs, n = "sample_size", striped_rows = T) +
  add_split_table(
    left_columns = c("term", "n"),
    right_columns = c("estimate"),
    column_labels = c("term" = "Variable", "estimate" = "Beta (95% CI)")
  )
```

## Start from a fitted model

If `broom` is installed, `ggforestplot()` can work directly from a fitted model.

```{r model-plot}
fit <- lm(mpg ~ wt + hp + qsec, data = mtcars)

ggforestplot(fit, sort_terms = "descending",
             term_labels = c("wt" = "Weight"),
             striped_rows = T) +
  scale_x_continuous(breaks = seq(-6,2,1)) +
  add_forest_table()
```

## Next articles

For more detail, see:

- `ggforestplotR-plot-customization` for enhanced customization of the plots and summary tables.
- `ggforestplotR-data-helpers` for `as_forest_data()` and `tidy_forest_model()`.

---
title: "Prepare Forest Data with Helper Functions"
output:
  rmarkdown::html_vignette:
    highlight: pygments
vignette: >
  %\VignetteIndexEntry{Prepare Forest Data with Helper Functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 4.5
)
```

```{r setup}
library(ggforestplotR)
library(ggplot2)
```

This short article covers the two helper functions that prepare data before the
plot is drawn.

## Use `as_forest_data()` to standardize a coefficient table

`as_forest_data()` converts your column names into the internal structure used
by `ggforestplotR`. The result contains the columns expected by
`ggforestplot()`, `add_forest_table()`, and `add_split_table()`.

```{r as-forest-data}
raw_coefs <- data.frame(
  variable = c("Age", "BMI", "Treatment"),
  beta = c(0.10, -0.08, 0.34),
  lower = c(0.02, -0.16, 0.12),
  upper = c(0.18, 0.00, 0.56),
  display = c("Age", "BMI", "Treatment"),
  section = c("Clinical", "Clinical", "Treatment"),
  sample_size = c(120, 115, 98),
  p_value = c(0.04, 0.15, 0.001)
)

forest_ready <- as_forest_data(
  data = raw_coefs,
  term = "variable",
  estimate = "beta",
  conf.low = "lower",
  conf.high = "upper",
  label = "display",
  grouping = "section",
  n = "sample_size",
  p.value = "p_value"
)
```

Once the data are standardized, you can pass them straight into `ggforestplot()`.

```{r helper-to-plot}
ggforestplot(forest_ready)
```

## Use `tidy_forest_model()` for model objects

If `broom` is available, `tidy_forest_model()` can pull coefficient estimates
and confidence limits from a fitted model.

```{r tidy-model}
fit <- lm(mpg ~ wt + hp + qsec, data = mtcars)

model_ready <- tidy_forest_model(fit)
```

The returned object can be passed directly into `ggforestplot()`.

```{r helper-to-plot-model}
ggforestplot(model_ready)
```

---
title: "Get Started with ggforestplotR"
output:
  rmarkdown::html_vignette:
    highlight: pygments
vignette: >
  %\VignetteIndexEntry{Get Started with ggforestplotR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 5
)
```

```{r setup}
library(ggforestplotR)
library(ggplot2)
```

`ggforestplotR` is built for coefficient-driven forest plots that stay inside a
normal `ggplot2` workflow.

## Choose a workflow

Use the package in one of three ways:

1. Start from a coefficient table and map the required columns directly.
2. Start from a fitted model and let `tidy_forest_model()` or `ggforestplot()` call `broom::tidy()`.
3. Add `add_forest_table()` or `add_split_table()` after styling the main plot.

This article covers the basic entry points and the minimum data you need.

## Start from a coefficient table

The simplest input is a data frame with a term, estimate, and confidence limits. If your columns use different names, map them explicitly.

```{r basic-plot}
basic_coefs <- data.frame(
  term = c("Age", "BMI", "Treatment"),
  estimate = c(0.10, -0.08, 0.34),
  conf.low = c(0.02, -0.16, 0.12),
  conf.high = c(0.18, 0.00, 0.56)
)

ggforestplot(basic_coefs)
```

## Add grouped sections and row striping

Use `grouping` when you want related variables separated into labeled panels.
Add `striped_rows = TRUE` to color alternating rows in the plot.

```{r grouped-striped}
sectioned_coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III", "Nodes"),
  estimate = c(0.10, -0.08, 0.20, 0.34, 0.52, 0.28),
  conf.low = c(0.02, -0.16, 0.05, 0.12, 0.20, 0.06),
  conf.high = c(0.18, 0.00, 0.35, 0.56, 0.84, 0.50),
  section = c("Clinical", "Clinical", "Clinical", "Tumor", "Tumor", "Tumor")
)

ggforestplot(
  sectioned_coefs,
  grouping = "section",
  striped_rows = TRUE,
  stripe_fill = "grey94"
)
```

## Add a summary table

Use `add_forest_table()` to add a summary table to your forest plot.

```{r side-table}
tabled_coefs <- data.frame(
  term = c("Age", "BMI", "Smoking", "Stage II", "Stage III"),
  estimate = c(0.12, -0.10, 0.18, 0.30, 0.46),
  conf.low = c(0.03, -0.18, 0.04, 0.10, 0.18),
  conf.high = c(0.21, -0.02, 0.32, 0.50, 0.74),
  sample_size = c(120, 115, 98, 87, 83)
)

ggforestplot(tabled_coefs, n = "sample_size", striped_rows = TRUE) +
  add_forest_table(
    position = "left",
    show_n = TRUE,
    estimate_label = "Beta"
  )
```

## Add split summary tables

Use `add_split_table()` to create a more traditional looking forest plot, with summary data on either side of the plot.

```{r split-table}
ggforestplot(tabled_coefs, n = "sample_size", striped_rows = T) +
  add_split_table(
    left_columns = c("term", "n"),
    right_columns = c("estimate")
  )
```

## Start from a fitted model

If `broom` is installed, `ggforestplot()` can work directly from a fitted model.

```{r model-plot}
fit <- lm(mpg ~ wt + hp + qsec, data = mtcars)

ggforestplot(fit, sort_terms = "descending")
```

## Next articles

For more detail, see:

- `ggforestplotR-plot-customization` for enhanced customization of the plots and summary tables.
- `ggforestplotR-data-helpers` for `as_forest_data()` and `tidy_forest_model()`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggforestplotR-package.R
\docType{package}
\name{ggforestplotR-package}
\alias{ggforestplotR}
\alias{ggforestplotR-package}
\title{ggforestplotR: Forest plots from model coefficients with ggplot2}
\description{
Transform model coefficients into flexible forest plots using
\code{ggplot2}. Provides helpers to standardize coefficient data
from a range of modelling workflows and render publication-ready
coefficient plots with a consistent interface.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://thatoneguy006.github.io/ggforestplotR/}
  \item \url{https://github.com/thatoneguy006/ggforestplotR}
  \item Report bugs at \url{https://github.com/thatoneguy006/ggforestplotR/issues}
}

}
\author{
\strong{Maintainer}: Carson Richardson \email{carson.richardson@outlook.com} [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggforestplot.R
\name{ggforestplot}
\alias{ggforestplot}
\title{Draw a ggplot2 forest plot}
\usage{
ggforestplot(
  data,
  term = "term",
  estimate = "estimate",
  conf.low = "conf.low",
  conf.high = "conf.high",
  label = term,
  term_labels = NULL,
  group = NULL,
  facet = NULL,
  facet_strip_position = c("left", "right"),
  grouping = NULL,
  grouping_strip_position = NULL,
  separate_groups = NULL,
  n = NULL,
  events = NULL,
  p.value = NULL,
  exponentiate = NULL,
  sort_terms = c("none", "descending", "ascending"),
  point_size = 2.3,
  point_shape = 19,
  linewidth = 0.5,
  line_size = NULL,
  staple_width = 0.2,
  ci_limits = NULL,
  ci_arrows = TRUE,
  ci_arrow_length = 0.08,
  ci_arrow_type = c("closed", "open"),
  dodge_width = 0.6,
  separate_lines = FALSE,
  separator_line_linetype = 2,
  separator_line_colour = "black",
  separator_line_size = 0.4,
  striped_rows = FALSE,
  stripe_fill = "grey95",
  stripe_colour = NA,
  stripe_alpha = 1,
  ref_line = NULL,
  ref_label = NULL,
  ref_linetype = 2,
  ref_color = "grey60"
)
}
\arguments{
\item{data}{Either a tidy coefficient data frame or a model object
supported by \code{\link[broom:tidy]{broom::tidy()}}.}

\item{term}{Column name holding the model term identifiers.}

\item{estimate}{Column name holding the point estimates.}

\item{conf.low}{Column name holding the lower confidence bounds.}

\item{conf.high}{Column name holding the upper confidence bounds.}

\item{label}{Optional column name used for the displayed row labels.}

\item{term_labels}{Optional named vector used to relabel displayed terms.
Names should match values in the term column and values are the labels to
display.}

\item{group}{Optional column name used for color-grouping estimates.}

\item{facet}{Optional column name used to split rows into faceted plot
sections. If this column is a factor, its levels control facet order.}

\item{facet_strip_position}{Positioning for facet strip labels.}

\item{grouping}{Deprecated. Use \code{facet} instead.}

\item{grouping_strip_position}{Deprecated. Use \code{facet_strip_position}
instead.}

\item{separate_groups}{Optional column name used to identify labeled
variable blocks that can be outlined with grid lines.}

\item{n}{Optional column name holding sample sizes or other N labels for
table helpers.}

\item{events}{Optional column name holding event counts or event labels for
table helpers.}

\item{p.value}{Optional column name holding p-values.}

\item{exponentiate}{Logical; if \code{TRUE}, transform the estimates and draw the
axis on the log scale with the reference line at 1. For model objects,
\code{NULL} uses the canonical scale when it can be inferred, such as hazard
ratios for Cox models.}

\item{sort_terms}{How to sort rows: \code{"none"}, \code{"descending"}, or
\code{"ascending"}.}

\item{point_size}{Point size for coefficient markers.}

\item{point_shape}{Shape used for coefficient markers.}

\item{linewidth}{Line width for confidence intervals.}

\item{line_size}{Deprecated. Use \code{linewidth} instead.}

\item{staple_width}{Width of the terminal staples on confidence interval
lines.}

\item{ci_limits}{Optional numeric vector of length 2 used to truncate
displayed confidence intervals. Intervals extending beyond these limits
are clipped to the boundary and marked with arrows when \code{ci_arrows} is
\code{TRUE}.}

\item{ci_arrows}{Logical; if \code{TRUE}, draw outward-facing arrows for
confidence intervals truncated by \code{ci_limits}.}

\item{ci_arrow_length}{Length of CI truncation arrows in inches.}

\item{ci_arrow_type}{Arrowhead type for truncated confidence intervals.
Passed to \code{\link[grid:arrow]{grid::arrow()}}.}

\item{dodge_width}{Horizontal dodging used for grouped estimates.}

\item{separate_lines}{Logical; if \code{TRUE}, draw grid lines
around each labeled block identified by \code{separate_groups}.}

\item{separator_line_linetype}{Line type used for separator lines.}

\item{separator_line_colour}{Colour used for separator lines.}

\item{separator_line_size}{Line width used for separator lines.}

\item{striped_rows}{Logical; if \code{TRUE}, shade alternating rows.}

\item{stripe_fill}{Fill color used for shaded rows.}

\item{stripe_colour}{Border color for shaded rows.}

\item{stripe_alpha}{Transparency for shaded rows.}

\item{ref_line}{Numeric x-value where the reference line is drawn, or
\code{NULL} to hide it. When omitted, defaults to \code{0} for additive effects and
\code{1} for exponentiated effects.}

\item{ref_label}{Optional label drawn alongside the reference line.}

\item{ref_linetype}{Line type for the reference line.}

\item{ref_color}{Color for the reference line.}
}
\value{
A \code{ggplot} object. Use standard \code{ggplot2} functions such as
\code{\link[ggplot2:labs]{ggplot2::labs()}} for plot labels, and add composition helpers after
styling the main plot.
}
\description{
Builds a forest plot from standardized coefficient data or directly from a
fitted model.
}
\examples{
coefs <- data.frame(
  term = c("Age", "BMI", "Treatment"),
  estimate = c(0.10, -0.08, 0.34),
  conf.low = c(0.02, -0.16, 0.12),
  conf.high = c(0.18, 0.00, 0.56)
)

ggforestplot(coefs)

ggforestplot(coefs, striped_rows = TRUE, point_shape = 17)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_split_table.R
\name{add_split_table}
\alias{add_split_table}
\title{Add split tables around a forest plot}
\usage{
add_split_table(
  plot = NULL,
  show_terms = TRUE,
  show_n = NULL,
  show_events = NULL,
  show_estimate = TRUE,
  show_p = FALSE,
  left_columns = NULL,
  right_columns = NULL,
  term_header = "Term",
  n_header = "N",
  events_header = "Events",
  estimate_label = "Estimate",
  p_header = "P-value",
  digits = NULL,
  text_size = NULL,
  header_text_size = NULL,
  header_fontface = "bold",
  header_family = NULL,
  striped_rows = NULL,
  stripe_fill = NULL,
  stripe_colour = NULL,
  left_width = NULL,
  plot_width = NULL,
  right_width = NULL
)
}
\arguments{
\item{plot}{A plot created by \code{\link[=ggforestplot]{ggforestplot()}}. Leave as \code{NULL} to use
\code{+ add_split_table(...)} syntax.}

\item{show_terms}{Whether to include the term column in the default
left-side selection when \code{left_columns} is not supplied.}

\item{show_n}{Whether to include the \code{N} column in the default left-side
selection when \code{left_columns} is not supplied. Defaults to \code{TRUE} when
the underlying plot data include an \code{n} column.}

\item{show_events}{Whether to include the \code{Events} column in the default
left-side selection when \code{left_columns} is not supplied. Defaults to
\code{TRUE} when the underlying plot data include an \code{events} column.}

\item{show_estimate}{Whether to include the formatted estimate and
confidence interval column in the default right-side selection when
\code{right_columns} is not supplied.}

\item{show_p}{Whether to include the p-value column in the default
right-side selection when \code{right_columns} is not supplied.}

\item{left_columns}{Optional explicit columns to place on the left side of
the forest plot. Accepts names such as \code{"term"}, \code{"n"}, and \code{"events"},
or positions \code{1:5} corresponding to \code{term}, \code{n}, \code{events}, \code{estimate},
and \code{p}.}

\item{right_columns}{Optional explicit columns to place on the right side
of the forest plot. Accepts names such as \code{"estimate"} and \code{"p"}, or
positions \code{1:5} corresponding to \code{term}, \code{n}, \code{events}, \code{estimate},
and \code{p}.}

\item{term_header}{Header text for the term column.}

\item{n_header}{Header text for the \code{N} column.}

\item{events_header}{Header text for the \code{Events} column.}

\item{estimate_label}{Header label for the estimate column.}

\item{p_header}{Header text for the p-value column.}

\item{digits}{Number of digits used when formatting estimates and p-values.
Defaults to \code{2}.}

\item{text_size}{Text size for table contents. Defaults to \code{3.2}.}

\item{header_text_size}{Header text size for table column labels. Defaults
to \code{11}.}

\item{header_fontface}{Font face used for table column labels. Defaults to
\code{"bold"}.}

\item{header_family}{Optional font family used for table column labels.}

\item{striped_rows}{Whether to draw alternating row stripes behind the
split table layout. Defaults to the stripe setting used in
\code{\link[=ggforestplot]{ggforestplot()}}.}

\item{stripe_fill}{Fill colour used for striped rows. Defaults to the
stripe fill used in \code{\link[=ggforestplot]{ggforestplot()}}.}

\item{stripe_colour}{Outline colour for striped rows. Defaults to the
stripe outline used in \code{\link[=ggforestplot]{ggforestplot()}}.}

\item{left_width}{Optional width allocated to the left table block. By
default this is derived from the number of displayed left-side columns
relative to \code{plot_width}.}

\item{plot_width}{Optional width allocated to the forest plot panel.
Defaults to \code{2.5}.}

\item{right_width}{Optional width allocated to the right table block. By
default this is derived from the number of displayed right-side columns
relative to \code{plot_width}.}
}
\value{
A patchwork-composed plot containing a left table, the forest plot,
and a right table, or a ggplot add-on object when \code{plot = NULL}.
}
\description{
Compose split table blocks around a forest plot so that summary data appear
on both sides of the plotting panel.
}
\examples{
coefs <- data.frame(
  term = c("Age", "BMI", "Treatment"),
  estimate = c(0.3, -0.2, 0.4),
  conf.low = c(0.1, -0.4, 0.2),
  conf.high = c(0.5, 0.0, 0.6),
  sample_size = c(120, 115, 98),
  p_value = c(0.012, 0.031, 0.004)
)

p <- ggforestplot(coefs, n = "sample_size", p.value = "p_value")
add_split_table(
  p,
  left_columns = c("term", "n"),
  right_columns = c("estimate", "p"),
  estimate_label = "HR"
)

ggforestplot(coefs, n = "sample_size", p.value = "p_value") +
  add_split_table(
    left_columns = c(1, 2),
    right_columns = c(4, 5),
    estimate_label = "HR"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_forest_model.R
\name{tidy_forest_model}
\alias{tidy_forest_model}
\title{Tidy a model object for forest plotting}
\usage{
tidy_forest_model(
  model,
  conf.int = TRUE,
  conf.level = 0.95,
  exponentiate = FALSE,
  intercept = FALSE,
  sort_terms = c("none", "descending", "ascending")
)
}
\arguments{
\item{model}{A fitted model object supported by \code{\link[broom:reexports]{broom::tidy()}}.}

\item{conf.int}{Logical; if \code{TRUE}, request confidence intervals from
\code{\link[broom:reexports]{broom::tidy()}}.}

\item{conf.level}{Confidence level for intervals.}

\item{exponentiate}{Logical; passed through to \code{\link[broom:reexports]{broom::tidy()}}.}

\item{intercept}{Logical; if \code{FALSE}, drop the intercept term.}

\item{sort_terms}{How to sort rows: \code{"none"}, \code{"descending"}, or
\code{"ascending"}.}
}
\value{
A standardized coefficient data frame ready for \code{\link[=ggforestplot]{ggforestplot()}}.
}
\description{
Uses \code{\link[broom:reexports]{broom::tidy()}} to convert a fitted model into forest-plot data.
}
\examples{
if (requireNamespace("broom", quietly = TRUE)) {
  fit <- lm(mpg ~ wt + hp + qsec, data = mtcars)
  tidy_forest_model(fit)

  set.seed(123)
  logit_data <- data.frame(
    age = rnorm(250, mean = 62, sd = 8),
    bmi = rnorm(250, mean = 28, sd = 4),
    treatment = factor(rbinom(250, 1, 0.45), labels = c("Control", "Treatment"))
  )
  linpred <- -9 + 0.09 * logit_data$age + 0.11 * logit_data$bmi +
    0.9 * (logit_data$treatment == "Treatment")
  logit_data$event <- rbinom(250, 1, plogis(linpred))
  logit_fit <- glm(event ~ age + bmi + treatment, data = logit_data, family = binomial())

  tidy_forest_model(logit_fit, exponentiate = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_forest_data.R
\name{as_forest_data}
\alias{as_forest_data}
\title{Standardize coefficient data for forest plots}
\usage{
as_forest_data(
  data,
  term,
  estimate,
  conf.low,
  conf.high,
  label = term,
  group = NULL,
  grouping = NULL,
  separate_groups = NULL,
  n = NULL,
  events = NULL,
  p.value = NULL,
  exponentiate = FALSE,
  sort_terms = c("none", "descending", "ascending")
)
}
\arguments{
\item{data}{A data frame containing coefficient estimates and intervals.}

\item{term}{Column name holding the model term identifier.}

\item{estimate}{Column name holding the point estimate.}

\item{conf.low}{Column name holding the lower confidence bound.}

\item{conf.high}{Column name holding the upper confidence bound.}

\item{label}{Optional column name used for the displayed row label.}

\item{group}{Optional column name used for color-grouping multiple
estimates per row.}

\item{grouping}{Optional column name used to split rows into grouped plot
sections.}

\item{separate_groups}{Optional column name used to identify labeled
variable blocks that can be outlined with separator lines.}

\item{n}{Optional column name holding sample sizes or other N labels for
table helpers.}

\item{events}{Optional column name holding event counts or event labels for
table helpers.}

\item{p.value}{Optional column name holding p-values.}

\item{exponentiate}{Logical; if \code{TRUE}, require positive values for
estimates and intervals.}

\item{sort_terms}{How to sort rows: \code{"none"}, \code{"descending"}, or
\code{"ascending"}.}
}
\value{
A standardized data frame ready for \code{\link[=ggforestplot]{ggforestplot()}} and the table
composition helpers.
}
\description{
Standardizes a coefficient table into the internal forest-plot data
structure used throughout \code{ggforestplotR}.
}
\examples{
raw <- data.frame(
  variable = c("Age", "BMI", "Treatment"),
  beta = c(0.10, -0.08, 0.34),
  lower = c(0.02, -0.16, 0.12),
  upper = c(0.18, 0.00, 0.56)
)

as_forest_data(
  data = raw,
  term = "variable",
  estimate = "beta",
  conf.low = "lower",
  conf.high = "upper"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_forest_table.R
\name{add_forest_table}
\alias{add_forest_table}
\title{Add a summary table to a forest plot}
\usage{
add_forest_table(
  plot = NULL,
  position = c("left", "right"),
  columns = NULL,
  term_header = "Term",
  n_header = "N",
  events_header = "Events",
  estimate_label = NULL,
  p_header = "P-value",
  column_labels = NULL,
  digits = NULL,
  estimate_digits = NULL,
  interval_digits = NULL,
  p_digits = NULL,
  estimate_fmt = NULL,
  ci_fmt = NULL,
  text_size = NULL,
  header_text_size = NULL,
  header_fontface = "bold",
  header_family = NULL,
  striped_rows = NULL,
  stripe_fill = NULL,
  stripe_colour = NULL,
  stripe_alpha = NULL,
  grid_lines = FALSE,
  grid_line_colour = "black",
  grid_line_size = 0.3,
  grid_line_linetype = 1
)
}
\arguments{
\item{plot}{A plot created by \code{\link[=ggforestplot]{ggforestplot()}}. Leave as \code{NULL} to use
\code{+ add_forest_table(...)} syntax.}

\item{position}{Whether to place the table on the left or right of the
forest plot.}

\item{columns}{Optional explicit columns to display in the side table, in
the order they should appear. Accepts built-in names such as \code{"term"},
\code{"n"}, \code{"events"}, \code{"estimate"}, \code{"ci"}, and \code{"p"}, arbitrary original
dataframe columns, or numeric positions in the supplied data. \code{"conf.low"}
and \code{"conf.high"} are accepted as aliases for \code{"ci"}.}

\item{term_header}{Header text for the term column.}

\item{n_header}{Header text for the \code{N} column.}

\item{events_header}{Header text for the \code{Events} column.}

\item{estimate_label}{Header label for the estimate column. Defaults to the
model-derived label when available.}

\item{p_header}{Header text for the p-value column.}

\item{column_labels}{Optional named vector used to relabel table column
headers. Names should match values supplied to \code{columns} after column
resolution, such as \code{"term"}, \code{"estimate"}, \code{"ci"}, \code{"p"}, or an arbitrary
original dataframe column.}

\item{digits}{Deprecated. Number of digits used when formatting estimates
and p-values. Defaults to \code{2}. Use \code{estimate_digits}, \code{interval_digits},
and \code{p_digits} for separate control.}

\item{estimate_digits}{Number of digits used for point estimates.}

\item{interval_digits}{Number of digits used for confidence interval bounds.}

\item{p_digits}{Number of digits used for p-values.}

\item{estimate_fmt}{Format string for the estimate column. Use
\code{{estimate}}, \code{{conf.low}}, and \code{{conf.high}} as placeholders. The
shorthand \verb{\{conf.low, conf.high\}} is also supported. Defaults to
\code{"{estimate} ({conf.low}, {conf.high})"}, or \code{"{estimate}"} when
\code{columns} includes \code{"ci"}.}

\item{ci_fmt}{Format string for the confidence interval column when
\code{columns} includes \code{"ci"}. Use \code{{conf.low}} and \code{{conf.high}} as
placeholders. The shorthand \verb{\{conf.low, conf.high\}} is also supported.
Defaults to \code{"({conf.low}, {conf.high})"}.}

\item{text_size}{Text size for table contents. Defaults to \code{3.2}.}

\item{header_text_size}{Header text size for table column labels. Defaults
to \code{11}.}

\item{header_fontface}{Font face used for table column labels. Defaults to
\code{"bold"}.}

\item{header_family}{Optional font family used for table column labels.}

\item{striped_rows}{Whether to draw alternating row stripes behind the
table. Defaults to the stripe setting used in \code{\link[=ggforestplot]{ggforestplot()}}.}

\item{stripe_fill}{Fill colour used for striped rows. Defaults to the
stripe fill used in \code{\link[=ggforestplot]{ggforestplot()}}.}

\item{stripe_colour}{Outline colour for striped rows. Defaults to the
stripe outline used in \code{\link[=ggforestplot]{ggforestplot()}}.}

\item{stripe_alpha}{Transparency for striped rows. Defaults to the stripe
alpha used in \code{\link[=ggforestplot]{ggforestplot()}}.}

\item{grid_lines}{Whether to draw black horizontal grid lines in the table.}

\item{grid_line_colour}{Colour used for the table grid lines.}

\item{grid_line_size}{Line width used for the table grid lines.}

\item{grid_line_linetype}{Line type used for the table grid lines.}
}
\value{
A patchwork-composed plot containing the forest plot and side
table, or a ggplot add-on object when \code{plot = NULL}.
}
\description{
Compose a summary table onto a forest plot.
}
\examples{
coefs <- data.frame(
  term = c("Age", "BMI", "Treatment"),
  estimate = c(0.3, -0.2, 0.4),
  conf.low = c(0.1, -0.4, 0.2),
  conf.high = c(0.5, 0.0, 0.6),
  sample_size = c(120, 115, 98),
  p_value = c(0.012, 0.031, 0.004)
)

p <- ggforestplot(coefs, n = "sample_size", p.value = "p_value")
add_forest_table(
  p,
  position = "left",
  columns = c("term", "n", "estimate", "p"),
  estimate_label = "Beta"
)

ggforestplot(coefs, n = "sample_size", p.value = "p_value") +
  add_forest_table(
    position = "right",
    columns = c("term", "n", "estimate", "p"),
    estimate_label = "Beta"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bind_forest_models.R
\name{bind_forest_models}
\alias{bind_forest_models}
\title{Bind multiple model summaries for a grouped forest plot}
\usage{
bind_forest_models(models, model_labels = NULL, exponentiate = NULL, ...)
}
\arguments{
\item{models}{A non-empty list of fitted model objects supported by
\code{\link[=tidy_forest_model]{tidy_forest_model()}}.}

\item{model_labels}{Optional labels used to identify each model in the
forest plot. Defaults to list names when present, otherwise \code{"Model 1"},
\code{"Model 2"}, and so on.}

\item{exponentiate}{\code{NULL}, a single logical value, or one logical value per
model. \code{NULL} uses the canonical scale inferred by \code{\link[=tidy_forest_model]{tidy_forest_model()}}
for each model.}

\item{...}{Additional arguments passed to \code{\link[=tidy_forest_model]{tidy_forest_model()}}, such as
\code{conf.level}, \code{intercept}, \code{term_labels}, or \code{sort_terms}.}
}
\value{
A standardized forest-plot data frame with one row per model term
and a \code{group} column containing the model labels.
}
\description{
Tidies multiple fitted models and stacks their fixed-effect coefficient
tables into a single forest-plot data frame. The resulting data can be
passed directly to \code{\link[=ggforestplot]{ggforestplot()}}, where model labels are used as the
grouping variable for dodged, color-coded estimates.
}
\examples{
if (requireNamespace("broom", quietly = TRUE)) {
  fit1 <- lm(mpg ~ wt + hp, data = mtcars)
  fit2 <- lm(mpg ~ wt + qsec, data = mtcars)

  bound <- bind_forest_models(
    list(Base = fit1, Adjusted = fit2)
  )

  ggforestplot(bound)
}
}
